---------------------------------------------------------------------------------
-- Alumne: Jessica Villamor 
-- Consultor: Juan Martnez Bolaos
-- Projecte: GESTIO DE VIDEOJOCS
---------------------------------------------------------------------------------

--
-- Ens situem a l'esquema de la base de dades de Gesti de Videojocs
--
ALTER SESSION SET CURRENT_SCHEMA = PFCBD;


-- Creaci de Taules de la base de dades Operacional

CREATE TABLE JOCS (
  ID NUMBER NOT NULL,
  Nom VARCHAR(255) NOT NULL,
  Vides NUMBER DEFAULT 1 NOT NULL ,
  CONSTRAINT PK_JOCS PRIMARY KEY(ID),
  CONSTRAINT U01_JOCS_NOM UNIQUE (NOM)
);
  
CREATE TABLE NIVELLS (
  ID NUMBER NOT NULL,
  NIVSEG_ID NUMBER NULL,
  JOCS_ID NUMBER NOT NULL,
  Nom VARCHAR(255) NOT NULL,
  CONSTRAINT PK_NIVELLS PRIMARY KEY(ID),
  CONSTRAINT FK2_NIVELLS_JOCS FOREIGN KEY (JOCS_ID) REFERENCES JOCS(ID),
  CONSTRAINT FK1_NIVELLS_NIVELLS FOREIGN KEY (NIVSEG_ID) REFERENCES NIVELLS(ID),
  CONSTRAINT U01_NIVELLS_JOCS_NOM UNIQUE (JOCS_ID, NOM)
);

CREATE TABLE PLATAFORMES (
  ID NUMBER NOT NULL,
  Nom VARCHAR(255) NOT NULL,
  CONSTRAINT PK_PLATAFORMES PRIMARY KEY(ID),
  CONSTRAINT U01_PLATAFORMES_NOM UNIQUE (NOM)
);

CREATE TABLE PLATAFORMES_JOC (
  PLATAFORMES_ID NUMBER NOT NULL,
  JOCS_ID NUMBER NOT NULL,
  CONSTRAINT PK_PLATAFORMES_JOC PRIMARY KEY(PLATAFORMES_ID, JOCS_ID),
  CONSTRAINT FK1_PLAJOC_PLATAFORMES FOREIGN KEY (PLATAFORMES_ID) REFERENCES PLATAFORMES(ID),
  CONSTRAINT FK2_PLAJOC_JOCS FOREIGN KEY(JOCS_ID) REFERENCES JOCS(ID)
);

CREATE TABLE NAVEGADORS (
  ID NUMBER NOT NULL,
  Nom VARCHAR(255) NOT NULL,
  CONSTRAINT PK_NAVEGADORS PRIMARY KEY(ID),
  CONSTRAINT U01_NAVEGADORS_NOM UNIQUE (NOM)
);

CREATE TABLE JOC_NAVEGADORS (
  JOCS_ID NUMBER NOT NULL,
  NAVEGADORS_ID NUMBER NOT NULL,
  CONSTRAINT PK_JOC_NAVEGADORS PRIMARY KEY(JOCS_ID, NAVEGADORS_ID),
  CONSTRAINT FK1_JOCNAV_JOCS FOREIGN KEY (JOCS_ID) REFERENCES JOCS(ID),
  CONSTRAINT FK2_JOCNAV_NAVEGADORS FOREIGN KEY (NAVEGADORS_ID) REFERENCES NAVEGADORS
);

CREATE TABLE PRODUCTES (
  ID NUMBER NOT NULL,
  JOCS_ID NUMBER NOT NULL,
  Nom VARCHAR(255) NOT NULL,
  Tipologia VARCHAR(2) NOT NULL,
  Preu FLOAT DEFAULT '0,0' NOT NULL ,
  DataBaixa DATE NULL,
  CONSTRAINT PK_PRODUCTES PRIMARY KEY(ID),
  CONSTRAINT FK1_PRODUCTES_JOCS FOREIGN KEY(JOCS_ID) REFERENCES JOCS(ID),
  CONSTRAINT CK_PRODUCTES_TIPOLOGIA CHECK (Tipologia IN ('AJ', 'VI'))
);

CREATE TABLE XARXES_SOCIALS (
  ID NUMBER NOT NULL,
  Nom VARCHAR(255) not NULL,
  CONSTRAINT PK_XARXES_SOCIALS PRIMARY KEY(ID),
  CONSTRAINT U01_XARSOC_NOM UNIQUE (NOM)
);

CREATE TABLE PAISOS (
  CODPAIS VARCHAR(2) NOT NULL,
  Nom VARCHAR(255) NOT NULL,
  CONSTRAINT PK_PAISOS PRIMARY KEY(CODPAIS)
);

CREATE TABLE PROVINCIES (
  CODPROV NUMBER NOT NULL,
  CODPAIS VARCHAR(2) NOT NULL,
  Nom VARCHAR(255) NOT NULL,
  CONSTRAINT PK_PROVINCIES PRIMARY KEY(CODPROV, CODPAIS),
  CONSTRAINT FK1_PROVINCIES_PAISOS FOREIGN KEY(CODPAIS) REFERENCES PAISOS(CODPAIS)
);

CREATE TABLE POBLACIONS (
  CODPAIS VARCHAR(2) NOT NULL, 
  CODPROV NUMBER NOT NULL,
  CODPOB NUMBER NOT NULL,
  Nom VARCHAR(255) NOT NULL,
  CONSTRAINT PK_POBLACIONS PRIMARY KEY(CODPOB, CODPAIS, CODPROV),
  CONSTRAINT FK1_POBLACIONS_PROVINCIES FOREIGN KEY(CODPROV, CODPAIS) REFERENCES PROVINCIES(CODPROV, CODPAIS)
);

CREATE TABLE ESTATS (
  ID NUMBER NOT NULL,
  ESTANT_ID NUMBER NULL,
  Nom VARCHAR(10) NOT NULL,
  Descriptiu VARCHAR(25) NOT NULL,
  CONSTRAINT PK_ESTATS PRIMARY KEY(ID),
  CONSTRAINT FK1_ESTATS_ESTATS FOREIGN KEY(ESTANT_ID) REFERENCES ESTATS(ID),
  CONSTRAINT U01_ESTATS_NOM UNIQUE (NOM)
);

CREATE TABLE JUGADORS (
  ID NUMBER NOT NULL,
  CODPROV NUMBER NULL,
  CODPAIS VARCHAR(2) NOT NULL,
  CODPOB NUMBER NULL,
  DataAlta DATE NOT NULL,
  NomUsuari VARCHAR(30) NOT NULL,
  Contrasenya VARCHAR(40) NOT NULL,
  Nom VARCHAR(255) NOT NULL,
  PrimerCog VARCHAR(255) NULL,
  SegonCog VARCHAR(255) NULL,
  Sexe VARCHAR(1) NOT NULL,
  Mail VARCHAR(255) NULL,
  CONSTRAINT PK_JUGADORS PRIMARY KEY(ID),
  CONSTRAINT FK1_JUGADORS_POBLACIONS FOREIGN KEY(CODPOB, CODPAIS, CODPROV) REFERENCES POBLACIONS(CODPOB, CODPAIS, CODPROV),
  CONSTRAINT FK2_JUGADORS_PROVINCIES FOREIGN KEY(CODPROV, CODPAIS) REFERENCES PROVINCIES(CODPROV, CODPAIS),
  CONSTRAINT FK3_JUGADORS_PAISOS FOREIGN KEY(CODPAIS) REFERENCES PAISOS(CODPAIS),
  CONSTRAINT U01_JUGADORS_NOMUSUARI UNIQUE (NomUsuari),
  CONSTRAINT CK_JUGADORS_SEXE CHECK (Sexe IN ('F', 'M'))
);

CREATE TABLE NIVELLS_JUGADORS (
  NIVELLS_ID NUMBER NOT NULL,
  JUGADORS_ID NUMBER NOT NULL,
  DataInici DATE NOT NULL,
  DataFi DATE NULL,
  Repte VARCHAR(1) DEFAULT 'N' NOT NULL,
  NIntents NUMBER DEFAULT '0' NOT NULL,
  CONSTRAINT PK_NIVELLS_JUGADORS PRIMARY KEY(NIVELLS_ID, JUGADORS_ID),
  CONSTRAINT FK1_NIVJUG_NIVELLS FOREIGN KEY(NIVELLS_ID) REFERENCES NIVELLS(ID),
  CONSTRAINT FK2_NIVJUG_JUGADORS FOREIGN KEY(JUGADORS_ID) REFERENCES JUGADORS(ID)
);

CREATE TABLE AJUDES_NIVELL (
  ID NUMBER NOT NULL,
  NIVJUG_JUGADORS_ID NUMBER NOT NULL,
  NIVJUG_NIVELLS_ID NUMBER NOT NULL,
  PRODUCTES_ID NUMBER NOT NULL,
  DataInici DATE NOT NULL,
  Activa VARCHAR(1) DEFAULT 'Y' NOT NULL,
  CONSTRAINT PK_AJUDES_NIVELL PRIMARY KEY(ID),
  CONSTRAINT FK1_AJUNIV_PRODUCTES FOREIGN KEY(PRODUCTES_ID) REFERENCES PRODUCTES(ID),
  CONSTRAINT FK2_AJUNIV_NIVJUG FOREIGN KEY(NIVJUG_NIVELLS_ID, NIVJUG_JUGADORS_ID) REFERENCES NIVELLS_JUGADORS(NIVELLS_ID, JUGADORS_ID)
);

CREATE TABLE JUGADORS_XARXES (
  ID NUMBER NOT NULL,
  XARXES_SOCIALS_ID NUMBER NOT NULL,
  JUGADORS_ID NUMBER NOT NULL,
  NomUsuari VARCHAR(50) NOT NULL,
  Contrasenya VARCHAR(50) NULL,
  DataAlta DATE NOT NULL,
  DataBaixa DATE NULL,
  CONSTRAINT PK_JUGADORS_XARXES PRIMARY KEY(ID),
  CONSTRAINT FK1_JUGXAR_XARXES_SOCIALS FOREIGN KEY(XARXES_SOCIALS_ID) REFERENCES XARXES_SOCIALS(ID),
  CONSTRAINT FK2_JUGXAR_JUGADORS FOREIGN KEY(JUGADORS_ID) REFERENCES JUGADORS(ID),
  CONSTRAINT U01_JUGXAR_NOMUSUARI UNIQUE (XARXES_SOCIALS_ID, JUGADORS_ID, NomUsuari)
);

CREATE TABLE COLLABORACIONS (
  ID NUMBER NOT NULL,
  JUGXAR_AMIC_ID NUMBER NOT NULL,
  JUGXAR_SOL_ID NUMBER NOT NULL,
  PRODUCTES_ID NUMBER NOT NULL,
  ESTATS_ID NUMBER NOT NULL,
  DataInici DATE NOT NULL,
  DataFi DATE NULL,
  CONSTRAINT PK_COLLABORACIONS PRIMARY KEY(ID),
  CONSTRAINT FK1_COLLAB_ESTATS FOREIGN KEY(ESTATS_ID) REFERENCES ESTATS(ID),
  CONSTRAINT FK2_COLLAB_PRODUCTES FOREIGN KEY(PRODUCTES_ID) REFERENCES PRODUCTES(ID),
  CONSTRAINT FK3_COLLAB_JUGXAR_SOLLICITANT FOREIGN KEY(JUGXAR_SOL_ID) REFERENCES JUGADORS_XARXES(ID),
  CONSTRAINT FK4_COLLAB_JUGXAR_AMIC FOREIGN KEY(JUGXAR_AMIC_ID) REFERENCES JUGADORS_XARXES(ID)
);


CREATE TABLE NAVEGADORS_JUGADORS (
  JUGADORS_ID NUMBER NOT NULL,
  NAVEGADORS_ID NUMBER NOT NULL,
  DarreraConnexio DATE NOT NULL,
  CONSTRAINT PK_NAVEGADORS_JUGADORS PRIMARY KEY(JUGADORS_ID, NAVEGADORS_ID),
  CONSTRAINT FK1_NAVJUG_JUGADORS FOREIGN KEY(JUGADORS_ID) REFERENCES JUGADORS(ID),
  CONSTRAINT FK2_NAVJUG_NAVEGADORS FOREIGN KEY(NAVEGADORS_ID) REFERENCES NAVEGADORS(ID)
);

CREATE TABLE COMPRES (
  ID NUMBER NOT NULL,
  JUGADORS_ID NUMBER NOT NULL,
  NIVELLS_ID NUMBER NOT NULL,
  PRODUCTES_ID NUMBER NOT NULL,
  DataCompra DATE NOT NULL,
  Preu FLOAT DEFAULT '0,0' NOT NULL,
  CONSTRAINT PK_COMPRES PRIMARY KEY(ID),
  CONSTRAINT FK1_COMPRES_PRODUCTES FOREIGN KEY(PRODUCTES_ID) REFERENCES PRODUCTES(ID),
  CONSTRAINT FK2_COMPRES_JUGADORS FOREIGN KEY(JUGADORS_ID) REFERENCES JUGADORS(ID),
  CONSTRAINT FK3_COMPRES_NIVELLS FOREIGN KEY(NIVELLS_ID) REFERENCES NIVELLS(ID)
);

CREATE TABLE PARTIDES (
  ID NUMBER NOT NULL,
  NIVJUG_JUGADORS_ID NUMBER NOT NULL,
  NIVJUG_NIVELLS_ID NUMBER NOT NULL,
  DataPartida DATE NOT NULL,
  VidesUtil NUMBER DEFAULT 0 NOT NULL,
  VidesExt NUMBER DEFAULT 0 NOT NULL,
  VidesCed NUMBER DEFAULT 0 NOT NULL,
  CONSTRAINT PK_PARTIDES PRIMARY KEY(ID),
  CONSTRAINT FK1_PARTIDES_NIVJUG FOREIGN KEY(NIVJUG_NIVELLS_ID, NIVJUG_JUGADORS_ID) REFERENCES NIVELLS_JUGADORS(NIVELLS_ID, JUGADORS_ID),
  CONSTRAINT U01_PARTIDES_NIVJUG UNIQUE (NIVJUG_JUGADORS_ID, NIVJUG_NIVELLS_ID, DataPartida)
);

CREATE TABLE PLATAFORMES_JUGADORS (
  PLATAFORMES_ID NUMBER NOT NULL,
  JUGADORS_ID NUMBER NOT NULL,
  DarreraConnexio DATE NOT NULL,
  CONSTRAINT PK_PLATAFORMES_JUGADORS PRIMARY KEY(PLATAFORMES_ID, JUGADORS_ID),
  CONSTRAINT FK1_PLAJUG_PLATAFORMES FOREIGN KEY (PLATAFORMES_ID) REFERENCES PLATAFORMES(ID),
  CONSTRAINT FK2_PLAJUG_JUGADORS FOREIGN KEY (JUGADORS_ID) REFERENCES JUGADORS(ID)
);

CREATE TABLE LOG (
  ID NUMBER NOT NULL,
  DataInici TIMESTAMP NOT NULL,
  DataFi TIMESTAMP NOT NULL,
  Procediment VARCHAR(50) NOT NULL,
  ParamEntrada VARCHAR(1000) NULL,
  ParamSortida VARCHAR(1000) NULL,
  Resultat VARCHAR(255) NOT NULL,
  CONSTRAINT PK_LOG PRIMARY KEY(ID)
);





