---------------------------------------------------------------------------------
-- Alumne: Jessica Villamor 
-- Consultor: Juan Martnez Bolaos
-- Projecte: GESTIO DE VIDEOJOCS
---------------------------------------------------------------------------------

--
-- Ens situem a l'esquema de la base de dades de Gesti de Videojocs
--
ALTER SESSION SET CURRENT_SCHEMA = PFCBD;


-- Creaci de la definici del paquet
create or replace
PACKAGE  GESTIO_JOCS  AS

			PROCEDURE REG_JUGADOR_NIVELLS(p_rs OUT	sys_refcursor,
										  p_jugadorId IN PFCBD.NIVELLS_JUGADORS.JUGADORS_ID%TYPE,
										  p_jocId IN PFCBD.NIVELLS.JOCS_ID%TYPE,
										  p_resultat OUT VARCHAR);
			PROCEDURE REG_PARTIDES_INICIAR(p_partidaid OUT PFCBD.PARTIDES.ID%TYPE, p_jugadorId IN PFCBD.PARTIDES.NIVJUG_JUGADORS_ID%TYPE,
								  p_nivellId IN PFCBD.PARTIDES.NIVJUG_NIVELLS_ID%TYPE,
								  p_navegadorId IN PFCBD.NAVEGADORS.ID%TYPE,
								  p_plataformaId IN PFCBD.PLATAFORMES.ID%TYPE,
								  p_data IN PFCBD.PARTIDES.DATAPARTIDA%TYPE,
								  p_resultat OUT VARCHAR);
			PROCEDURE CNS_JUGADOR_VIDES(p_numVides OUT NUMBER,
										p_jugadorId IN PFCBD.JUGADORS.ID%TYPE,
										p_jocId IN PFCBD.JOCS.ID%TYPE,
										p_data IN PFCBD.PARTIDES.DATAPARTIDA%TYPE,
										p_resultat OUT VARCHAR);
			PROCEDURE CNS_JUGADOR_AJUDES(p_rs OUT sys_refcursor,
										p_jugadorId IN PFCBD.JUGADORS.ID%TYPE,
										p_nivellId IN PFCBD.NIVELLS.ID%TYPE,
										p_resultat OUT VARCHAR);
			PROCEDURE UPD_PARTIDA_CNSVIDA(p_partidaId IN PFCBD.PARTIDES.ID%TYPE, p_resultat OUT VARCHAR);
			PROCEDURE UPD_PARTIDA_SUPREPTE(p_jugadorId IN PFCBD.JUGADORS.ID%TYPE,
											p_nivellId IN PFCBD.NIVELLS.ID%TYPE,
											p_resultat OUT VARCHAR);
			PROCEDURE REG_PLATAFORMES_JUGADORS(p_plataformaId IN PFCBD.PLATAFORMES_JUGADORS.PLATAFORMES_ID%TYPE,
								  p_jugadorId IN PFCBD.PLATAFORMES_JUGADORS.JUGADORS_ID%TYPE,
								  p_data IN PFCBD.PLATAFORMES_JUGADORS.DARRERACONNEXIO%TYPE,
								  p_resultat OUT VARCHAR);
			PROCEDURE REG_NAVEGADORS_JUGADORS(p_navegadorId IN PFCBD.NAVEGADORS_JUGADORS.NAVEGADORS_ID%TYPE,
								  p_jugadorId IN PFCBD.NAVEGADORS_JUGADORS.JUGADORS_ID%TYPE,
								  p_data IN PFCBD.NAVEGADORS_JUGADORS.DARRERACONNEXIO%TYPE,
								  p_resultat OUT VARCHAR);

END GESTIO_JOCS;
/


-- Creaci del cos del paquet
CREATE OR REPLACE PACKAGE BODY PFCBD.GESTIO_JOCS AS


---------------------------------------------------------------------------------
-- Procediments CU02: Gesti del Joc
---------------------------------------------------------------------------------

---------------------------------------------------------------------------------
-- CU02-01: Seleccionar Nivell
-- Parmetres d'entrada:
--		* p_jugadorId: Identificador del Jugador
--		* p_jocId: Identificador del Joc
-- Parmetres de sortida: 
--		* p_rs: Recordset amb la informaci
--		* p_resultat: Resultat del procediment OK/KO
--
-- Aquest procediment retornar un recordset amb els nivells disponibles per al jugador d'un joc donat.
-- Un jugador sempre podr jugar als nivells que ha superat el repte i al segent nivell, tots aquests nivells es trobaran donats d'alta a la taula NIVELLS_JUGADORS.
-- Si el jugador no ha jugat mai al joc es registrar el primer nivell del joc donat a la taula NIVELLS_JUGADORS, donat que es a l'unic nivell al que podr jugar, i es retornar aquest registre.
---------------------------------------------------------------------------------
PROCEDURE REG_JUGADOR_NIVELLS(p_rs OUT	sys_refcursor, 
							  p_jugadorId IN PFCBD.NIVELLS_JUGADORS.JUGADORS_ID%TYPE,
							  p_jocId IN PFCBD.NIVELLS.JOCS_ID%TYPE,
							  p_resultat OUT VARCHAR) 
IS
	v_num number;
	v_id PFCBD.JUGADORS.ID%TYPE;
	v_jugadorId PFCBD.NIVELLS_JUGADORS.JUGADORS_ID%TYPE;
	v_jocId PFCBD.NIVELLS_JUGADORS.JUGADORS_ID%TYPE;
	v_nivellId PFCBD.NIVELLS_JUGADORS.NIVELLS_ID%TYPE;
	
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
	
BEGIN
  
	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.REG_JUGADOR_NIVELLS';
	v_paramEnt       := 'p_jugadorId = ' || p_jugadorId 
                      || ', p_jocId = ' || p_jocId;
  
	-- Parmetres d'entrada obligatoris
	if p_jugadorId is null OR p_jocId is null then
		RAISE_APPLICATION_ERROR(-20000,'');
	end if;
 
	-- Es verifica que el Jugador existeix en el sistema
	BEGIN
    IF p_jugadorId IS NOT NULL THEN
        SELECT ID into v_jugadorId
        FROM JUGADORS
        WHERE ID = p_jugadorId;
    END IF;
    EXCEPTION
        WHEN OTHERS THEN
             IF (SQLCODE = 100) THEN
                   -- Jugador no trobat
                   RAISE_APPLICATION_ERROR(-20001,'');
             END IF; 
	END;  
	
	-- Es verifica que el Joc existeix en el sistema
	BEGIN
		IF p_jocId IS NOT NULL THEN
			SELECT ID into v_jocId
			FROM JOCS
			WHERE ID = p_jocId;
		END IF;
		EXCEPTION
			WHEN OTHERS THEN
				 IF (SQLCODE = 100) THEN
					   -- Joc no trobat
					   RAISE_APPLICATION_ERROR(-20002,'');
				 END IF; 
	END;  

	-- Es comprova si el Jugador ja ha iniciat alguna vegada el joc.
      select count(*) into v_num
      from nivells n, nivells_jugadors nj 
      where n.id = nj.nivells_id
      and n.jocs_id = p_jocId
      and nj.jugadors_id = p_jugadorId;

      if v_num = 0 then
	--	El jugador no ha jugat mai al joc, per tant pot comenar amb el primer nivell s'inserir aquest nivell a la taula NIVELLS_JUGADORS
         BEGIN  
            SELECT id INTO v_nivellId
            FROM NIVELLS
            Where rownum = 1 AND jocs_id = p_jocId
            START WITH nom = (select NOM from NIVELLS WHERE id NOT IN (SELECT DISTINCT NIVSEG_ID FROM NIVELLS WHERE NIVSEG_ID is not null and jocs_id = p_jocId ) and jocs_id = p_jocId)
            CONNECT BY PRIOR nivseg_id = p_jocId; 
            EXCEPTION
                WHEN OTHERS THEN
                   IF (SQLCODE = 100) THEN
                       RAISE_APPLICATION_ERROR(-20003,'');
                    end if;
          END;
          
          INSERT INTO NIVELLS_JUGADORS (NIVELLS_ID, JUGADORS_ID, DATAINICI) VALUES (v_nivellId, p_jugadorId, SYSDATE);
      END IF;

	--	Es retornaran els nivells accesibles (tots els que figuren a la taula de NIVELLS_JUGADORS     
	   OPEN p_rs FOR
		  select n.jocs_id, n.id, n.NOM, n.nivseg_id
		  from nivells n, nivells_jugadors nj 
		  where n.id = nj.nivells_id
			and n.jocs_id = p_jocId
			and nj.jugadors_id = p_jugadorId; 
    
	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat || ', p_rs: Recordset amb els nivells';
	v_resultat:= '00: Nivell seleccionat correctament.';
      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);		
	
	EXCEPTION
        WHEN OTHERS THEN
				 CASE (SQLCODE) 
                    WHEN -20000 THEN
                         v_resultat := '01: Ha d''introduir els parmetres d''entrada obligatoris.';
                    WHEN -20001 THEN
                         v_resultat := '02: Jugador no trobat amb ID ' || p_jugadorId;
                    WHEN -20002 THEN
                         v_resultat := '02: Joc no trobat amb ID ' || p_jocId;
                    WHEN -20003 THEN
                         v_resultat := '02: No es pot seleccionar un nivell vlid.';
                    ELSE
                          v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
        END CASE;
	
	-- Es genera un recordset buit
        OPEN p_rs FOR
        SELECT * FROM DUAL WHERE DUMMY IS NULL;
	
	-- S'assignem els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat || ', p_rs: Recordset buit.';
        
	DBMS_OUTPUT.put_line (v_resultat);				
-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
	
END REG_JUGADOR_NIVELLS;
---------------------------------------------------------------------------------


---------------------------------------------------------------------------------
-- CU02-02: Iniciar Partida
-- Parmetres d'entrada:
--		* p_jugadorId: Identificador del Jugador
--		* p_nivellId: Identificador del Nivell
--		* p_navegadorId: Identificador del Navegador des d'on s'inicia la partida
--		* p_plataformaId: Identificador de la plataforma des d'on s'inicia la partida
--		* p_data: Data de la partida (si no es rep cap valor s'utilitzarà la data del dia)
-- Parmetres de sortida:
--		* p_partidaId: Identificador de la partida
--		* p_resultat: Resultat del procediment OK/KO
--
-- Aquest procediment crear la partida al sistema si es el primer cop que juga en aquell dia.
-- Far s del procediment CNS_JUGADOR_VIDES que ens retorna el numero de vides disponibles. Si no hi han vides disponibles no es pot iniciar la partida.
---------------------------------------------------------------------------------
PROCEDURE REG_PARTIDES_INICIAR(p_partidaid OUT PFCBD.PARTIDES.ID%TYPE, p_jugadorId IN PFCBD.PARTIDES.NIVJUG_JUGADORS_ID%TYPE,
                      p_nivellId IN PFCBD.PARTIDES.NIVJUG_NIVELLS_ID%TYPE, 
                      p_navegadorId IN PFCBD.NAVEGADORS.ID%TYPE, 
					  p_plataformaId IN PFCBD.PLATAFORMES.ID%TYPE, 
					  p_data IN PFCBD.PARTIDES.DATAPARTIDA%TYPE,
					  p_resultat OUT VARCHAR) 
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
  
    v_retorn PFCBD.LOG.RESULTAT%TYPE;
	v_id PFCBD.PARTIDES.ID%TYPE;
	v_jugadorId PFCBD.PARTIDES.NIVJUG_JUGADORS_ID%TYPE;
	v_navegadorId PFCBD.NAVEGADORS.ID%TYPE;
	v_plataformaId PFCBD.PLATAFORMES.ID%TYPE;
	v_jocid PFCBD.JOCS.ID%TYPE;
	v_vides number;
	
BEGIN
	
	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.REG_PARTIDES_INICIAR';
	v_paramEnt       := 'p_partidaid = ' || p_partidaid 
                      || ', p_nivellId = ' || p_nivellId
					  || ', p_navegadorId = ' || p_navegadorId
					  || ', p_plataformaId = ' || p_plataformaId
					  || ', p_data = ' || p_data;

					  
	-- Parmetres d'entrada obligatoris
	if p_jugadorId is null OR p_nivellId is null or (p_navegadorId is null and p_plataformaId is null) then
		RAISE_APPLICATION_ERROR(-20000,'');
	end if;
	
	-- Es verifica que existeix el registre a Nivells_Jugadors, assegurant aix que el jugador i el nivell son vlids
	BEGIN
    IF p_jugadorId IS NOT NULL AND p_nivellId IS NOT NULL THEN
        SELECT JUGADORS_ID into v_jugadorId
        FROM NIVELLS_JUGADORS
        WHERE JUGADORS_ID = p_jugadorId
			  AND NIVELLS_ID = p_nivellId;
    END IF;
    EXCEPTION
        WHEN OTHERS THEN
             IF (SQLCODE = 100) THEN
                   -- Registre NIVELL_JUGADORS no trobat
                   RAISE_APPLICATION_ERROR(-20001,'');
             END IF; 
	END;  


	-- Es verifica que existeix el navegador si esta informat
	BEGIN
    IF p_navegadorId IS NOT NULL THEN
        SELECT ID into v_navegadorId
        FROM NAVEGADORS
        WHERE ID = p_navegadorId;
    END IF;
    EXCEPTION
        WHEN OTHERS THEN
             IF (SQLCODE = 100) THEN
                   -- Registre Navegadors no trobat
                   RAISE_APPLICATION_ERROR(-20003,'');
             END IF; 
	END;  	

	-- Es verifica que existeix la plataforma, si esta informada
	BEGIN
    IF p_plataformaId IS NOT NULL THEN
        SELECT ID into v_plataformaId
        FROM PLATAFORMES
        WHERE ID = p_plataformaId;
    END IF;
    EXCEPTION
        WHEN OTHERS THEN
             IF (SQLCODE = 100) THEN
                   -- Registre Plataformes no trobat
                   RAISE_APPLICATION_ERROR(-20004,'');
             END IF; 
	END;  	
	
	-- Verifiquem si el jugador t vides disponibles
	SELECT JOCS_ID INTO v_jocid FROM NIVELLS WHERE ID = p_nivellId;
	CNS_JUGADOR_VIDES(v_vides,p_jugadorId,v_jocid,nvl(p_data,sysdate), v_retorn);
  
	if (v_retorn = 'KO') then
		RAISE_APPLICATION_ERROR(-20005,'');
	end if;
	
	if v_vides > 0 then
		-- Es comprova si s la primera partida del dia
		IF p_jugadorId IS NOT NULL AND p_nivellId IS NOT NULL THEN
			BEGIN
              SELECT ID into p_partidaid
              FROM PARTIDES
              WHERE NIVJUG_JUGADORS_ID = p_jugadorId
              AND NIVJUG_NIVELLS_ID = p_nivellId
              and trunc(datapartida) = (nvl(trunc(p_data),trunc(sysdate)));
              EXCEPTION
                  WHEN OTHERS THEN
                     IF (SQLCODE = 100) THEN
                        -- Registre partides no trobat, s'inserir el nou registre
                       insert into partides (NIVJUG_JUGADORS_ID, NIVJUG_NIVELLS_ID, DATAPARTIDA) values (p_jugadorId, p_nivellId, nvl(p_data,sysdate));              
                       SELECT ID into p_partidaid
                       FROM PARTIDES
                       WHERE NIVJUG_JUGADORS_ID = p_jugadorId
                       AND NIVJUG_NIVELLS_ID = p_nivellId
                       and datapartida = nvl(p_data,sysdate);
                      END IF; 
			END;
	    		
			IF p_navegadorId IS NOT NULL THEN
				REG_NAVEGADORS_JUGADORS(p_navegadorId,p_jugadorId, nvl(p_data,sysdate), v_retorn);
        if (v_retorn = 'KO') then
            RAISE_APPLICATION_ERROR(-20006,'');
        end if;
  
			END IF;
			
			IF p_plataformaId IS NOT NULL THEN
				REG_PLATAFORMES_JUGADORS(p_plataformaId,p_jugadorId, nvl(p_data,sysdate), v_retorn);
        if (v_retorn = 'KO') then
            RAISE_APPLICATION_ERROR(-20007,'');
        end if;
        
			END IF;
		
		END IF;

  ELSE
    RAISE_APPLICATION_ERROR(-20002,'');
  END IF;

  
	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat;
	v_resultat:= '00: Acci realitzada correctament';
      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	 
  
  EXCEPTION
        WHEN OTHERS THEN
				 CASE (SQLCODE) 
                    WHEN -20000 THEN
                         v_resultat := '01: Ha d''introduir els parmetres d''entrada obligatoris.';
                    WHEN -20001 THEN
                         v_resultat := '02: Registre NIVELL_JUGADORS no trobat Nivell ' || p_nivellId || ' Jugador ' || p_jugadorId;
                    WHEN -20002 THEN
                         v_resultat := '02: No hi han vides disponibles, no es pot iniciar la partida';
                    WHEN -20003 THEN
                         v_resultat := '02: Registre Navegadors no trobat Id ' || p_navegadorId;
                    WHEN -20004 THEN
                         v_resultat := '02: Registre Plataformes no trobat Id ' || p_plataformaId;
                    WHEN -20005 THEN
                         v_resultat := '03: Error en la crida del procediment emmagatzemat de consultar vides.';
                    WHEN -20006 THEN
                         v_resultat := '02: Error en la crida del procediment emmagatzemat de registrar Navegador.';
                    WHEN -20007 THEN
                         v_resultat := '02: Error en la crida del procediment emmagatzemat de registrar Plataforma.';
                    ELSE
                          v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
        END CASE;
   
   
   -- S'assignem els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat;
        
	DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	

END REG_PARTIDES_INICIAR;
---------------------------------------------------------------------------------


---------------------------------------------------------------------------------
-- CU02-03: Consulta Vides Disponibles
-- Parmetres d'entrada:
--		* p_jugadorId: Identificador del Jugador
--		* p_jocId: Identificador del Joc
--		* p_data: Data de consulta (Si no s'introdueix cap data es tindr en compte la data del dia)
-- Parmetres de sortida:
--		* p_numVides: Numero de vides disponibles
--		* p_resultat: Resultat del procediment OK/KO
---------------------------------------------------------------------------------
PROCEDURE CNS_JUGADOR_VIDES(p_numVides OUT NUMBER, 
							p_jugadorId IN PFCBD.JUGADORS.ID%TYPE,
							p_jocId IN PFCBD.JOCS.ID%TYPE,
							p_data IN PFCBD.PARTIDES.DATAPARTIDA%TYPE,
							p_resultat OUT VARCHAR) 
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
	
	v_numVides number;
	v_numGastades number;
	v_id PFCBD.JUGADORS.ID%TYPE;
	v_data PFCBD.PARTIDES.DATAPARTIDA%TYPE;
	v_jugadorId PFCBD.NIVELLS_JUGADORS.JUGADORS_ID%TYPE;
	v_jocId PFCBD.NIVELLS_JUGADORS.JUGADORS_ID%TYPE;
BEGIN
	
	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.CNS_JUGADOR_VIDES';
	v_paramEnt       := 'p_jugadorId = ' || p_jugadorId 
                      || ', p_jocId = ' || p_jocId
					  || ', p_data = ' || p_data;
	
	
  p_numVides := 0;
  if p_data is null then
    v_data := sysdate;
  else
    v_data := p_data;
  end if;
  
	-- Parmetres d'entrada obligatoris
	if p_jugadorId is null OR p_jocId is null then
		RAISE_APPLICATION_ERROR(-20000,'');
	end if;
	
	-- Es verifica que el Joc existeix en el sistema, i es consultes les vides per defecte.
	BEGIN
    IF p_jocId IS NOT NULL THEN
        SELECT VIDES into v_numVides
        FROM JOCS
        WHERE ID = p_jocId;
    END IF;
    EXCEPTION
        WHEN OTHERS THEN
             IF (SQLCODE = 100) THEN
                   -- Joc no trobat
                   RAISE_APPLICATION_ERROR(-20001,'');
             END IF; 
	END;  
	
	-- Es verifica que el Jugador existeix en el sistema
	BEGIN
    IF p_jugadorId IS NOT NULL THEN
        SELECT ID into v_jugadorId
        FROM JUGADORS
        WHERE ID = p_jugadorId;
    END IF;
    EXCEPTION
        WHEN OTHERS THEN
             IF (SQLCODE = 100) THEN
                   -- Jugador no trobat
                   RAISE_APPLICATION_ERROR(-20002,'');
             END IF; 
	END;  
	
	IF v_numVides > 0 THEN

		select nvl((sum(p.videsutil)+ sum(p.videsced)-sum(p.videsext)),0) as VidesUsades into v_numGastades
			from nivells n, nivells_jugadors nj, partides p
			where n.id = nj.nivells_id 
			and p.nivjug_jugadors_id = nj.jugadors_id
			and p.nivjug_nivells_id = nj.nivells_id
			and nj.jugadors_id = p_jugadorId
			and n.jocs_id = p_jocId
		and trunc(p.datapartida) = trunc(v_data);

		if (v_numVides > v_numGastades) then
			p_numVides:= v_numVides - v_numGastades;
		ELSE
			p_numVides := 0;
		END IF;
	ELSE
		p_numVides:= 0;
	END IF;

  	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat || ', p_numVides: ' || p_numVides;
	v_resultat:= '00: Consulta realitzada correctament';
      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	

  EXCEPTION
        WHEN OTHERS THEN
				 CASE (SQLCODE) 
                    WHEN -20000 THEN
                         v_resultat := '01: Ha d''introduir els parmetres d''entrada obligatoris.';
                    WHEN -20002 THEN
                         v_resultat := '02: Jugador no trobat amb ID ' || p_jugadorId;
                    WHEN -20001 THEN
                         v_resultat := '02: Joc no trobat amb ID ' || p_jocId;
                    ELSE
                          v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
        END CASE;
   
	-- S'assignem els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat || ', p_numVides: buit';
        
	DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
END CNS_JUGADOR_VIDES;
---------------------------------------------------------------------------------


---------------------------------------------------------------------------------
-- CU02-04: Consulta Ajudes Disponibles
-- Parmetres d'entrada:
--		* p_jugadorId: Identificador del Jugador
--		* p_nivellId: Identificador del Nivell
-- Parmetres de sortida:
--		* p_rs: Recordset amb les ajudes disponibles
-- 		* p_resultat: Resultat del procediment OK/KO
---------------------------------------------------------------------------------
PROCEDURE CNS_JUGADOR_AJUDES(p_rs OUT sys_refcursor, 
							p_jugadorId IN PFCBD.JUGADORS.ID%TYPE,
							p_nivellId IN PFCBD.NIVELLS.ID%TYPE, 
							p_resultat OUT VARCHAR) 
IS
	v_jugadorId PFCBD.NIVELLS_JUGADORS.JUGADORS_ID%TYPE;
	v_jocId PFCBD.NIVELLS_JUGADORS.JUGADORS_ID%TYPE;
	v_nivellId PFCBD.NIVELLS.ID%TYPE;
	
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
BEGIN
	
	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.CNS_JUGADOR_AJUDES';
	v_paramEnt       := 'p_jugadorId = ' || p_jugadorId 
                      || ', p_nivellId = ' || p_nivellId;	
  
	-- Parmetres d'entrada obligatoris
	if p_jugadorId is null OR p_nivellId is null then
		RAISE_APPLICATION_ERROR(-20000,'');
	end if;
	
	-- Es verifica que el Jugador existeix en el sistema
	BEGIN
    IF p_jugadorId IS NOT NULL THEN
        SELECT ID into v_jugadorId
        FROM JUGADORS
        WHERE ID = p_jugadorId;
    END IF;
    EXCEPTION
        WHEN OTHERS THEN
             IF (SQLCODE = 100) THEN
                   -- Jugador no trobat
                   RAISE_APPLICATION_ERROR(-20001,'');
             END IF; 
	END;  

	-- Es verifica que el Nivell existeix en el sistema
	BEGIN
    IF p_nivellId IS NOT NULL THEN
        SELECT ID, jocs_id into v_nivellId, v_jocId
        FROM NIVELLS
        WHERE ID = p_nivellId;
    END IF;
    EXCEPTION
        WHEN OTHERS THEN
             IF (SQLCODE = 100) THEN
                   -- Nivell no trobat
                   RAISE_APPLICATION_ERROR(-20002,'');
             END IF; 
	END;	

	-- Es realitza la consulta
	OPEN p_rs FOR
	SELECT an.id, an.nivjug_jugadors_id, an.nivjug_nivells_id, an.productes_id, p.nom, an.activa
	FROM PRODUCTES P,
     NIVELLS N,
     AJUDES_NIVELL AN,
     JOCS J
	WHERE J.ID = p.jocs_id
      AND J.ID = n.jocs_id
      AND j.id = v_jocId
      AND p.tipologia = 'AJ'
	  and an.productes_id = p.ID
      AND an.nivjug_nivells_id = n.id
      AND an.nivjug_jugadors_id = p_jugadorId
      AND N.ID = p_nivellId
      AND an.activa = 'Y'
      and trunc(an.datainici) <= trunc(sysdate);
  	
    -- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat || ', p_rs: Cursor amb la informaci.';
	v_resultat:= '00: Consulta realitzada correctament';
      
-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
	
  EXCEPTION
        WHEN OTHERS THEN
				 CASE (SQLCODE) 
                    WHEN -20000 THEN
                         v_resultat := '01: Ha d''introduir els parmetres d''entrada obligatoris.';
                    WHEN -20001 THEN
                         v_resultat := '02: Jugador no trobat amb ID ' || p_jugadorId;
                   	WHEN -20002 THEN
                         v_resultat := '02: Nivell no trobat amb ID ' || p_nivellId;
                    ELSE
                          v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
        END CASE;
    
	-- Es genera un recordset buit
        OPEN p_rs FOR
        SELECT * FROM DUAL WHERE DUMMY IS NULL;
				
	-- S'assignem els valors de sortida 
		p_resultat := 'KO';
		v_dataFi    := SYSDATE;
		v_paramSor	:= 'p_resultat: ' || p_resultat || ', p_rs: Cursor buit.';
        
		DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
		GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	

END CNS_JUGADOR_AJUDES;
---------------------------------------------------------------------------------


---------------------------------------------------------------------------------
-- CU02-05: Consumeix Vida
-- Parmetres d'entrada:
--		* p_partidaId: Identificador de la partida
-- Parmetres de sortida;
--		* p_resultat: Resultat del procediment OK/KO
---------------------------------------------------------------------------------
PROCEDURE UPD_PARTIDA_CNSVIDA(p_partidaId IN PFCBD.PARTIDES.ID%TYPE, p_resultat OUT VARCHAR) 
IS
	v_jugadorId PFCBD.NIVELLS_JUGADORS.JUGADORS_ID%TYPE;
	v_partidaId PFCBD.PARTIDES.ID%TYPE;
	v_nivellId PFCBD.NIVELLS.ID%TYPE;
	v_numVides NUMBER;
	v_numIntents NUMBER;
	v_repte PFCBD.NIVELLS_JUGADORS.REPTE%TYPE;
	
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
	

BEGIN
	
  -- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.UPD_PARTIDA_CNSVIDA';
	v_paramEnt       := 'p_partidaId = ' || p_partidaId;

					  
	-- Parmetres d'entrada obligatoris
	if p_partidaId is null then
		RAISE_APPLICATION_ERROR(-20000,'');
	end if;
	
	-- Es verifica que existeix la partida al sistema, i recuperem el numero de vides utilitzades
	BEGIN
    IF p_partidaId IS NOT NULL THEN
        SELECT NIVJUG_JUGADORS_ID, NIVJUG_NIVELLS_ID, VIDESUTIL into v_jugadorId, v_nivellId, v_numVides
        FROM PARTIDES
        WHERE ID = p_partidaId;
    END IF;
    EXCEPTION
        WHEN OTHERS THEN
             IF (SQLCODE = 100) THEN
                   -- Partida no trobada
                   RAISE_APPLICATION_ERROR(-20002,'');
             END IF; 
	END;  
  
	-- Es verifica que existeix el registre a Nivells_Jugadors, assegurant axi que el jugador i el nivell son vlids.
	-- Es recupera el numero d'intents
	BEGIN
    IF v_jugadorId IS NOT NULL AND v_nivellId IS NOT NULL THEN
        SELECT NINTENTS, REPTE into v_numIntents, v_repte
        FROM NIVELLS_JUGADORS
        WHERE JUGADORS_ID = v_jugadorId
			  AND NIVELLS_ID = v_nivellId;
    END IF;
    EXCEPTION
        WHEN OTHERS THEN
             IF (SQLCODE = 100) THEN
                   -- Registre NIVELL_JUGADORS no trobat
                   RAISE_APPLICATION_ERROR(-20001,'');
             END IF; 
	END;  

	v_numIntents := v_numIntents + 1;
	v_numVides := v_numVides + 1 ;
	
	UPDATE PARTIDES set videsutil = v_numVides where ID = p_partidaId;
	
	-- Noms s'incrementar el nmero d'intens si encara no s'ha superat el repte
	IF v_repte = 'N' THEN
		UPDATE NIVELLS_JUGADORS SET NINTENTS = v_numIntents WHERE NIVELLS_ID = v_nivellId AND JUGADORS_ID = v_jugadorId;
	END IF;
		
	
	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat;
	v_resultat:= '00: Acci realitzada correctament';
      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
	

  EXCEPTION
        WHEN OTHERS THEN
				 CASE (SQLCODE) 
                    WHEN -20000 THEN
                         v_resultat := '01: Ha d''introduir els parmetres d''entrada obligatoris.';
                    WHEN -20001 THEN
                         v_resultat := '02: Registre NIVELL_JUGADORS no trobat Nivell ' || v_nivellId || ' Jugador ' || v_jugadorId;
					WHEN -20002 THEN
                         v_resultat := '02: Partida no trobada amb ID ' || p_partidaId;
                    ELSE
                         v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
        END CASE;
    
	-- S'assignem els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat;
        
	DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
	   

END UPD_PARTIDA_CNSVIDA;
---------------------------------------------------------------------------------


---------------------------------------------------------------------------------
-- CU02-06: Supera Repte
-- Parmetres d'entrada:
--		* p_jugadorId: Identificador del Jugador
--		* p_nivellId: Identificador del Nivell
-- Parmetres de sortida;
--		* p_resultat: Resultat del procediment OK/KO
-- Aquest procediment marcar com superat el repte del nivell/jugador indicat si aquest no havia estat ja superat. Informar a ms la data fi del nivell per qestions estadistiques.
-- Si el repte ja havia estat superat no realitzar cap actualitzaci a base de dades.
---------------------------------------------------------------------------------
PROCEDURE UPD_PARTIDA_SUPREPTE(p_jugadorId IN PFCBD.JUGADORS.ID%TYPE,
								p_nivellId IN PFCBD.NIVELLS.ID%TYPE,
								p_resultat OUT VARCHAR) 
IS
	v_Repte PFCBD.NIVELLS_JUGADORS.REPTE%TYPE;
	v_reg number;
	v_segNivellId PFCBD.NIVELLS.ID%TYPE;
  
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
BEGIN
	
	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.UPD_PARTIDA_SUPREPTE';
	v_paramEnt       := 'p_jugadorId = ' || p_jugadorId 
                      || ', p_nivellId = ' || p_nivellId;

 	-- Parmetres d'entrada obligatoris
	if p_jugadorId is null OR p_nivellId is null then
		RAISE_APPLICATION_ERROR(-20000,'');
	end if;
	
	-- Es verifica que existeix el registre a Nivells_Jugadors, assegurant aix que el jugador i el nivell son vlids.
	-- Es recupera el numero d'intents
	BEGIN
    IF p_jugadorId IS NOT NULL AND p_nivellId IS NOT NULL THEN
        SELECT REPTE into v_Repte
        FROM NIVELLS_JUGADORS
        WHERE JUGADORS_ID = p_jugadorId
			  AND NIVELLS_ID = p_nivellId;
    END IF;
    EXCEPTION
        WHEN OTHERS THEN
             IF (SQLCODE = 100) THEN
                   -- Registre NIVELL_JUGADORS no trobat
                   RAISE_APPLICATION_ERROR(-20001,'');
             END IF; 
	END;  


	-- Verifiquem que s la primera vegada que supera el repte
	  if v_Repte = 'N' then
		  UPDATE NIVELLS_JUGADORS SET REPTE = 'Y',  datafi = sysdate WHERE NIVELLS_ID = p_nivellId AND JUGADORS_ID = p_jugadorId;
		
		  SELECT NIVSEG_ID  into v_segNivellId FROM NIVELLS WHERE ID = p_nivellId;
		
	-- Verifiquem si havia iniciat la partida en el nivell superior, per tal de crear el nou registre si no existeix.
		  if v_segNivellId is not null then
				select count(*) into v_reg
			  from NIVELLS_JUGADORS WHERE JUGADORS_ID = p_jugadorId
			  AND NIVELLS_ID = v_segNivellId;
						
			  if v_reg = 0 THEN
				insert into nivells_jugadors (nivells_id, jugadors_id, datainici) values (v_segNivellId, p_jugadorId, sysdate);
			  end if;
		  end if;
		
	  end if;

	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat;
	v_resultat:= '00: Acci realitzada correctament';
      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	  

  EXCEPTION
        WHEN OTHERS THEN
				 CASE (SQLCODE) 
                    WHEN -20000 THEN
                         v_resultat := '01: Ha d''introduir els parmetres d''entrada obligatoris.';
                    WHEN -20001 THEN
                         v_resultat := '02: Registre NIVELL_JUGADORS no trobat Nivell ' || p_nivellId || ' Jugador ' || p_jugadorId;
					ELSE
                          v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
        END CASE;
    
	-- S'assignem els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat;
        
	DBMS_OUTPUT.put_line (v_resultat);				
-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	

END UPD_PARTIDA_SUPREPTE;
---------------------------------------------------------------------------------  


---------------------------------------------------------------------------------  
-- CU02-07: Registrar Plataforma de la Partida
-- Parmetres d'entrada:
--		* p_plataformaId: Identificador de la Plataforma
--		* p_jugadorId: Identificador del Jugador
--		* p_data: Data de connexi (si no es rep cap valor s'utilitzar la data del dia)
-- Parmetres de sortida;
--		* p_resultat: Resultat del procediment OK/KO
---------------------------------------------------------------------------------  
PROCEDURE REG_PLATAFORMES_JUGADORS(p_plataformaId IN PFCBD.PLATAFORMES_JUGADORS.PLATAFORMES_ID%TYPE,
                      p_jugadorId IN PFCBD.PLATAFORMES_JUGADORS.JUGADORS_ID%TYPE, 
                      p_data IN PFCBD.PLATAFORMES_JUGADORS.DARRERACONNEXIO%TYPE, 
					  p_resultat OUT VARCHAR) 
IS
	v_jugadorId PFCBD.PLATAFORMES_JUGADORS.JUGADORS_ID%TYPE;
	v_plataformaId PFCBD.PLATAFORMES_JUGADORS.PLATAFORMES_ID%TYPE;
	v_data PFCBD.PLATAFORMES_JUGADORS.DARRERACONNEXIO%TYPE;
	v_darreraCon PFCBD.PLATAFORMES_JUGADORS.DARRERACONNEXIO%TYPE;
	
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;

BEGIN

	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.REG_PLATAFORMES_JUGADORS';
	v_paramEnt       := 'p_plataformaId = ' || p_plataformaId 
                      || ', p_jugadorId = ' || p_jugadorId
					  || ', p_data = ' || p_data;
  
	-- Parmetres d'entrada obligatoris
	if p_plataformaId is null OR p_jugadorId is null then
		RAISE_APPLICATION_ERROR(-20000,'');
	end if;
	
	-- Si no hi ha data, utilitzarem la del dia
	if p_data is null then
		v_darreraCon := sysdate;
	else
		v_darreraCon := p_data;
	end if;
	
	-- Es verifica que existeix el Jugador
	BEGIN
    IF p_jugadorId IS NOT NULL THEN
        SELECT ID into v_jugadorId
        FROM JUGADORS
        WHERE ID = p_jugadorId;
    END IF;
    EXCEPTION
        WHEN OTHERS THEN
             IF (SQLCODE = 100) THEN
                   -- Registre JUGADORS no trobat
                   RAISE_APPLICATION_ERROR(-20001,'');
             END IF; 
	END;  

	-- Es verifica que existeix la Plataforma
	BEGIN
    IF p_plataformaId IS NOT NULL THEN
        SELECT ID into V_plataformaId
        FROM PLATAFORMES
        WHERE ID = p_plataformaId;
    END IF;
    EXCEPTION
        WHEN OTHERS THEN
             IF (SQLCODE = 100) THEN
                   -- Registre PLATAFORMES no trobat
                   RAISE_APPLICATION_ERROR(-20002,'');
             END IF; 
	END;  

	
	-- Verifiquem si existeix el registre per actualitzar-lo o inserir-lo
    BEGIN
        SELECT DarreraConnexio into v_data
        FROM PLATAFORMES_JUGADORS
        WHERE PLATAFORMES_ID = p_plataformaId
        AND JUGADORS_ID = p_jugadorId;
              EXCEPTION
                  WHEN OTHERS THEN
                     IF (SQLCODE = 100) THEN
                        -- Registre no trobat, s el primer cop que el jugador juga des de la plataforma
                       insert into PLATAFORMES_JUGADORS (PLATAFORMES_ID, JUGADORS_ID, DARRERACONNEXIO) values (p_plataformaId, p_jugadorId, v_darreraCon);              
                     END IF; 
		
	END;

	IF (v_data is not null) and (trunc(v_data) < trunc(v_darreraCon)) then
			update PLATAFORMES_JUGADORS set DARRERACONNEXIO = v_darreraCon WHERE PLATAFORMES_ID = p_plataformaId and JUGADORS_ID = p_jugadorId;
	end if;
        
	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat;
	v_resultat:= '00: Acci realitzada correctament';
      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
	
  EXCEPTION
        WHEN OTHERS THEN
				 CASE (SQLCODE) 
                    WHEN -20000 THEN
                         v_resultat := '01: Ha d''introduir els parmetres d''entrada obligatoris.';
                    WHEN -20001 THEN
                         v_resultat := '02: Registre Jugadors no trobat Id ' || p_jugadorId;
                     WHEN -20002 THEN
                         v_resultat := '02: Registre Plataformes no trobat Id ' || p_plataformaId;
                    ELSE
                          v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
        END CASE;
    
	-- S'assignem els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat;
        
	DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	

END REG_PLATAFORMES_JUGADORS;
---------------------------------------------------------------------------------  


---------------------------------------------------------------------------------  
-- CU02-08: Registrar Navegador Partida
-- Parmetres d'entrada:
--		* p_navegadorId: Identificador del Navegador
--		* p_jugadorId: Identificador del Jugador
--		* p_data: Data de connexi (si no es rep cap valor s'utilitzar la data del dia)
-- Parmetres de sortida;
--		* p_resultat: Resultat del procediment OK/KO
---------------------------------------------------------------------------------  
PROCEDURE REG_NAVEGADORS_JUGADORS(p_navegadorId IN PFCBD.NAVEGADORS_JUGADORS.NAVEGADORS_ID%TYPE,
                      p_jugadorId IN PFCBD.NAVEGADORS_JUGADORS.JUGADORS_ID%TYPE, 
                      p_data IN PFCBD.NAVEGADORS_JUGADORS.DARRERACONNEXIO%TYPE,
					  p_resultat OUT VARCHAR) 
IS
	v_jugadorId PFCBD.NAVEGADORS_JUGADORS.JUGADORS_ID%TYPE;
	v_navegadorId PFCBD.NAVEGADORS_JUGADORS.NAVEGADORS_ID%TYPE;
	v_data PFCBD.NAVEGADORS_JUGADORS.DARRERACONNEXIO%TYPE;
	v_darreraCon PFCBD.NAVEGADORS_JUGADORS.DARRERACONNEXIO%TYPE;
	
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
	
BEGIN
	
	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.REG_NAVEGADORS_JUGADORS';
	v_paramEnt       := 'p_navegadorId = ' || p_navegadorId 
                      || ', p_jugadorId = ' || p_jugadorId
					  || ', p_data = ' || p_data;

	 
	-- Parmetres d'entrada obligatoris
	if p_navegadorId is null OR p_jugadorId is null then
		RAISE_APPLICATION_ERROR(-20000,'');
	end if;
	
	-- Si no hi ha data, utilitzarem la del dia
	if p_data is null then
		v_darreraCon := sysdate;
	else
		v_darreraCon := p_data;
  end if;
	
	-- Es verifica que existeix el Jugador
	BEGIN
    IF p_jugadorId IS NOT NULL THEN
        SELECT ID into v_jugadorId
        FROM JUGADORS
        WHERE ID = p_jugadorId;
    END IF;
    EXCEPTION
        WHEN OTHERS THEN
             IF (SQLCODE = 100) THEN
                   -- Registre JUGADORS no trobat
                   RAISE_APPLICATION_ERROR(-20001,'');
             END IF; 
	END;  

	-- Es verifica que existeix la Plataforma
	BEGIN
    IF p_navegadorId IS NOT NULL THEN
        SELECT ID into v_navegadorId
        FROM NAVEGADORS
        WHERE ID = p_navegadorId;
    END IF;
    EXCEPTION
        WHEN OTHERS THEN
             IF (SQLCODE = 100) THEN
                   -- Registre PLATAFORMES no trobat
                   RAISE_APPLICATION_ERROR(-20002,'');
             END IF; 
	END;  

	
	-- Verifiquem si existeix el registre per actualitzar-lo o inserir-lo
    BEGIN
        SELECT DarreraConnexio into v_data
        FROM NAVEGADORS_JUGADORS
        WHERE NAVEGADORS_ID = p_navegadorId
        AND JUGADORS_ID = p_jugadorId;
              EXCEPTION
                  WHEN OTHERS THEN
                     IF (SQLCODE = 100) THEN
                        -- Registre no trobat, s el primer cop que el jugador juga des del navegador
                       insert into NAVEGADORS_JUGADORS (NAVEGADORS_ID, JUGADORS_ID, DARRERACONNEXIO) values (p_navegadorId, p_jugadorId, v_darreraCon);              
                     END IF; 
		
	END;

	IF (v_data is not null) and (trunc(v_data) < trunc(v_darreraCon)) then
			update NAVEGADORS_JUGADORS set DARRERACONNEXIO = v_darreraCon WHERE NAVEGADORS_ID = p_navegadorId and JUGADORS_ID = p_jugadorId;
	end if;
        
	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat;
	v_resultat:= '00: Acci realitzada correctament';
      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);
	
		
  EXCEPTION
        WHEN OTHERS THEN
				 CASE (SQLCODE) 
                    WHEN -20000 THEN
                         v_resultat := '01: Ha d''introduir els parmetres d''entrada obligatoris.';
                    WHEN -20001 THEN
                         v_resultat := '02: Registre Jugadors no trobat Id ' || p_jugadorId;
                     WHEN -20002 THEN
                         v_resultat := '02: Registre Navegadors no trobat Id ' || p_navegadorId;
                    ELSE
                          v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
        END CASE;
   
	-- S'assignem els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat;
        
	DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	

END REG_NAVEGADORS_JUGADORS;
---------------------------------------------------------------------------------  



END GESTIO_JOCS;
/

