---------------------------------------------------------------------------------
-- Alumne: Jessica Villamor 
-- Consultor: Juan Martnez Bolaos
-- Projecte: GESTIO DE VIDEOJOCS
---------------------------------------------------------------------------------
--
-- Ens situem a l'esquema de la base de dades de Gesti de Videojocs
--
ALTER SESSION SET CURRENT_SCHEMA = PFCBD;

-- Creaci de la definici del paquet
create or replace
PACKAGE    PFCBD.GESTIO_COMPRES AS
		PROCEDURE INS_AJUDES_COMPRES ( p_jugadorId IN PFCBD.AJUDES_NIVELL.NIVJUG_JUGADORS_ID%TYPE,
									   p_nivellId IN PFCBD.AJUDES_NIVELL.NIVJUG_NIVELLS_ID%TYPE,
									   p_producteId IN PFCBD.AJUDES_NIVELL.PRODUCTES_ID%TYPE, 
									   p_resultat OUT VARCHAR);

		PROCEDURE INS_VIDES_COMPRES ( p_jugadorId IN PFCBD.AJUDES_NIVELL.NIVJUG_JUGADORS_ID%TYPE, 
									  p_nivellId IN PFCBD.AJUDES_NIVELL.NIVJUG_NIVELLS_ID%TYPE, 
									  p_producteId IN PFCBD.PRODUCTES.ID%TYPE, 
									  p_resultat OUT VARCHAR);
		PROCEDURE CNS_COMPRES ( p_rs IN OUT	sys_refcursor, 
								p_jocId IN PFCBD.JOCS.ID%TYPE,  
								p_jugadorId IN PFCBD.AJUDES_NIVELL.NIVJUG_JUGADORS_ID%TYPE, 
								p_resultat OUT VARCHAR);


END GESTIO_COMPRES;
/


-- Creaci del cos del paquet
CREATE OR REPLACE PACKAGE BODY PFCBD.GESTIO_COMPRES AS


---------------------------------------------------------------------------------
-- Procediments CU03: Gesti de Compres
---------------------------------------------------------------------------------

---------------------------------------------------------------------------------
-- CU03-01: Comprar Ajuda
-- Parmetres d'entrada:
--		* p_jugadorId: Identificador del Jugador
--		* p_nivellId: Identificador del Nivell
--		* p_producteId: Identificador del Producte
-- Parmetres de sortida: 
--		* p_resultat: Resultat del procediment OK/KO
---------------------------------------------------------------------------------
PROCEDURE INS_AJUDES_COMPRES ( p_jugadorId IN PFCBD.AJUDES_NIVELL.NIVJUG_JUGADORS_ID%TYPE,
							   p_nivellId IN PFCBD.AJUDES_NIVELL.NIVJUG_NIVELLS_ID%TYPE,
							   p_producteId IN PFCBD.AJUDES_NIVELL.PRODUCTES_ID%TYPE, 
							   p_resultat OUT VARCHAR) 
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
	
	v_jugadorId PFCBD.NIVELLS_JUGADORS.JUGADORS_ID%TYPE;
	v_tipologia PFCBD.PRODUCTES.TIPOLOGIA%TYPE;
	v_preu PFCBD.PRODUCTES.PREU%TYPE;
BEGIN
  
  	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.INS_AJUDES_COMPRES';
	v_paramEnt       := 'p_jugadorId = ' || p_jugadorId 
                      || ', p_nivellId = ' || p_nivellId
					  || ', p_producteId = ' || p_producteId;
  
  
	-- Parmetres d'entrada obligatoris
	if p_jugadorId is null OR p_nivellId is null OR p_producteId is null then
		RAISE_APPLICATION_ERROR(-20000,'');
	end if;
 
	-- Es verifica que existeix el registre a Nivells_Jugadors, assegurant aix que el jugador i el nivell son vlids.
	BEGIN
    IF p_jugadorId IS NOT NULL AND p_nivellId IS NOT NULL THEN
        SELECT JUGADORS_ID into v_jugadorId
        FROM NIVELLS_JUGADORS
        WHERE JUGADORS_ID = p_jugadorId
			  AND NIVELLS_ID = p_nivellId;
    END IF;
    EXCEPTION
        WHEN OTHERS THEN
             IF (SQLCODE = 100) THEN
                   -- Registre NIVELL_JUGADORS no trobat
                   RAISE_APPLICATION_ERROR(-20001,'');
             END IF; 
	END;  

	-- Es verifica que el Producte existeix en el sistema i es recupera el tipus de producte que s'est tractant
	BEGIN
    IF p_producteId IS NOT NULL THEN
        SELECT TIPOLOGIA, PREU into v_tipologia, v_preu
        FROM PRODUCTES
        WHERE ID = p_producteId AND (DATABAIXA IS NULL OR DATABAIXA > SYSDATE);
    END IF;
    EXCEPTION
        WHEN OTHERS THEN
             IF (SQLCODE = 100) THEN
                   -- Producte no disponible
                   RAISE_APPLICATION_ERROR(-20002,'');
             END IF; 
	END;  

	-- Es verifica que el producte s de tipus 'Ajuda'

	if v_tipologia <> 'AJ' then
		-- Es retornar un error si el tipus de producte indicat no s del tipus Ajuda.
		RAISE_APPLICATION_ERROR(-20003,'');
	end if;
	
	insert into ajudes_nivell (nivjug_jugadors_id, nivjug_nivells_id, productes_id, datainici) values (p_jugadorId, p_nivellId, p_producteId, sysdate);
    insert into compres (JUGADORS_ID, NIVELLS_ID, PRODUCTES_ID, DATACOMPRA, PREU) values (p_jugadorId, p_nivellId, p_producteId, sysdate, v_preu);
	
	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat;
	v_resultat:= '00: Ajuda inserida correctament';
      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
	   

  EXCEPTION
        WHEN OTHERS THEN
				 CASE (SQLCODE) 
                    WHEN -20000 THEN
                         v_resultat := '01: Ha d''introduir els parmetres d''entrada obligatoris.';
                    WHEN -20001 THEN
                         v_resultat := '02: Registre NIVELL_JUGADORS no trobat Nivell ' || p_nivellId || ' Jugador ' || p_jugadorId;
                    WHEN -20002 THEN
                         v_resultat := '02: Producte no disponible amb ID ' || p_producteId;
					WHEN -20003 THEN
                         v_resultat := '03: El producte ha de ser de tipus Ajuda';
                    ELSE
                          v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
        END CASE;
    
	-- S'assignem els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat;
        
	DBMS_OUTPUT.put_line (v_resultat);				
-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
	
END INS_AJUDES_COMPRES;
---------------------------------------------------------------------------------


---------------------------------------------------------------------------------
-- CU03-01: Comprar Vida
-- Parmetres d'entrada:
--		* p_jugadorId: Identificador del Jugador
--		* p_nivellId: Identificador del Nivell
--		* p_producteId: Identificador del Producte
-- Parmetres de sortida: 
--		* p_resultat: Resultat del procediment OK/KO
---------------------------------------------------------------------------------
PROCEDURE INS_VIDES_COMPRES ( p_jugadorId IN PFCBD.AJUDES_NIVELL.NIVJUG_JUGADORS_ID%TYPE, 
							  p_nivellId IN PFCBD.AJUDES_NIVELL.NIVJUG_NIVELLS_ID%TYPE, 
							  p_producteId IN PFCBD.PRODUCTES.ID%TYPE, 
							  p_resultat OUT VARCHAR) 
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
	
	v_jugadorId PFCBD.NIVELLS_JUGADORS.JUGADORS_ID%TYPE;
	v_tipologia PFCBD.PRODUCTES.TIPOLOGIA%TYPE;
	v_partidaId PFCBD.PARTIDES.ID%TYPE;
	v_vides number;
	v_preu PFCBD.PRODUCTES.PREU%TYPE;
BEGIN
  	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.INS_VIDES_COMPRES';
	v_paramEnt       := 'p_jugadorId = ' || p_jugadorId 
                      || ', p_nivellId = ' || p_nivellId
					  || ', p_producteId = ' || p_producteId;
    
	-- Parmetres d'entrada obligatoris
	if p_jugadorId is null OR p_nivellId is null OR p_producteId is null then
		RAISE_APPLICATION_ERROR(-20000,'');
	end if;
 
	-- Es verifica que existeix el registre a Nivells_Jugadors, assegurant aix que el jugador i el nivell son vlids.
	BEGIN
    IF p_jugadorId IS NOT NULL AND p_nivellId IS NOT NULL THEN
        SELECT JUGADORS_ID into v_jugadorId
        FROM NIVELLS_JUGADORS
        WHERE JUGADORS_ID = p_jugadorId
			  AND NIVELLS_ID = p_nivellId;
    END IF;
    EXCEPTION
        WHEN OTHERS THEN
             IF (SQLCODE = 100) THEN
                   -- Registre NIVELL_JUGADORS no trobat
                   RAISE_APPLICATION_ERROR(-20001,'');
             END IF; 
	END;  

	-- Es verifica que el Producte existeix en el sistema i es recupera el tipus de producte que s'esta tractant
	BEGIN
    IF p_producteId IS NOT NULL THEN
        SELECT TIPOLOGIA, PREU into v_tipologia, v_preu
        FROM PRODUCTES
        WHERE ID = p_producteId AND (DATABAIXA IS NULL OR DATABAIXA > SYSDATE);
    END IF;
    EXCEPTION
        WHEN OTHERS THEN
             IF (SQLCODE = 100) THEN
                   -- Producte no disponible
                   RAISE_APPLICATION_ERROR(-20002,'');
             END IF; 
	END;  

	-- Es verifica que el producte s de tipus 'Vida'

	if v_tipologia <> 'VI' then
	-- Es retornar un error si el tipus de producte indicat no s del tipus Vida.
		RAISE_APPLICATION_ERROR(-20003,'');
	end if;

	-- Es localitza la partida en la que incrementar la vida
	BEGIN
        SELECT ID, VIDESEXT into v_partidaId, v_vides
        FROM PARTIDES
        WHERE NIVJUG_JUGADORS_ID = p_jugadorId
			  and NIVJUG_NIVELLS_ID = p_nivellId
			  and trunc(datapartida) = trunc(sysdate)
        and rownum = 1;
    
		-- S'incrementa el numero de vides extres	
		v_vides := v_vides + 1;
		update partides set VIDESEXT = v_vides where id = v_partidaId;
	
	
    EXCEPTION
        WHEN OTHERS THEN
             IF (SQLCODE = 100) THEN
                   -- Partida no iniciada, crearem un registre de partida per comptabilitzar la nova vida.
				   -- L'aplicatiu est pensat per a que es comprin vides desprs d'haver iniciat la partida, per 
				   -- no es vol limitar a no permetre comprar un producte vida si no has iniciat partida.
              insert into partides ( NIVJUG_JUGADORS_ID, NIVJUG_NIVELLS_ID, DATAPARTIDA, VIDESEXT) values (p_jugadorId, p_nivellId, sysdate, 1);
             END IF; 
	END; 	
	
	-- Ja s'ha registrat la vida, es registrar tamb la compra
	insert into compres (JUGADORS_ID, NIVELLS_ID, PRODUCTES_ID, DATACOMPRA, PREU) values (p_jugadorId, p_nivellId, p_producteId, sysdate, v_preu);
	
	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat;
	v_resultat:= '00: Vida afegida correctament';
      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);

  EXCEPTION
        WHEN OTHERS THEN
				 CASE (SQLCODE) 
                    WHEN -20000 THEN
                         v_resultat := '01: Ha d''introduir els parmetres d''entrada obligatoris.';
                    WHEN -20001 THEN
                         v_resultat := '02: Registre NIVELL_JUGADORS no trobat Nivell ' || p_nivellId || ' Jugador ' || p_jugadorId;
                    WHEN -20002 THEN
                         v_resultat := '02: Producte no disponible amb ID ' || p_producteId;
					WHEN -20003 THEN
                         v_resultat := '03: El producte ha de ser de tipus Vida.';
				    ELSE
                          v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
        END CASE;
    
	-- S'assignem els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat;
        
	DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
	
END INS_VIDES_COMPRES;
---------------------------------------------------------------------------------



---------------------------------------------------------------------------------
-- CU03-03: Consultar Compres realitzades
-- Parmetres d'entrada:
--		* p_jocId: Identificador del Joc
--		* p_jugadorId: Identificador del Jugador
-- Parmetres de sortida: 
--		* p_rs: Recordset amb la informaci
--		* p_resultat: Resultat del procediment OK/KO
--
-- Procediment per recuperar les compres del sistema.
-- Recuperara les compres d'un jugador, si no s'introdueix cap criteri de selecci s'extreuran tots els registres de les compres d'un joc donat
---------------------------------------------------------------------------------
PROCEDURE CNS_COMPRES ( p_rs IN OUT	sys_refcursor, 
						p_jocId IN PFCBD.JOCS.ID%TYPE,  
						p_jugadorId IN PFCBD.AJUDES_NIVELL.NIVJUG_JUGADORS_ID%TYPE, 
						p_resultat OUT VARCHAR) 
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
	
	v_jugadorId PFCBD.NIVELLS_JUGADORS.JUGADORS_ID%TYPE;
	v_jocId PFCBD.JOCS.ID%TYPE;
BEGIN
 
	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.CNS_COMPRES';
	v_paramEnt       := 'p_jocId = ' || p_jocId 
                      || ', p_jugadorId = ' || p_jugadorId;

 
	-- Parmetres d'entrada obligatoris
	if p_jocId is null then
		RAISE_APPLICATION_ERROR(-20000,'');
	end if;


	-- Es verifica que el Jugador existeix en el sistema i es recupera el preu
	BEGIN
    IF p_jugadorId IS NOT NULL THEN
        SELECT ID into v_jugadorId
        FROM JUGADORS
        WHERE ID = p_jugadorId;
    END IF;
    EXCEPTION
        WHEN OTHERS THEN
             IF (SQLCODE = 100) THEN
                   -- Jugador inexistent
                   RAISE_APPLICATION_ERROR(-20001,'');
             END IF; 
	END;  

	-- Es verifica que el Joc existeix en el sistema
	BEGIN
    IF p_jocId IS NOT NULL THEN
        SELECT ID into v_jocId
        FROM JOCS
        WHERE ID = p_jocId;
    END IF;
    EXCEPTION
        WHEN OTHERS THEN
             IF (SQLCODE = 100) THEN
                   -- Joc inexistent
                   RAISE_APPLICATION_ERROR(-20002,'');
             END IF; 
	END; 	
	
	-- Es consulten les compres
	if p_jugadorId is not null then
		open p_rs for
			select c.*
			from compres c, productes p
			where p.id = c.productes_id
				and p.jocs_id = v_jocId
				and c.jugadors_id = p_jugadorId;
	else
		open p_rs for
			select c.*
			from compres c, productes p
			where p.id = c.productes_id
				and p.jocs_id = v_jocId;
	end if;
	
	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat || ', p_rs: Amb resultat de la consulta';
	v_resultat:= '00: Consulta registrada correctament';
      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	

  EXCEPTION
        WHEN OTHERS THEN
				 CASE (SQLCODE) 
                WHEN -20000 THEN
                       v_resultat := '01: Ha d''introduir els parmetres d''entrada obligatoris.';
                WHEN -20001 THEN
                       v_resultat := '02: Jugador no trobat amb ID ' || p_jugadorId;
                WHEN -20002 THEN
                       v_resultat := '02: Joc no trobat amb ID ' || p_jocId;
                ELSE
                       v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
        END CASE;
    
	-- Es genera un recordset buit
        OPEN p_rs FOR
            SELECT * FROM DUAL WHERE DUMMY IS NULL;
						
	-- S'assignem els valors de sortida 
		p_resultat := 'KO';
		v_dataFi    := SYSDATE;
		v_paramSor	:= 'p_resultat: ' || p_resultat || ', p_rs: Buit.';
        
	DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
	
END CNS_COMPRES;
---------------------------------------------------------------------------------


END GESTIO_COMPRES;
/

