---------------------------------------------------------------------------------
-- Alumne: Jessica Villamor 
-- Consultor: Juan Martnez Bolaos
-- Projecte: GESTIO DE VIDEOJOCS
---------------------------------------------------------------------------------
--
-- Ens situem a l'esquema de la base de dades de Gesti de Videojocs
--
ALTER SESSION SET CURRENT_SCHEMA = PFCBD;

-- Creaci de la definici del paquet
create or replace
PACKAGE    PFCBD.COLLABORACIO AS

			PROCEDURE CNS_COLLABORACIONS_JUGXAR ( p_rs OUT	sys_refcursor, 
										   p_xarxaId IN PFCBD.JUGADORS_XARXES.XARXES_SOCIALS_ID%TYPE,
										   p_jugadorId IN PFCBD.JUGADORS_XARXES.JUGADORS_ID%TYPE,
										   p_baixes IN VARCHAR,
										   p_resultat OUT VARCHAR);

			PROCEDURE INS_COLLABORACIONS_SOL ( p_perfilSolId IN PFCBD.COLLABORACIONS.JUGXAR_SOL_ID%TYPE,
								   p_perfilAmiId IN PFCBD.COLLABORACIONS.JUGXAR_AMIC_ID%TYPE,
								   p_producteId IN PFCBD.PRODUCTES.ID%TYPE,
								   p_resultat OUT VARCHAR);
								   
			PROCEDURE UPD_COLLABORACIONS_ACE (p_collaboracioId IN PFCBD.COLLABORACIONS.ID%TYPE, p_resultat OUT VARCHAR);
								   
			PROCEDURE UPD_COLLABORACIONS_REF (p_collaboracioId IN PFCBD.COLLABORACIONS.ID%TYPE, p_resultat OUT VARCHAR);
								   
			PROCEDURE CNS_COLLABORACIONS_LIST (p_rs IN OUT sys_refcursor, 
                                   p_jocId IN PFCBD.JOCS.ID%TYPE,
                                   p_collaboracioId IN PFCBD.COLLABORACIONS.ID%TYPE, 
                                   p_perfilsolId IN PFCBD.JUGADORS_XARXES.ID%TYPE, 
                                   p_actives IN varchar,
								   p_resultat OUT VARCHAR);
								   
			PROCEDURE UPD_COLLABORACIONS_CAN (p_collaboracioId IN PFCBD.COLLABORACIONS.ID%TYPE, p_resultat OUT VARCHAR);

END COLLABORACIO;
/


-- Creaci del cos del paquet
CREATE OR REPLACE PACKAGE BODY PFCBD.COLLABORACIO AS

---------------------------------------------------------------------------------
-- Procediments CU04: Collaboraci entre Jugadors
---------------------------------------------------------------------------------

---------------------------------------------------------------------------------
-- CU04-01: Consultar Llistat de Jugadors d'una xarxa social
-- Parmetres d'entrada:
--		* p_xarxaId: Identificador de la Xarxa Social
--		* p_jugadorId: Identificador del Jugador
--		* p_baixes: Indicador per extreure tamb els perfils donats de baixa, si aquest camp esta informat s'extreuran tots els perfils amb independcia de si estan donats de baixa.
-- Parmetres de sortida: 
--		* p_rs: Recordset amb la informaci
--		* p_resultat: Resultat del procediment OK/KO
--
-- Procediment per extreure tots els jugadors que tenen perfil de la mateixa xarxa social per sollicitar collaboraci.
-- No s'inclouran els perfils del propi jugador.
---------------------------------------------------------------------------------
PROCEDURE CNS_COLLABORACIONS_JUGXAR ( p_rs OUT	sys_refcursor, 
							   p_xarxaId IN PFCBD.JUGADORS_XARXES.XARXES_SOCIALS_ID%TYPE,
							   p_jugadorId IN PFCBD.JUGADORS_XARXES.JUGADORS_ID%TYPE,
							   p_baixes IN VARCHAR,
							   p_resultat OUT VARCHAR) 
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
	
	v_xarxaId PFCBD.JUGADORS_XARXES.XARXES_SOCIALS_ID%TYPE;
	v_jugadorId PFCBD.JUGADORS_XARXES.JUGADORS_ID%TYPE;
BEGIN

	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment     := 'CNS_COLLABORACIONS_JUGXAR';
	v_paramEnt       := 'p_xarxaId = ' || p_xarxaId 
                      || ', p_jugadorId = ' || p_jugadorId
					  || ', p_baixes = ' || p_baixes;


	-- Parmetres d'entrada obligatoris
	if p_xarxaId is null OR p_jugadorId IS NULL then
		RAISE_APPLICATION_ERROR(-20000,'');
	end if;
 
	-- Es verifica que la Xarxa Social existeix en el sistema
	BEGIN
    IF p_xarxaId IS NOT NULL THEN
        SELECT ID into v_xarxaId
        FROM XARXES_SOCIALS
        WHERE ID = p_xarxaId;
    END IF;
    EXCEPTION
        WHEN OTHERS THEN
             IF (SQLCODE = 100) THEN
                   -- Xarxa Social no trobada
                   RAISE_APPLICATION_ERROR(-20001,'');
             END IF; 
	END; 

	-- Es verifica que el Jugador existeix en el sistema
	BEGIN
    IF p_jugadorId IS NOT NULL THEN
        SELECT ID into v_jugadorId
        FROM JUGADORS
        WHERE ID = p_jugadorId;
    END IF;
    EXCEPTION
        WHEN OTHERS THEN
             IF (SQLCODE = 100) THEN
                   -- Jugador inexistent
                   RAISE_APPLICATION_ERROR(-20002,'');
             END IF; 
	END;  	

	-- Es realitza la consulta sobre la taula
	if p_baixes is not null then
		open p_rs for
		select * from jugadors_xarxes
		where XARXES_SOCIALS_ID = p_xarxaId
			  and JUGADORS_ID <> p_jugadorId;
	else
		open p_rs for
		select * from jugadors_xarxes
		where XARXES_SOCIALS_ID = p_xarxaId and JUGADORS_ID <> p_jugadorId AND (databaixa is null or databaixa > sysdate);
	end if;
	

	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat || ', p_rs: Recordset amb la informaci.';
	v_resultat:= '00: Consulta realitzada correctament';
      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
	 
  EXCEPTION
        WHEN OTHERS THEN
				 CASE (SQLCODE) 
                    WHEN -20000 THEN
                         v_resultat := '01: Ha d''introduir els parmetres d''entrada obligatoris.';
                    WHEN -20001 THEN
                         v_resultat := '02: Registre Xarxa Social no trobada ' || p_xarxaId;
					WHEN -20002 THEN
                         v_resultat := '02: Registre Jugador no trobat ' || p_jugadorId;                
					ELSE
                          v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
        END CASE;
   
   
    -- Es genera un recordset buit
         OPEN p_rs FOR
         SELECT * FROM DUAL WHERE DUMMY IS NULL;
				
	-- S'assignem els valors de sortida 
		p_resultat := 'KO';
		v_dataFi    := SYSDATE;
		v_paramSor	:= 'p_resultat: ' || p_resultat || ', p_rs: Recordset buit.';
        
		DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
		GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
	
END CNS_COLLABORACIONS_JUGXAR;
---------------------------------------------------------------------------------


---------------------------------------------------------------------------------
-- CU04-02: Sollicitar collaboraci
-- Parmetres d'entrada:
--		* p_perfilSolId: Identificador del perfil sollicitant
--		* p_perfilAmiId: Identificador del perfil amic
--		* p_producteId: Identificador del producte sol·licitat
-- Parmetres de sortida: 
--		* p_resultat: Resultat del procediment OK/KO
---------------------------------------------------------------------------------
PROCEDURE INS_COLLABORACIONS_SOL ( p_perfilSolId IN PFCBD.COLLABORACIONS.JUGXAR_SOL_ID%TYPE,
								   p_perfilAmiId IN PFCBD.COLLABORACIONS.JUGXAR_AMIC_ID%TYPE,
								   p_producteId IN PFCBD.PRODUCTES.ID%TYPE,
								   p_resultat OUT VARCHAR) 
IS
	v_XarxaSolId PFCBD.COLLABORACIONS.JUGXAR_SOL_ID%TYPE;
	v_XarxaAmiId PFCBD.COLLABORACIONS.JUGXAR_AMIC_ID%TYPE;
	v_producteId PFCBD.PRODUCTES.ID%TYPE;
	v_estatId PFCBD.ESTATS.ID%TYPE;
	
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
	
BEGIN
  	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.INS_COLLABORACIONS_SOL';
	v_paramEnt       := 'p_perfilSolId = ' || p_perfilSolId 
                      || ', p_perfilAmiId = ' || p_perfilAmiId
					  || ', p_producteId = ' || p_producteId;
  
	-- Parmetres d'entrada obligatoris
	if p_perfilSolId is null OR p_perfilAmiId IS NULL OR p_producteId IS NULL then
		RAISE_APPLICATION_ERROR(-20000,'');
	end if;
 
	-- Es verifica que el perfil sollicitant existeix en el sistema, i es recupera l'identificador de la xarxa social.
	BEGIN
    IF p_perfilSolId IS NOT NULL THEN
        SELECT XARXES_SOCIALS_ID into v_XarxaSolId
        FROM JUGADORS_XARXES
        WHERE ID = p_perfilSolId
			 AND (DATABAIXA IS NULL OR DATABAIXA > SYSDATE);
    END IF;
    EXCEPTION
        WHEN OTHERS THEN
             IF (SQLCODE = 100) THEN
                   -- Perfil sollicitant no trobat
                   RAISE_APPLICATION_ERROR(-20001,'');
             END IF; 
	END; 

	-- Es verifica que el perfil sollicitant existeix en el sistema
	BEGIN
    IF p_perfilAmiId IS NOT NULL THEN
        SELECT XARXES_SOCIALS_ID into v_XarxaAmiId
        FROM JUGADORS_XARXES
        WHERE ID = p_perfilAmiId
			 AND (DATABAIXA IS NULL OR DATABAIXA > SYSDATE);
    END IF;
    EXCEPTION
        WHEN OTHERS THEN
             IF (SQLCODE = 100) THEN
                   -- Perfil Amic no trobat
                   RAISE_APPLICATION_ERROR(-20002,'');
             END IF; 
	END; 

	-- Es verifica que el Producte existeix en el sistema
	BEGIN
    IF p_producteId IS NOT NULL THEN
        SELECT ID into v_producteId
        FROM PRODUCTES
        WHERE ID = p_producteId;
    END IF;
    EXCEPTION
        WHEN OTHERS THEN
             IF (SQLCODE = 100) THEN
                   -- Producte inexistent
                   RAISE_APPLICATION_ERROR(-20003,'');
             END IF; 
	END;  	


	-- Es verifica que l'estat SOL existeix
	BEGIN
        SELECT id into v_estatId
        FROM ESTATS
        WHERE NOM = 'SOL';
    EXCEPTION
        WHEN OTHERS THEN
             IF (SQLCODE = 100) THEN
                   -- Estat inexistent
                   RAISE_APPLICATION_ERROR(-20004,'');
             END IF; 
	END; 
	
	
	-- Es verifica que estan collaborant dos perfils de la mateixa xarxa social
	if (v_XarxaAmiId <>	v_XarxaSolId ) then
		RAISE_APPLICATION_ERROR(-20005,'');
	end if;	

	-- S'afegeix la sollicitut de collaboraci al sistema
	insert into collaboracions(JUGXAR_SOL_ID, JUGXAR_AMIC_ID, PRODUCTES_ID, ESTATS_ID, DATAINICI) values (p_perfilSolId, p_perfilAmiId, p_producteId,v_estatId, sysdate);
   
   -- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat;
	v_resultat:= '00: Collaboraci creada correctament';
      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
	
  EXCEPTION
        WHEN OTHERS THEN
				 CASE (SQLCODE) 
                    WHEN -20000 THEN
                         v_resultat := '01: Ha d''introduir els parmetres d''entrada obligatoris.';
                    WHEN -20001 THEN
                         v_resultat := '02: Perfil sollicitant no trobat ' || p_perfilSolId;
					WHEN -20002 THEN
                         v_resultat := '02: Perfil amic no trobat ' || p_perfilAmiId;
					WHEN -20003 THEN
                         v_resultat := '02: Producte no trobat amb Id ' || p_producteId;       
					WHEN -20004 THEN
                         v_resultat := '02: Estat SOL inexistent'; 		
					WHEN -20005 THEN
                         v_resultat := '02: Els dos perfils han de ser de la mateixa Xarxa Social'; 								 
					ELSE
                          v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
        END CASE;
    
	-- S'assignem els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat;
	       
	DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
	
END INS_COLLABORACIONS_SOL;
---------------------------------------------------------------------------------



---------------------------------------------------------------------------------
-- CU04-03: Acceptar collaboraci
-- Parmetres d'entrada:
--		* p_collaboracioId: Identificador de la collaboraci
-- Parmetres de sortida: 
--		* p_resultat: Resultat del procediment OK/KO
---------------------------------------------------------------------------------
PROCEDURE UPD_COLLABORACIONS_ACE (p_collaboracioId IN PFCBD.COLLABORACIONS.ID%TYPE, p_resultat OUT VARCHAR) 
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
	
	v_retorn PFCBD.LOG.RESULTAT%TYPE;
	v_vides number;
	v_colId PFCBD.COLLABORACIONS.ID%TYPE;
	v_estatId PFCBD.ESTATS.ID%TYPE;
	v_tipologia PFCBD.PRODUCTES.TIPOLOGIA%TYPE;
	v_perfilSol PFCBD.COLLABORACIONS.JUGXAR_SOL_ID%TYPE;
	v_perfilAmi PFCBD.COLLABORACIONS.JUGXAR_AMIC_ID%TYPE;
	v_producteId PFCBD.PRODUCTES.ID%TYPE;
	v_jocId PFCBD.PRODUCTES.JOCS_ID%TYPE;
	v_JugadorSolId PFCBD.JUGADORS_XARXES.JUGADORS_ID%TYPE;
	v_JugadorAmiId PFCBD.JUGADORS_XARXES.JUGADORS_ID%TYPE;
	v_partidaSolId PFCBD.PARTIDES.ID%TYPE;
	v_partidaAmiId PFCBD.PARTIDES.ID%TYPE;
	v_nivellSolId PFCBD.PARTIDES.NIVJUG_NIVELLS_ID%TYPE;
	v_nivellAmiId PFCBD.PARTIDES.NIVJUG_NIVELLS_ID%TYPE;
	v_videsExSoli PFCBD.PARTIDES.VIDESEXT%TYPE;
	v_videsCedAmi PFCBD.PARTIDES.VIDESCED%TYPE;
	v_AjNivId PFCBD.AJUDES_NIVELL.ID%TYPE;

BEGIN
  
  -- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.UPD_COLLABORACIONS_ACE';
	v_paramEnt       := 'p_collaboracioId = ' || p_collaboracioId;
  
  -- Parmetres d'entrada obligatoris
	if p_collaboracioId IS NULL then
		RAISE_APPLICATION_ERROR(-20000,'');
	end if;
 
	-- Es verifica que la collaboraci es troba donada d'alta
	BEGIN
    IF p_collaboracioId IS NOT NULL THEN
        SELECT C.JUGXAR_AMIC_ID, C.JUGXAR_SOL_ID, C.PRODUCTES_ID into v_perfilAmi, v_perfilSol, v_producteId
        FROM COLLABORACIONS C, ESTATS S
        WHERE C.ID = p_collaboracioId AND C.ESTATS_ID = S.ID AND(C.DATAFI IS NULL OR C.DATAFI > SYSDATE) and S.NOM = 'SOL';
    END IF;
    EXCEPTION
        WHEN OTHERS THEN
             IF (SQLCODE = 100) THEN
                   -- Collaboraci no vigent o inexistent
                   RAISE_APPLICATION_ERROR(-20001,'');
             END IF; 
	END; 

	-- Es verifica que l'estat ACE existeix
	BEGIN
        SELECT id into v_colId
        FROM ESTATS
        WHERE NOM = 'ACE';
    EXCEPTION
        WHEN OTHERS THEN
             IF (SQLCODE = 100) THEN
                   -- Estat inexistent
                   RAISE_APPLICATION_ERROR(-20002,'');
             END IF; 
	END; 

	-- Es verifica que el producte existeix, obtenim la seva tipologia i joc associat
	BEGIN
		SELECT tipologia, jocs_id into v_tipologia, v_jocId
        FROM PRODUCTES
        WHERE ID = v_producteId;
    EXCEPTION
        WHEN OTHERS THEN
             IF (SQLCODE = 100) THEN
                   -- Producte inexistent
                   RAISE_APPLICATION_ERROR(-20003,'');
             END IF; 
	END; 

	-- Es recupera el identificador del jugador sollicitant
	BEGIN
		SELECT JUGADORS_ID into v_JugadorSolId
        FROM JUGADORS_XARXES
        WHERE ID = v_perfilSol;
    EXCEPTION
        WHEN OTHERS THEN
             IF (SQLCODE = 100) THEN
                   -- Perfil inexistent
                   RAISE_APPLICATION_ERROR(-20004,'');
             END IF; 
	END; 

	-- Es recupera el identificador del jugador amic
	BEGIN
		SELECT JUGADORS_ID into v_JugadorAmiId
        FROM JUGADORS_XARXES
        WHERE ID = v_perfilAmi;
    EXCEPTION
        WHEN OTHERS THEN
             IF (SQLCODE = 100) THEN
                   -- Perfil inexistent
                   RAISE_APPLICATION_ERROR(-20005,'');
             END IF; 
	END; 
	

-- TRANSFERIM UNA VIDA
----------------------	
	if v_tipologia = 'VI' then
-- Es recuperaran els identificadors de les partides vigents, si existeixen, per registrar l'intercanvi de vida entre el jugador sollicitant i el jugador amic.
-- Si no existeixen partides en el dia que s'ha registrat la collaboraci, es crear una partida en l'ultim nivel jugat, donat que la vida es vigent per a tots els nivells aquell dia.
-- Si el jugador amic no ha jugat mai al joc, es crear una partida al nivell inicial.

-- Primer verificarem si el jugador amic t vides per poder cedir

	GESTIO_JOCS.CNS_JUGADOR_VIDES(v_vides,v_JugadorAmiId,v_jocId,sysdate, v_retorn);
	
	IF v_retorn = 'KO' THEN
		  RAISE_APPLICATION_ERROR(-20009,'');
	END IF;
    
	IF v_vides < 1 THEN
		RAISE_APPLICATION_ERROR(-20008,'');
	END IF;


-- Dades de partida del Jugador sollicitant del dia d'avui, si avui ha sollicitat ajuda i li han acceptat s registrar en aquesta mateixa partida
	BEGIN
      select p.ID, p.VIDESEXT into v_partidaSolId, v_videsExSoli
			from partides p, nivells_jugadors nj, nivells n
			where p.nivjug_jugadors_id = nj.jugadors_id
            and p. nivjug_nivells_id = nj.nivells_id
            and nj.nivells_id = n.id
            and n.jocs_id = v_jocId
            and p.nivjug_jugadors_id = v_JugadorSolId
            and trunc(p.datapartida) = trunc(sysdate) and rownum = 1;
		EXCEPTION
			WHEN OTHERS THEN
             IF (SQLCODE = 100) THEN
                   -- El jugador sollicitant no t partida iniciada aquest dia, es crear  una partida en l'ultim nivel que ha jugat
                   BEGIN
                    select nivjug_nivells_id into v_nivellSolId from 
                          (select p.nivjug_nivells_id 
                              from partides p, nivells_jugadors nj, nivells n
                                where p.nivjug_jugadors_id = nj.jugadors_id
                                    and p.nivjug_nivells_id = nj.nivells_id
                                    and nj.nivells_id = n.id
                                    and n.jocs_id = v_jocId
                                    and nivjug_jugadors_id = v_JugadorSolId
                              order by datapartida desc)
                          where rownum = 1;
     				EXCEPTION
						WHEN OTHERS THEN
						 IF (SQLCODE = 100) THEN
							   -- No t gaire sentit que un jugador sollicitant no hagi jugat mai al nostre joc, però es registrarà una partida al nivell inicial, 
								-- per permetre aquesta casuística i no limitar el sistema.
								  select id into v_nivellSolId  
								  from NIVELLS WHERE id NOT IN 
									(SELECT DISTINCT NIVSEG_ID FROM NIVELLS WHERE NIVSEG_ID is not null and jocs_id = v_jocId )
								  and jocs_id = v_jocId;
						 END IF; 
					END; 
			END IF; 
	END; 
		
-- Dades de partida del Jugador Amic, si ha iniciat ja una partida aquest dia es registrar en aquesta.
	BEGIN
      select p.ID, p.VIDESCED into v_partidaAmiId, v_videsCedAmi
			from partides p, nivells_jugadors nj, nivells n
			where p.nivjug_jugadors_id = nj.jugadors_id
            and p. nivjug_nivells_id = nj.nivells_id
            and nj.nivells_id = n.id
            and n.jocs_id = v_jocId
            and p.nivjug_jugadors_id = v_JugadorAmiId
            and trunc(p.datapartida) = trunc(sysdate) and rownum = 1;
		EXCEPTION
			WHEN OTHERS THEN
             IF (SQLCODE = 100) THEN
                   -- El jugador Amic no t partida iniciada aquest dia, es crear  una partida en l'ultim nivel que ha jugat
                   BEGIN
                       select nivjug_nivells_id into v_nivellAmiId from 
                          (select p.nivjug_nivells_id 
                              from partides p, nivells_jugadors nj, nivells n
                                where p.nivjug_jugadors_id = nj.jugadors_id
                                    and p.nivjug_nivells_id = nj.nivells_id
                                    and nj.nivells_id = n.id
                                    and n.jocs_id = v_jocId
                                    and nivjug_jugadors_id = v_JugadorAmiId
                              order by datapartida desc)
                        where rownum = 1;
                   EXCEPTION
                      WHEN OTHERS THEN
                       IF (SQLCODE = 100) THEN
                           -- Si el jugador amic no havia jugat mai a aquest joc, es crear  un registre partida al nivell inicial
                            select ID into v_nivellAmiId from NIVELLS 
                            WHERE id NOT IN 
                              (SELECT DISTINCT NIVSEG_ID FROM NIVELLS WHERE NIVSEG_ID is not null and jocs_id = v_jocId ) 
                              and jocs_id = v_jocId;
                       END IF; 
                    END; 
			END IF; 
	END; 	
	
	
		-- Afegim la vida extra al jugador que va sollicitar la vida	
			if v_partidaSolId is not null then
				v_videsExSoli := v_videsExSoli + 1;
				update partides set VIDESEXT = v_videsExSoli where id = v_partidaSolId;
			else
			    BEGIN
					SELECT NIVELLS_ID INTO v_nivellSolId
					FROM NIVELLS_JUGADORS
					WHERE NIVELLS_ID = v_nivellSolId AND JUGADORS_ID = v_JugadorSolId;
				  EXCEPTION
					WHEN OTHERS THEN
					   IF (SQLCODE = 100) THEN
						   -- Es contempla que el jugador no sollicitant no hagi jugat al joc mai, s'inicia a nivell 1.
						  insert into nivells_jugadors (NIVELLS_ID, JUGADORS_ID, DATAINICI) values (v_nivellSolId,v_JugadorSolId, sysdate );
					   END IF; 
				END; 
				insert into partides  (NIVJUG_JUGADORS_ID, NIVJUG_NIVELLS_ID, DATAPARTIDA, VIDESEXT) values (v_JugadorSolId, v_nivellSolId, sysdate, 1);
			end if;
			
		-- Afegim la vida cedida al jugador que va acceptar la collaboraci	
			if v_partidaAmiId is not null then
				v_videsCedAmi := v_videsCedAmi + 1;
				update partides set VIDESCED = v_videsCedAmi where id = v_partidaAmiId;
			else
      
         BEGIN
            SELECT NIVELLS_ID INTO v_nivellAmiId
            FROM NIVELLS_JUGADORS
            WHERE NIVELLS_ID = v_nivellAmiId AND JUGADORS_ID = v_JugadorAmiId;
          EXCEPTION
            WHEN OTHERS THEN
               IF (SQLCODE = 100) THEN
                   -- Es contempla que el jugador no sollicitant no hagi jugat al joc mai, s'inicia a nivell 1.
                  insert into nivells_jugadors (NIVELLS_ID, JUGADORS_ID, DATAINICI) values (v_nivellAmiId,v_JugadorAmiId, sysdate );
               END IF; 
            END; 
            	insert into partides  (NIVJUG_JUGADORS_ID, NIVJUG_NIVELLS_ID, DATAPARTIDA, VIDESCED) values (v_JugadorAmiId, v_nivellAmiId, sysdate, 1);
			end if;
	End if;
	

-- TRANSFERIM UNA AJUDA
-----------------------	
	if v_tipologia = 'AJ' then
	
		-- Es verifica que l'ajuda a transferir la t activa el jugador amic
		BEGIN
		IF v_producteId IS NOT NULL THEN
			SELECT id, nivjug_nivells_id into v_AjNivId, v_nivellAmiId
			FROM AJUDES_NIVELL
			WHERE PRODUCTES_ID = v_producteId AND NIVJUG_JUGADORS_ID = v_JugadorAmiId AND ACTIVA = 'Y' and trunc(datainici) <= trunc(sysdate);
		END IF;
		EXCEPTION
			WHEN OTHERS THEN
				 IF (SQLCODE = 100) THEN
					   -- L'ajuda no es pot transferir perque ja no la t activa el jugador amic.
					   RAISE_APPLICATION_ERROR(-20006,'');
				 END IF; 
		END; 
		
    
		-- Es verifica que el jugador sollicitant ha jugat al nivell de l'ajuda
		BEGIN
			SELECT NIVELLS_ID INTO v_nivellAmiId
			FROM NIVELLS_JUGADORS
			WHERE NIVELLS_ID = v_nivellAmiId AND JUGADORS_ID = v_JugadorSolId;
		EXCEPTION
			WHEN OTHERS THEN
				 IF (SQLCODE = 100) THEN
					   -- L'ajuda no es pot transferir, el jugador sollicitant no ha pogut demanar l'ajuda perque no ha iniciat el nivell
					   RAISE_APPLICATION_ERROR(-20007,'');
				 END IF; 
		END; 
		
		UPDATE AJUDES_NIVELL SET ACTIVA = 'N' WHERE ID = v_AjNivId;
		INSERT INTO AJUDES_NIVELL (NIVJUG_JUGADORS_ID, NIVJUG_NIVELLS_ID, PRODUCTES_ID, datainici ) VALUES (v_JugadorSolId, v_nivellAmiId, v_producteId, sysdate);
	
	end if;
	
	
	-- ACTUALITZEM LA COLLABORACI A ACCEPTADA	
	update collaboracions set ESTATS_ID = v_colId, datafi = sysdate where id = p_collaboracioId;
   
    -- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat;
	v_resultat:= '00: Collaboraci Acceptada correctament';
      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
  

  EXCEPTION
        WHEN OTHERS THEN
				 CASE (SQLCODE) 
                    WHEN -20000 THEN
                         v_resultat := '01: Ha d''introduir els parmetres d''entrada obligatoris.';
                    WHEN -20001 THEN
                         v_resultat := '02: Collaboraci no vigent o inexistent ' || p_collaboracioId;
                    WHEN -20002 THEN
                         v_resultat := '02: Estat ACE inexistent';
                    WHEN -20003 THEN
                         v_resultat := '02: Producte no trobat amb Id ' || v_producteId;
                    WHEN -20004 THEN
                         v_resultat := '02: Perfil del sollicitant inexistent ' || v_perfilSol;
                    WHEN -20005 THEN
                         v_resultat := '02: Perfil del jugador amic inexistent ' || v_perfilAmi;
                    WHEN -20006 THEN
                         v_resultat := '02: L''ajuda no est activa en el jugador amic.' || p_collaboracioId;    
                    WHEN -20007 THEN
                         v_resultat := '02: El jugador sollicitant no ha desbloquejat el nivel pel que requereix l''ajuda.';        
                    WHEN -20008 THEN
                         v_resultat := '02: El jugador amic no t vides disponibles, avui no podr collaborar.';     
                    WHEN -20009 THEN
                         v_resultat := '03: Error en la crida del procediment emmagatzemat ConsultaVida.';     
                    ELSE
                         v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
        END CASE;
    
	-- S'assignem els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat;
        
	DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
	
END UPD_COLLABORACIONS_ACE;
---------------------------------------------------------------------------------


---------------------------------------------------------------------------------
-- CU04-04: Rebutjar Collaboraci
-- Parmetres d'entrada:
--		* p_collaboracioId: Identificador de la Collaboraci
-- Parmetres de sortida: 
--		* p_resultat: Resultat del procediment OK/KO
---------------------------------------------------------------------------------
PROCEDURE UPD_COLLABORACIONS_REF (p_collaboracioId IN PFCBD.COLLABORACIONS.ID%TYPE, p_resultat OUT VARCHAR) 
IS
	v_colId PFCBD.COLLABORACIONS.ID%TYPE;
	v_estatId PFCBD.ESTATS.ID%TYPE;
	
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;

BEGIN
  -- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.UPD_COLLABORACIONS_REF';
	v_paramEnt       := 'p_collaboracioId = ' || p_collaboracioId;

	-- Parmetres d'entrada obligatoris
	if p_collaboracioId IS NULL then
		RAISE_APPLICATION_ERROR(-20000,'');
	end if;
 
	-- Es verifica que la Collaboraci es troba donada d'alta
	BEGIN
    IF p_collaboracioId IS NOT NULL THEN
        SELECT C.ID into v_colId
        FROM COLLABORACIONS C, ESTATS S
        WHERE C.ID = p_collaboracioId AND C.ESTATS_ID = S.ID AND (C.DATAFI IS NULL OR C.DATAFI > SYSDATE) and S.NOM = 'SOL';
    END IF;
    EXCEPTION
        WHEN OTHERS THEN
             IF (SQLCODE = 100) THEN
                   -- Collaboraci no vigent o inexistent
                   RAISE_APPLICATION_ERROR(-20001,'');
             END IF; 
	END; 

	-- Es verifica que l'estat SOL existeix
	BEGIN
        SELECT id into v_colId
        FROM ESTATS
        WHERE NOM = 'REF';
    EXCEPTION
        WHEN OTHERS THEN
             IF (SQLCODE = 100) THEN
                   -- Estat inexistent
                   RAISE_APPLICATION_ERROR(-20002,'');
             END IF; 
	END; 
	
	
	-- S'actualitza la collaboraci
	update collaboracions set ESTATS_ID = v_colId, datafi = sysdate where id = p_collaboracioId;
    
	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat;
	v_resultat:= '00: Collaboraci Rebutjada correctament';
      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);
	 

  EXCEPTION
        WHEN OTHERS THEN
				 CASE (SQLCODE) 
                    WHEN -20000 THEN
                         v_resultat := '01: Ha d''introduir els parmetres d''entrada obligatoris.';
                    WHEN -20001 THEN
                         v_resultat := '02: Collaboraci no vigent o inexistent ' || p_collaboracioId;
					WHEN -20002 THEN
                         v_resultat := '02: Estat REF inexistent'; 		
					ELSE
                         v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
        END CASE;
    
	-- S'assignem els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat;
        
	DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
	
END UPD_COLLABORACIONS_REF;
---------------------------------------------------------------------------------


---------------------------------------------------------------------------------
-- CU04-05: Consultar Collaboraci
-- Parmetres d'entrada:
--		* p_jocId: Identificador del Joc
--		* p_collaboracioId: Identificador de la Collaboraci
--		* p_perfilsolId: Identificador del perfil
--		* p_actives: Indicador per extreure noms les collaboracions encara actives.
-- Parmetres de sortida: 
--		* p_rs: Recordset amb la informaci
--		* p_resultat: Resultat del procediment OK/KO
-- Procediment per extreure el llistat de les collaboracions d'un perfil concret, si el procediment rep l'identificador de la Collaboraci noms extreur les dades
-- d'aquella Collaboraci.
-- Es permetr incloure el filtrat del Joc, per permetre extreure totes les collaboracions d'un perfil, o noms les collaboracions del perfil dins del Joc donat.
---------------------------------------------------------------------------------
PROCEDURE CNS_COLLABORACIONS_LIST (p_rs IN OUT sys_refcursor, 
                                   p_jocId IN PFCBD.JOCS.ID%TYPE,
                                   p_collaboracioId IN PFCBD.COLLABORACIONS.ID%TYPE, 
                                   p_perfilsolId IN PFCBD.JUGADORS_XARXES.ID%TYPE, 
                                   p_actives IN varchar,
								   p_resultat OUT VARCHAR) 
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
	
	v_colId PFCBD.COLLABORACIONS.ID%TYPE;
	v_perfilId PFCBD.JUGADORS_XARXES.ID%TYPE;
	v_jocId PFCBD.JOCS.ID%TYPE;
BEGIN
  	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.CNS_COLLABORACIONS_LIST';
	v_paramEnt       := 'p_jocId = ' || p_jocId 
                      || ', p_collaboracioId = ' || p_collaboracioId
					  || ', p_perfilsolId = ' || p_perfilsolId
					  || ', p_actives = ' || p_actives;
  
	-- Parmetres d'entrada obligatoris
	if p_collaboracioId IS NULL AND p_perfilsolId is null then
		RAISE_APPLICATION_ERROR(-20000,'');
	end if;
 
	-- Es verifica que la Collaboraci es troba donada d'alta
	BEGIN
    IF p_collaboracioId IS NOT NULL THEN
        SELECT ID into v_colId
        FROM COLLABORACIONS
        WHERE ID = p_collaboracioId;
    END IF;
    EXCEPTION
        WHEN OTHERS THEN
             IF (SQLCODE = 100) THEN
                   -- Collaboraci inexistent
                   RAISE_APPLICATION_ERROR(-20001,'');
             END IF; 
	END; 

	-- Es verifica que el perfil existeix
	BEGIN
    IF p_perfilsolId IS NOT NULL THEN
        SELECT ID into v_perfilId
        FROM JUGADORS_XARXES
        WHERE ID = p_perfilsolId;
    END IF;
    EXCEPTION
        WHEN OTHERS THEN
             IF (SQLCODE = 100) THEN
                   -- Perfil inexistent
                   RAISE_APPLICATION_ERROR(-20002,'');
             END IF; 
	END; 
    
	-- Es verifica que el Joc existeix
	BEGIN
    IF p_jocId IS NOT NULL THEN
        SELECT ID into v_jocId
        FROM JOCS
        WHERE ID = p_jocId;
    END IF;
    EXCEPTION
        WHEN OTHERS THEN
             IF (SQLCODE = 100) THEN
                   -- Joc inexistent
                   RAISE_APPLICATION_ERROR(-20003,'');
             END IF; 
	END; 
  

	-- Es realitza la consulta sobre la taula de collaboracions	
	if p_collaboracioId	is not null then
		open p_rs for
		select * from collaboracions where id = p_collaboracioId;
	else
		if p_actives is not null then
			  if p_jocId is null then
				  open p_rs for
				  select * from collaboracions where (jugxar_sol_id = p_perfilsolId or jugxar_amic_id = p_perfilsolId) and (datafi is null or datafi > sysdate);
			  else
				  open p_rs for
				  select c.* 
				  from collaboracions c, productes p, jocs j
				  where c.productes_id = p.id  and p.jocs_id = j.id and j.id = p_jocId  
						 and (c.jugxar_sol_id = p_perfilsolId or c.jugxar_amic_id = p_perfilsolId) and (c.datafi is null or c.datafi > sysdate);    
			   end if;
		else
			if p_jocId is null then
				open p_rs for
				select * from collaboracions where (jugxar_sol_id = p_perfilsolId or jugxar_amic_id = p_perfilsolId);
			else
				open p_rs for
				  select c.* 
				  from collaboracions c, productes p, jocs j
				  where c.productes_id = p.id  and p.jocs_id = j.id and j.id = p_jocId  
					 and (c.jugxar_sol_id = p_perfilsolId or c.jugxar_amic_id = p_perfilsolId);    
			end if;
		end if;
	end If;
	
	  
	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat || ', p_rs: Recordset amb el resultat.';
	v_resultat:= '00: Consulta realitzada correctament';
      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
 

  EXCEPTION
        WHEN OTHERS THEN
				 CASE (SQLCODE) 
                    WHEN -20000 THEN
                         v_resultat := '01: Ha d''introduir els parmetres d''entrada obligatoris.';
                    WHEN -20001 THEN
                         v_resultat := '02: Collaboraci inexistent Id ' || p_collaboracioId;
                    WHEN -20002 THEN
                         v_resultat := '02: Perfil inexistent Id ' || p_perfilsolId; 	
                    WHEN -20003 THEN
                         v_resultat := '02: Joc inexistent Id ' || p_jocId; 	
                    ELSE
                         v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
        END CASE;
    
	-- Es genera un recordset buit
        OPEN p_rs FOR
            SELECT * FROM DUAL WHERE DUMMY IS NULL;
				
				
	-- S'assignem els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat || ', p_rs: Recordset buit.';
        
	DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
	
END CNS_COLLABORACIONS_LIST;
---------------------------------------------------------------------------------


---------------------------------------------------------------------------------
-- CU04-06: Cancellar Collaboraci
-- Parmetres d'entrada:
--		* p_collaboracioId: Identificador de la Collaboraci
-- Parmetres de sortida:
--		* p_resultat: Resultat del procediment OK/KO
---------------------------------------------------------------------------------
PROCEDURE UPD_COLLABORACIONS_CAN (p_collaboracioId IN PFCBD.COLLABORACIONS.ID%TYPE, p_resultat OUT VARCHAR) 
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
	
	v_colId PFCBD.COLLABORACIONS.ID%TYPE;
	v_estatId PFCBD.ESTATS.ID%TYPE;

BEGIN
  	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.UPD_COLLABORACIONS_CAN';
	v_paramEnt       := 'p_collaboracioId = ' || p_collaboracioId;
  
	-- Parmetres d'entrada obligatoris
	if p_collaboracioId IS NULL then
		RAISE_APPLICATION_ERROR(-20000,'');
	end if;
 
	-- Es verifica que la Collaboraci es troba donada d'alta
	BEGIN
    IF p_collaboracioId IS NOT NULL THEN
        SELECT C.ID into v_colId
        FROM COLLABORACIONS C, ESTATS S
        WHERE C.ID = p_collaboracioId AND C.ESTATS_ID = S.ID AND (C.DATAFI IS NULL OR C.DATAFI > SYSDATE) and S.NOM = 'SOL';
    END IF;
    EXCEPTION
        WHEN OTHERS THEN
             IF (SQLCODE = 100) THEN
                   -- Collaboraci no vigent o inexistent
                   RAISE_APPLICATION_ERROR(-20001,'');
             END IF; 
	END; 

	-- Es verifica que l'estat SOL existeix
	BEGIN
        SELECT id into v_colId
        FROM ESTATS
        WHERE NOM = 'CAN';
    EXCEPTION
        WHEN OTHERS THEN
             IF (SQLCODE = 100) THEN
                   -- Estat inexistent
                   RAISE_APPLICATION_ERROR(-20002,'');
             END IF; 
	END; 
	
	-- S'actualitza la collaboraci
	update collaboracions set ESTATS_ID = v_colId, datafi = sysdate where id = p_collaboracioId;

    -- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat;
	v_resultat:= '00: Collaboraci cancellada correctament';
      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
  
  EXCEPTION
        WHEN OTHERS THEN
				 CASE (SQLCODE) 
                    WHEN -20000 THEN
                         v_resultat := '01: Ha d''introduir els parmetres d''entrada obligatoris.';
                    WHEN -20001 THEN
                         v_resultat := '02: Collaboraci no vigent o inexistent ' || p_collaboracioId;
					WHEN -20002 THEN
                         v_resultat := '02: Estat CAN inexistent'; 		
					ELSE
                         v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
        END CASE;
   
  	-- S'assignem els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat;
        
	DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);
	
END UPD_COLLABORACIONS_CAN;
---------------------------------------------------------------------------------

END COLLABORACIO;
/
