---------------------------------------------------------------------------------
-- Alumne: Jessica Villamor 
-- Consultor: Juan Martnez Bolaos
-- Projecte: GESTIO DE VIDEOJOCS
---------------------------------------------------------------------------------
--
-- Ens situem a l'esquema de la base de dades de Gesti de Videojocs
--
ALTER SESSION SET CURRENT_SCHEMA = PFCBD;

-- Creaci de la definici del paquet
Create or replace
PACKAGE    PFCBD.DM_JOCS AS

	PROCEDURE INS_JOCS ( nom IN PFCBD.jocs.nom%TYPE, vides IN PFCBD.jocs.vides%TYPE, p_resultat OUT VARCHAR);

	PROCEDURE UPD_JOCS ( p_id IN PFCBD.jocs.id%TYPE, p_nom IN PFCBD.jocs.nom%TYPE, p_vides IN PFCBD.jocs.vides%TYPE, p_resultat OUT VARCHAR);

	PROCEDURE  DEL_JOCS (p_id IN PFCBD.jocs.id%TYPE, p_resultat OUT VARCHAR);

	PROCEDURE CNS_JOCS (p_rs IN OUT	sys_refcursor, p_id	IN	PFCBD.jocs.id%TYPE,	p_nom IN PFCBD.jocs.nom%TYPE, p_resultat OUT VARCHAR);
	
END DM_JOCS;
/



-- Creaci del cos del paquet
CREATE OR REPLACE PACKAGE BODY PFCBD.DM_JOCS AS

---------------------------------------------------------------------------------
-- Procediments CU05: Gesti Dades Joc
---------------------------------------------------------------------------------

---------------------------------------------------------------------------------
-- CU05-01: Alta Joc
-- Parmetres d'entrada:
--		* nom: Nom del Joc
--		* vides: Vides per defecte
-- Parmetres de sortida: 
--		* p_resultat: Resultat del procediment OK/KO
---------------------------------------------------------------------------------
PROCEDURE INS_JOCS ( nom IN PFCBD.jocs.nom%TYPE, vides IN PFCBD.jocs.vides%TYPE, p_resultat OUT VARCHAR)
IS
 -- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
BEGIN
	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.INS_JOCS';
	v_paramEnt       := 'nom = ' || nom 
                      || ', vides = ' || vides;
	
	INSERT INTO JOCS(NOM, VIDES) VALUES(UPPER(nom),vides);

	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat;
	v_resultat     := '00: Joc Creat Correctament';
	
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	

EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                    WHEN -1 THEN
                        -- Registre duplicat
                        v_resultat := '01: El Joc ja existeix.';
                    WHEN -1400 THEN
                        -- Valor nulo
                        v_resultat := '02: Sha intentat inserir un valor NULL.' || SUBSTR(SQLERRM,INSTR(SQLERRM,'('));
                    ELSE
                        -- Error no controlado
                        v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
                END CASE;            

	-- S'assignem els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat;
        
	DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
				
END INS_JOCS;
---------------------------------------------------------------------------------


---------------------------------------------------------------------------------
-- CU05-02: Actualitzar Joc
-- Parmetres d'entrada:
--		* p_id: Identificador del Joc
--		* p_nom: Nom del Joc
--		* p_vides: Vides per defecte
-- Parmetres de sortida: 
--		* p_resultat: Resultat del procediment OK/KO
---------------------------------------------------------------------------------
PROCEDURE UPD_JOCS ( p_id IN PFCBD.jocs.id%TYPE, p_nom IN PFCBD.jocs.nom%TYPE, p_vides IN PFCBD.jocs.vides%TYPE, p_resultat OUT VARCHAR)
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
BEGIN
	
	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.UPD_JOCS';
	v_paramEnt       := 'p_id = ' || p_id 
                      || ', p_nom = ' || p_nom
					  || ', p_vides = ' || p_vides;
		
 			  
	UPDATE JOCS SET NOM = upper(p_nom),	VIDES = p_vides WHERE JOCS.ID = p_id;
    
  IF SQL%rowcount = 0
	THEN
		RAISE_APPLICATION_ERROR(-20000,'');
	ELSE
		v_resultat := '00: Joc Actualitzat Correctament';
	END IF;

	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat;
      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	

EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                    WHEN -1 THEN
                        -- Registre duplicat
                        v_resultat := '01: Ja existeix un joc amb el mateix Nom.';
                    WHEN -1407 THEN
                        -- Valor nulo
                        v_resultat := '02: Sha intentat inserir un valor NULL.' || SUBSTR(SQLERRM,INSTR(SQLERRM,'('));
                    WHEN -20000 then
                         v_resultat := '03: Joc no trobat amb ID = ' || p_id;
                    ELSE
                        -- Error no controlat
                        v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
                END CASE;            

  -- S'assignem els valors de sortida 
    p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat;
        
    DBMS_OUTPUT.put_line (v_resultat);				
-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
				
END UPD_JOCS;
---------------------------------------------------------------------------------


---------------------------------------------------------------------------------
-- CU05-03: Esborrar Joc
-- Parmetres d'entrada:
--		* p_id: Identificador del Joc
-- Parmetres de sortida: 
--		* p_resultat: Resultat del procediment OK/KO
---------------------------------------------------------------------------------
PROCEDURE  DEL_JOCS (p_id IN PFCBD.jocs.id%TYPE, p_resultat OUT VARCHAR)
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
BEGIN
		
	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.DEL_JOCS';
	v_paramEnt       := 'p_id = ' || p_id;

	
	-- Inici de l'execuci del procediment
	DELETE JOCS
	WHERE id = p_id;

	-- Comprovem si s'ha trobat el registre a esborrar
	IF SQL%rowcount = 0
	THEN
		RAISE_APPLICATION_ERROR(-20000,'');
	ELSE
		v_resultat := '00: Joc esborrat correctament';
	END IF;

	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat;
	      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
 
EXCEPTION
	WHEN OTHERS THEN
		CASE (SQLCODE)
            WHEN -2292 THEN
            -- Restricci d'integritat
                    v_resultat := '04: S''ha intentar esborrar un registre amb dependncies.' || SUBSTR(SQLERRM,INSTR(SQLERRM,'('));
            WHEN -20000 then
                    v_resultat := '03: Joc no trobat amb ID = ' || p_id;
			ELSE
                   -- Error no controlat
                   v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
        END CASE;            
		
	-- S'assignem els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat;
        
	DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
		
END DEL_JOCS;
---------------------------------------------------------------------------------


---------------------------------------------------------------------------------
-- CU05-04: Consultar Joc
-- Parmetres d'entrada:
--		* p_id: Identificador del Joc
--		* p_nom: Nom del Joc
-- Parmetres de sortida:
--		* p_rs: Recordset amb el resultat
--		* p_resultat: Resultat del procediment OK/KO
--
-- Si s'indica l'idnetificador del joc es tornar les dades del joc, si s'indica el nom es tornaran els jocs que continguin el nom especificat.
-- Si no s'especifica cap filtre es retornaran tots els jocs del sistema.
---------------------------------------------------------------------------------
PROCEDURE CNS_JOCS (   
	p_rs		IN OUT	sys_refcursor,
	p_id	IN		PFCBD.jocs.id%TYPE,
	p_nom	IN		PFCBD.jocs.nom%TYPE,
	p_resultat OUT VARCHAR
)
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
	
BEGIN
	
	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.CNS_JOCS';
	v_paramEnt       := 'p_id = ' || p_id 
                      || ', p_nom = ' || p_nom;
					  
	-- Inici de l'execuci del procediment
	IF p_id IS NULL AND p_nom IS NULL THEN
		OPEN p_rs FOR
			SELECT * FROM JOCS; 
	ELSIF p_id IS NOT NULL THEN
		OPEN p_rs FOR
			SELECT * FROM JOCS WHERE ID = p_id;
	ELSIF p_nom IS NOT NULL THEN
		OPEN p_rs FOR
			SELECT * FROM JOCS WHERE NOM LIKE '%' || UPPER(p_nom) || '%';
	END IF;

	   
	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat || ', p_rs: Rercordset amb el resultat de la consulta.';
	v_resultat:= '00: Consulta realitzada correctament'; 
      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);

EXCEPTION
	WHEN OTHERS THEN
	 		 v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
	
	 -- Es genera un recordset buit
        OPEN p_rs FOR
            SELECT * FROM DUAL WHERE DUMMY IS NULL;
				
				
	-- S'assignem els valors de sortida 
		p_resultat := 'KO';
		v_dataFi    := SYSDATE;
		v_paramSor	:= 'p_resultat: ' || p_resultat || ', p_rs: Rercordset buit.';
        
		DBMS_OUTPUT.put_line (v_resultat);				
-- Gravem al LOG
		GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
END CNS_JOCS;
---------------------------------------------------------------------------------


END DM_JOCS;
/
