---------------------------------------------------------------------------------
-- Alumne: Jessica Villamor 
-- Consultor: Juan Martnez Bolaos
-- Projecte: GESTIO DE VIDEOJOCS
---------------------------------------------------------------------------------
--
-- Ens situem a l'esquema de la base de dades de Gesti de Videojocs
--
ALTER SESSION SET CURRENT_SCHEMA = PFCBD;

-- Creaci de la definici del paquet
Create or replace
PACKAGE    PFCBD.DM_NIVELLS AS
	PROCEDURE INS_NIVELLS ( p_joc_id IN PFCBD.NIVELLS.JOCS_ID%TYPE, p_nom IN PFCBD.NIVELLS.NOM%TYPE, p_segniv IN PFCBD.NIVELLS.NIVSEG_ID%TYPE, p_resultat OUT VARCHAR);

	PROCEDURE UPD_NIVELLS ( p_id IN PFCBD.NIVELLS.id%TYPE, p_joc_id IN PFCBD.NIVELLS.JOCS_ID%TYPE, p_nom IN PFCBD.NIVELLS.NOM%TYPE, p_segniv IN PFCBD.NIVELLS.NIVSEG_ID%TYPE, p_resultat OUT VARCHAR);

	PROCEDURE  DEL_NIVELLS (p_id IN PFCBD.NIVELLS.id%TYPE, p_resultat OUT VARCHAR);

	PROCEDURE CNS_NIVELLS ( p_rs IN OUT	sys_refcursor, 	p_id IN PFCBD.NIVELLS.id%TYPE, 	p_joc_id IN PFCBD.NIVELLS.JOCS_ID%TYPE, p_nom IN PFCBD.NIVELLS.NOM%TYPE, p_segniv IN PFCBD.NIVELLS.NIVSEG_ID%TYPE, p_resultat OUT VARCHAR);

END DM_NIVELLS;
/


-- Creaci del cos del paquet
CREATE OR REPLACE PACKAGE BODY PFCBD.DM_NIVELLS AS

---------------------------------------------------------------------------------
-- Procediments CU06: Gesti Dades Nivells
---------------------------------------------------------------------------------

---------------------------------------------------------------------------------
-- CU06-01: Alta Nivell
-- Parmetres d'entrada:
--		* p_joc_id: Identificador del Joc
--		* p_nom: Nom del Nivell
--		* p_segniv: Identificador del segent nivell
-- Parmetres de sortida: 
--		* p_resultat: Resultat del procediment OK/KO
---------------------------------------------------------------------------------
PROCEDURE INS_NIVELLS ( p_joc_id IN PFCBD.NIVELLS.JOCS_ID%TYPE, p_nom IN PFCBD.NIVELLS.NOM%TYPE, p_segniv IN PFCBD.NIVELLS.NIVSEG_ID%TYPE, p_resultat OUT VARCHAR)
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
	
	v_jocid PFCBD.NIVELLS.JOCS_ID%TYPE;
	v_segniv PFCBD.NIVELLS.NIVSEG_ID%TYPE;
  
BEGIN
	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.INS_NIVELLS';
	v_paramEnt       := 'p_joc_id = ' || p_joc_id 
                      || ', p_nom = ' || p_nom
					  || ', p_segniv = ' || p_segniv;
	
	BEGIN
		SELECT ID into v_jocid
            FROM JOCS
            WHERE ID = p_joc_id;
		EXCEPTION
            WHEN OTHERS THEN
                IF (SQLCODE = 100) THEN
                    -- Joc no trobat
                    RAISE_APPLICATION_ERROR(-20000,'');
                END IF; 
	END;   
  
	BEGIN		
		IF p_segniv IS NOT NULL THEN
				SELECT ID INTO v_segniv
				FROM NIVELLS
				WHERE ID = p_segniv;
		END IF;    
		
			EXCEPTION
				WHEN OTHERS THEN
					IF (SQLCODE = 100) THEN
						-- Nivell no trobat
						RAISE_APPLICATION_ERROR(-20001,'');
		END IF; 
	END;  
				
	INSERT INTO NIVELLS(JOCS_ID, NOM, NIVSEG_ID) VALUES (p_joc_id, UPPER(p_nom), p_segniv);

		
	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat;
	v_resultat     := '00: Nivell creat Correctament';
      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
	

EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                    WHEN -1 THEN
                        -- Registre duplicat
                        v_resultat := '01: El Nivell ja existeix.';
                    WHEN -1400 THEN
                        -- Valor nul
                        v_resultat := '02: S''ha intentat inserir un valor NULL.' || SUBSTR(SQLERRM,INSTR(SQLERRM,'('));
					WHEN -20000 THEN
                        v_resultat := '03: Joc no trobat amb ID = ' || p_joc_id;
					WHEN -20001 THEN
                        v_resultat := '03: Nivell no trobat amb ID = ' || p_segniv;
                    ELSE
                        -- Error no controlat
                        v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
                END CASE;            

	-- S'assignem els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat;
        
	DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
				
END INS_NIVELLS;
---------------------------------------------------------------------------------



---------------------------------------------------------------------------------
-- CU06-02: Actualitzar Nivell
-- Parmetres d'entrada:
--		* p_id: Identificador del Nivell
--		* p_joc_id: Identificador del Joc
--		* p_nom: Nom del Nivell
--		* p_segniv: Identificador del següent nivell
-- Parmetres de sortida: 
--		* p_resultat: Resultat del procediment OK/KO
---------------------------------------------------------------------------------
PROCEDURE UPD_NIVELLS ( p_id IN PFCBD.NIVELLS.id%TYPE, p_joc_id IN PFCBD.NIVELLS.JOCS_ID%TYPE, p_nom IN PFCBD.NIVELLS.NOM%TYPE, p_segniv IN PFCBD.NIVELLS.NIVSEG_ID%TYPE, p_resultat OUT VARCHAR)
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
	
	v_jocid PFCBD.NIVELLS.JOCS_ID%TYPE;
	v_segniv PFCBD.NIVELLS.NIVSEG_ID%TYPE;
  
BEGIN

	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.UPD_NIVELLS';
	v_paramEnt       := 'p_id = ' || p_id 
                      || ', p_joc_id = ' || p_joc_id
					  || ', p_nom = ' || p_nom
					  || ', p_segniv = ' || p_segniv;
  
	-- Es verifica que el Joc existeix al sistema
	BEGIN
  	    IF p_joc_id IS NOT NULL THEN
        SELECT ID into v_jocid
                  FROM JOCS
                  WHERE ID = p_joc_id;
		END IF;
		EXCEPTION
			WHEN OTHERS THEN
				 IF (SQLCODE = 100) THEN
					   -- Joc no trobat
					   RAISE_APPLICATION_ERROR(-20000,'');
				 END IF; 
	END;  
    
  
	BEGIN		
		-- Es verifica que el segent nivell ja existeix al sistema
		IF p_segniv IS NOT NULL THEN
				SELECT ID INTO v_segniv
				FROM NIVELLS
				WHERE ID = p_segniv;
		END IF;    
		EXCEPTION
				WHEN OTHERS THEN
					IF (SQLCODE = 100) THEN
						-- Nivell no trobat
						RAISE_APPLICATION_ERROR(-20001,'');
					END IF; 
	END;  
			
			
	UPDATE NIVELLS SET JOCS_ID = p_joc_id, NOM = upper(p_nom),	NIVSEG_ID = p_segniv WHERE NIVELLS.ID = p_id;
    
	IF SQL%rowcount = 0
		THEN
			RAISE_APPLICATION_ERROR(-20001,'');
		ELSE
			v_resultat := '00: Nivell Actualitzat Correctament';
	END IF;
    
	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat;
	      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	

EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                    WHEN -1 THEN
                        -- Registre duplicat
                        v_resultat := '01: Ja existeix un nivell amb el mateix Joc/Nom.';
                    WHEN -1407 THEN
                        -- Valor nulo
                        v_resultat := '02: S''ha intentat inserir un valor NULL.' || SUBSTR(SQLERRM,INSTR(SQLERRM,'('));
                    WHEN -20000 THEN
                                  v_resultat := '03: Joc no trobat amb ID = ' || p_joc_id;
                    WHEN -20001 THEN
                                  v_resultat := '03: Nivell no trobat amb ID = ' || p_segniv || '_' || p_id;
                    ELSE
                        -- Error no controlat
                        v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
                END CASE;            

	-- S'assignem els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat;
        
	DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
				
END UPD_NIVELLS;
---------------------------------------------------------------------------------


---------------------------------------------------------------------------------
-- CU06-03: Esborrar Nivell
-- Parmetres d'entrada:
--		* p_id: Identificador del Nivell
-- Parmetres de sortida: 
--		* p_resultat: Resultat del procediment OK/KO
---------------------------------------------------------------------------------
PROCEDURE  DEL_NIVELLS (p_id IN PFCBD.NIVELLS.id%TYPE, p_resultat OUT VARCHAR)
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
BEGIN
	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.DEL_NIVELLS';
	v_paramEnt       := 'p_id = ' || p_id;
		
	-- Inici de l'execuci del procediment
	DELETE NIVELLS
	WHERE id = p_id;

	-- Comprovem si s'ha trobat el registre a esborrar
	IF SQL%rowcount = 0
	THEN
		RAISE_APPLICATION_ERROR(-20000,'');
	ELSE
		v_resultat := '00: Nivell esborrat correctament';
	END IF;

	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat;
      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
  
EXCEPTION
	WHEN OTHERS THEN
		CASE (SQLCODE)
            WHEN -2292 THEN
					-- Restricci d'integritat
                    v_resultat := '04: S''ha intentat esborrar un registre amb dependncies.' || SUBSTR(SQLERRM,INSTR(SQLERRM,'('));
            WHEN -20000 THEN
                    v_resultat := '03: Nivell no trobat amb ID = ' || p_id;
			ELSE
                   -- Error no controlat
                   v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
        END CASE;            
		
	-- S'assignem els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat;
        
	DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
		
END DEL_NIVELLS;
---------------------------------------------------------------------------------



---------------------------------------------------------------------------------
-- CU06-04: Consultar Nivell
-- Parmetres d'entrada:
--		* p_id: Identificador del Nivell
--		* p_joc_id: Identificador del Joc
--		* p_nom: Nom del Nivell
--		* p_segniv: Nom del Nivell
-- Parmetres de sortida: 
--		* p_rs: Recordset amb el resultat
--		* p_resultat: Resultat del procediment OK/KO
--
-- Si s'especifica l'identificador del Nivell es tornaran les dades del Nivell, si s'indica el nom es tornaran els jocs que continguin el nom especificat. 
-- Si no s'especifica res es retornaran tots els jocs.
---------------------------------------------------------------------------------
PROCEDURE CNS_NIVELLS ( p_rs IN OUT	sys_refcursor, 	p_id IN PFCBD.NIVELLS.id%TYPE, 	p_joc_id IN PFCBD.NIVELLS.JOCS_ID%TYPE, p_nom IN PFCBD.NIVELLS.NOM%TYPE, p_segniv IN PFCBD.NIVELLS.NIVSEG_ID%TYPE, p_resultat OUT VARCHAR)
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
	
	v_jocid PFCBD.NIVELLS.JOCS_ID%TYPE;
	v_id PFCBD.NIVELLS.id%TYPE;
	v_segniv PFCBD.NIVELLS.NIVSEG_ID%TYPE;
BEGIN
	
	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.CNS_NIVELLS';
	v_paramEnt       := 'p_id = ' || p_id 
                      || ', p_joc_id = ' || p_joc_id
					  || ', p_nom = ' || p_nom
					  || ', p_segniv = ' || p_segniv;
					  
	-- Es verifica que existeix el joc
	BEGIN		
		IF p_joc_id IS NOT NULL THEN
		  SELECT ID into v_jocid
				  FROM JOCS
				  WHERE ID = p_joc_id;
		END IF;
		EXCEPTION
		   WHEN OTHERS THEN
			   IF (SQLCODE = 100) THEN
					-- Joc no trobat
					RAISE_APPLICATION_ERROR(-20000,'');
				END IF; 
	END;   
  
	-- Es verifica que existeix el nivell
	BEGIN		
		IF p_id IS NOT NULL THEN
		  SELECT ID, JOCS_ID into v_id, v_jocid
				  FROM NIVELLS
				  WHERE ID = p_id;
		END IF;
		EXCEPTION
		   WHEN OTHERS THEN
			   IF (SQLCODE = 100) THEN
				   -- Nivell no trobat
			   RAISE_APPLICATION_ERROR(-20001,'');
			   END IF; 
	END; 
	
	-- Es verifica que existeix el segent nivell
	BEGIN
		IF p_segniv IS NOT NULL THEN
			SELECT ID into v_segniv
				FROM NIVELLS
				WHERE ID = p_segniv;
		END IF;
		EXCEPTION
				WHEN OTHERS THEN
					IF (SQLCODE = 100) THEN
						-- Nivell no trobat
						RAISE_APPLICATION_ERROR(-20002,'');
					END IF; 
	END;   
	
  
	-- Consulta per extreure els nivells d'un Joc
	IF v_id IS NULL AND p_nom is null AND v_jocid is not null THEN
		OPEN p_rs FOR
			SELECT LEVEL, jocs_id, id, NOM, nivseg_id
			FROM NIVELLS
			where jocs_id = v_jocid
			START WITH nom = (select NOM from NIVELLS WHERE id NOT IN (SELECT DISTINCT NIVSEG_ID FROM NIVELLS WHERE NIVSEG_ID is not null and jocs_id = v_jocid ) and jocs_id = v_jocid)
			CONNECT BY PRIOR nivseg_id = id; 
	
	-- Consulta per extreure les dades a partir d'un nivell, d'un Joc
	ELSIF v_id IS NOT NULL AND v_jocid IS NOT NULL THEN
		OPEN p_rs FOR
		    SELECT LEVEL, jocs_id, id, NOM, nivseg_id
			FROM NIVELLS
			where jocs_id = v_jocid
			START WITH nom = (select NOM from NIVELLS WHERE id = v_id ) 
			CONNECT BY PRIOR nivseg_id = id; 
	
	-- Consulta per extreure les dades dels nivells que contenen el nom indicat, d'un joc donat
	ELSIF p_nom IS NOT NULL and v_jocid is not null THEN
		OPEN p_rs FOR
		SELECT 0, jocs_id, id, NOM, nivseg_id
		FROM NIVELLS
		WHERE NOM LIKE '%' || upper(p_nom) || '%' and JOCS_ID = v_jocid;
		
	-- Consulta per extreure les dades dels nivells que contenen el nom indicat
	ELSIF p_nom IS NOT NULL THEN
		OPEN p_rs FOR
		SELECT 0, jocs_id, id, NOM, nivseg_id
		FROM NIVELLS
		WHERE NOM LIKE '%' || upper(p_nom) || '%';
	
	-- Consulta per extreure els nivells anteriors a un nivell donat
	ELSIF v_segniv IS NOT NULL THEN
		OPEN p_rs FOR
		SELECT 0, jocs_id, id, NOM, nivseg_id
		FROM NIVELLS
		WHERE NIVSEG_ID = v_segniv;
	ELSE
		RAISE_APPLICATION_ERROR(-20003,'');
	END IF;

  
  	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat || ', p_rs: Recordset amb el resultat de la consulta.';
	v_resultat := '00: Consulta realitzada correctament';
      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
	
  
	EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                    WHEN -20000 THEN
                            v_resultat := '03: Joc no trobat amb ID = ' || p_joc_id;
                    WHEN -20001 THEN
                            v_resultat := '03: Nivell no trobat amb ID = ' || p_id;
                    WHEN -20002 THEN
                            v_resultat := '03: Nivell no trobat amb ID = ' || p_segniv;
                    WHEN -20003 THEN
                            v_resultat := '04: Ha dinformar almenys un criteri de selecci.';
                    ELSE
                            v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
                END CASE;            

  	-- Es genera un recordset buit
        OPEN p_rs FOR
            SELECT * FROM DUAL WHERE DUMMY IS NULL;
				
				
	-- S'assignem els valors de sortida 
		p_resultat := 'KO';
		v_dataFi    := SYSDATE;
		v_paramSor	:= 'p_resultat: ' || p_resultat || ', p_rs: Recordset buit.';
        
		DBMS_OUTPUT.put_line (v_resultat);				
-- Gravem al LOG
		GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
				
END CNS_NIVELLS;
---------------------------------------------------------------------------------

END DM_NIVELLS;
/