---------------------------------------------------------------------------------
-- Alumne: Jessica Villamor 
-- Consultor: Juan Martnez Bolaos
-- Projecte: GESTIO DE VIDEOJOCS
---------------------------------------------------------------------------------
--
-- Ens situem a l'esquema de la base de dades de Gesti de Videojocs
--
ALTER SESSION SET CURRENT_SCHEMA = PFCBD;

-- Creaci de la definici del paquet
Create or replace
PACKAGE    PFCBD.DM_PRODUCTES AS


		PROCEDURE INS_PRODUCTES( p_joc_id IN PFCBD.PRODUCTES.JOCS_ID%TYPE, 
						p_nom IN PFCBD.PRODUCTES.NOM%TYPE, 
						p_tip IN PFCBD.PRODUCTES.TIPOLOGIA%TYPE, 
						p_preu IN PFCBD.PRODUCTES.PREU%TYPE, 
						p_dataBaixa IN PFCBD.PRODUCTES.DATABAIXA%TYPE, 
						p_resultat OUT VARCHAR);
						
		PROCEDURE UPD_PRODUCTES ( p_id IN PFCBD.PRODUCTES.ID%TYPE, 
						p_joc_id IN PFCBD.PRODUCTES.JOCS_ID%TYPE, 
						p_nom IN PFCBD.PRODUCTES.NOM%TYPE, 
						p_tip IN PFCBD.PRODUCTES.TIPOLOGIA%TYPE, 
						p_preu IN PFCBD.PRODUCTES.PREU%TYPE, 
						p_dataBaixa IN PFCBD.PRODUCTES.DATABAIXA%TYPE, 
						p_resultat OUT VARCHAR);

		PROCEDURE  BX_PRODUCTES (p_id IN PFCBD.PRODUCTES.id%TYPE, p_dataBaixa IN PFCBD.PRODUCTES.DATABAIXA%TYPE, p_resultat OUT VARCHAR);
						
		PROCEDURE  DEL_PRODUCTES (p_id IN PFCBD.PRODUCTES.id%TYPE, p_resultat OUT VARCHAR);
							
		PROCEDURE CNS_PRODUCTES (p_rs IN OUT sys_refcursor, 	
						p_id IN PFCBD.PRODUCTES.id%TYPE, 	
						p_joc_id IN PFCBD.PRODUCTES.JOCS_ID%TYPE, 
						p_nom IN PFCBD.PRODUCTES.NOM%TYPE, p_tipo IN PFCBD.PRODUCTES.TIPOLOGIA%TYPE,
						p_dataBaixa IN PFCBD.PRODUCTES.NOM%TYPE,
						p_resultat OUT VARCHAR);
END DM_PRODUCTES;
/


-- Creaci del cos del paquet
CREATE OR REPLACE PACKAGE BODY PFCBD.DM_PRODUCTES AS

---------------------------------------------------------------------------------
-- Procediments CU07: Gesti Dades Productes
---------------------------------------------------------------------------------

---------------------------------------------------------------------------------
-- CU07-01: Alta Producte
-- Parmetres d'entrada:
--		* p_joc_id: Identificador del Joc
--		* p_nom: Nom del Producte
--		* p_tip: Tipologia del Producte
--		* p_preu: Preu del Producte
--		* p_dataBaixa: Data de Baixa
-- Parmetres de sortida: 
--		* p_resultat: Resultat del procediment OK/KO
---------------------------------------------------------------------------------
PROCEDURE INS_PRODUCTES( p_joc_id IN PFCBD.PRODUCTES.JOCS_ID%TYPE, 
						p_nom IN PFCBD.PRODUCTES.NOM%TYPE, 
						p_tip IN PFCBD.PRODUCTES.TIPOLOGIA%TYPE, 
						p_preu IN PFCBD.PRODUCTES.PREU%TYPE, 
						p_dataBaixa IN PFCBD.PRODUCTES.DATABAIXA%TYPE, 
						p_resultat OUT VARCHAR)
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
	
	v_jocid PFCBD.PRODUCTES.JOCS_ID%TYPE;
  
BEGIN
	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.INS_PRODUCTES';
	v_paramEnt       := 'p_joc_id = ' || p_joc_id 
                      || ', p_nom = ' || p_nom
					  || ', p_tip = ' || p_tip
					  || ', p_preu = ' || p_preu
					  || ', p_dataBaixa = ' || p_dataBaixa;
	
-- Es verifica que el Joc existeix al sistema
	BEGIN
		IF p_joc_id is not NULL THEN
			SELECT ID into v_jocid
					FROM JOCS
				WHERE ID = p_joc_id;
		END IF;
		EXCEPTION
			WHEN OTHERS THEN
				IF (SQLCODE = 100) THEN
					-- Joc no trobat
					RAISE_APPLICATION_ERROR(-20000,'');
				END IF; 
	END;   
  
  	-- S'inserta el nou nivell		
	INSERT INTO PRODUCTES(JOCS_ID, NOM, TIPOLOGIA, PREU, DATABAIXA) VALUES (p_joc_id, UPPER(p_nom), p_tip, p_preu, p_dataBaixa);
	
	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat;
	v_resultat     := '00: Producte creat correctament';
      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
	

	EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE) 
                    WHEN -1 THEN
                        -- Registre duplicat
                        v_resultat := '01: El Producte ja existeix.';
                    WHEN -1400 THEN
                        -- Valor nulo
                        v_resultat := '02: S''ha intentat inserir un valor NULL' || SUBSTR(SQLERRM,INSTR(SQLERRM,'('));
					WHEN -20000 THEN
                        v_resultat := '03: Joc no trobat amb ID = ' || p_joc_id;
                    WHEN -2290 THEN
                        v_resultat := '05: Valor no perms al camp tipologia: ' || p_tip;
					ELSE
                        -- Error no controlado
                        v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
                END CASE;            

	-- S'assignem els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat;
        
	DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
				
END INS_PRODUCTES;
---------------------------------------------------------------------------------


---------------------------------------------------------------------------------
-- CU07-02: Actualitzar Producte
-- Parmetres d'entrada:
--		* p_id: Identificador del Producte
--		* p_joc_id: Identificador del Joc
--		* p_nom: Nom del Nivell
--		* p_tip: Tipologia del Producte
--		* p_preu: Preu del Producte
--		* p_dataBaixa: Data de Baixa
-- Parmetres de sortida: 
--		* p_resultat: Resultat del procediment OK/KO
---------------------------------------------------------------------------------
PROCEDURE UPD_PRODUCTES ( p_id IN PFCBD.PRODUCTES.ID%TYPE, 
						p_joc_id IN PFCBD.PRODUCTES.JOCS_ID%TYPE, 
						p_nom IN PFCBD.PRODUCTES.NOM%TYPE, 
						p_tip IN PFCBD.PRODUCTES.TIPOLOGIA%TYPE, 
						p_preu IN PFCBD.PRODUCTES.PREU%TYPE, 
						p_dataBaixa IN PFCBD.PRODUCTES.DATABAIXA%TYPE, 
						p_resultat OUT VARCHAR)
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
	
	v_jocid PFCBD.PRODUCTES.JOCS_ID%TYPE;
 
BEGIN

	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.UPD_PRODUCTES';
	v_paramEnt       := 'p_id = ' || p_id 
                      || ', p_joc_id = ' || p_joc_id
					  || ', p_nom = ' || p_nom
					  || ', p_tip = ' || p_tip
					  || ', p_preu = ' || p_preu
					  || ', p_dataBaixa = ' || p_dataBaixa;
					  

	-- Es verifica que el Joc existeix al sistema
	BEGIN
  
	   IF p_joc_id IS NOT NULL THEN
			SELECT ID into v_jocid
					  FROM JOCS
					  WHERE ID = p_joc_id;
		END IF;
		EXCEPTION
			WHEN OTHERS THEN
				 IF (SQLCODE = 100) THEN
					   -- Joc no trobat
					   RAISE_APPLICATION_ERROR(-20000,'');
				 END IF; 
	END;  
    
	UPDATE PRODUCTES SET JOCS_ID = p_joc_id, NOM = upper(p_nom), TIPOLOGIA = p_tip, PREU = p_preu, DATABAIXA = p_dataBaixa WHERE PRODUCTES.ID = p_id;
    
	IF SQL%rowcount = 0 THEN
		RAISE_APPLICATION_ERROR(-20001,'');
	ELSE
		v_resultat := '00: Producte Actualitzat Correctament';
	END IF;
  
	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat;
	      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
  
 
	EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                   WHEN -1407 THEN
                        -- Valor nulo
                        v_resultat := '02: S''ha intentat inserir un valor NULL.' || SUBSTR(SQLERRM,INSTR(SQLERRM,'('));
                    WHEN -20000 THEN
                        v_resultat := '03: Joc no trobat amb ID = ' || p_joc_id;
					WHEN -20001 THEN
                        v_resultat := '03: Producte no trobat amb ID = ' || p_id;
					WHEN -2290 THEN
                        v_resultat := '05: Valor no perms al camp tipologia: ' || p_tip;
					ELSE
                        -- Error no controlat
                        v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
                END CASE;            
  
  -- S'assignem els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat;
        
	DBMS_OUTPUT.put_line (v_resultat);				
-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
				
END UPD_PRODUCTES;
---------------------------------------------------------------------------------


---------------------------------------------------------------------------------
-- CU07-03: Baixa lgica Producte
-- Parmetres d'entrada:
--		* p_id: Identificador del Producte
-- 		* p_dataBaixa: Data de baixa
-- Parmetres de sortida: 
--		* p_resultat: Resultat del procediment OK/KO
--
-- Si no s'informa cap data s'agafar la data del dia.
---------------------------------------------------------------------------------
PROCEDURE  BX_PRODUCTES (p_id IN PFCBD.PRODUCTES.id%TYPE, p_dataBaixa IN PFCBD.PRODUCTES.DATABAIXA%TYPE, p_resultat OUT VARCHAR)
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
BEGIN
	
	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.BX_PRODUCTES';
	v_paramEnt       := 'p_id = ' || p_id 
                      || ', p_dataBaixa = ' || p_dataBaixa;
					  
	-- Inici de l'execuci del procediment
	UPDATE PRODUCTES SET DATABAIXA = (NVL(p_dataBaixa, SYSDATE))
	WHERE id = p_id;

	-- Comprovem si s'ha trobat el registre a esborrar
	IF SQL%rowcount = 0
	THEN
		RAISE_APPLICATION_ERROR(-20000,'');
	ELSE
		v_resultat := '00: Baixa realitzada correctament';
	END IF;

	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat;
	      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
  
	EXCEPTION
						
	WHEN OTHERS THEN
	 CASE (SQLCODE)
            WHEN -20000 THEN
                v_resultat := '03: Producte no trobat amb ID = ' || p_id;
			ELSE
				-- Error no controlat
				v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
		END CASE;
    
	-- S'assignem els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat;
        
	DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
		
END BX_PRODUCTES;
---------------------------------------------------------------------------------


---------------------------------------------------------------------------------
-- CU07-04: Esborrar Producte
-- Parm,etres d'entrada:
--		* p_id: Identificador del Nivell
-- Parmetres de sortida: 
--		* p_resultat: Resultat del procediment OK/KO
---------------------------------------------------------------------------------
PROCEDURE  DEL_PRODUCTES (p_id IN PFCBD.PRODUCTES.id%TYPE, p_resultat OUT VARCHAR)
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
BEGIN
	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.DEL_PRODUCTES';
	v_paramEnt       := 'p_id = ' || p_id;
	
	
	-- Inici de l'execuci del procediment
	DELETE PRODUCTES
	WHERE id = p_id;

	-- Comprovem si s'ha trobat el registre a esborrar
	IF SQL%rowcount = 0 THEN
		RAISE_APPLICATION_ERROR(-20000,'');
	ELSE
		v_resultat := '00: Producte esborrat correctament';
	END IF;

	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat;
      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
  
	EXCEPTION
	WHEN OTHERS THEN
		CASE (SQLCODE)
            WHEN -20000 THEN
                v_resultat := '03: Producte no trobat amb ID = ' || p_id;
				
			WHEN -2292 THEN
				-- Restricci d'integritat
                   v_resultat := '04: S''ha intentar esborrar un registre amb dependncies.' || SUBSTR(SQLERRM,INSTR(SQLERRM,'('));
            ELSE
                -- Error no controlat
                   v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
        END CASE;            
		
	-- S'assignem els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat;
        
	DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
		
END DEL_PRODUCTES;
---------------------------------------------------------------------------------


---------------------------------------------------------------------------------
-- CU07-05: Consultar Productes
-- Parmetres d'entrada:
--		* p_id: Identificador del Producte
--		* p_joc_id: Identificador del Joc
--		* p_nom: Nom del Producte
--		* p_tipo: Tipus de Producte
--		* p_dataBaixa: Data de baixa del producte
-- Parmetres de sortida:
--		* p_rs: Recordset amb el resultat
--		* p_resultat: Resultat del procediment OK/KO
-- Si s'informa el codi del producte cercat, es retornar com a resultat la informaci d'aquest producte. 
-- Si no s'informa l'identificador del producte a cercar, es crear una consulta dinmica amb tots els parmetres d'entrada introduts permetent aix realitzar consultes filtrant per quasi tots els 
-- camps de la taula, donant llibertat controlada d's als desenvolupadors de l'aplicatiu.
-- Si no s'informa cap parmetre d'entrada, com la taula mestre de productes no haur de ser molt gran, s'extreuran les dades de tots els productes del sistema.
---------------------------------------------------------------------------------
PROCEDURE CNS_PRODUCTES (p_rs IN OUT sys_refcursor, 	
						p_id IN PFCBD.PRODUCTES.id%TYPE, 	
						p_joc_id IN PFCBD.PRODUCTES.JOCS_ID%TYPE, 
						p_nom IN PFCBD.PRODUCTES.NOM%TYPE, 
						p_tipo IN PFCBD.PRODUCTES.TIPOLOGIA%TYPE,
						p_dataBaixa IN PFCBD.PRODUCTES.NOM%TYPE,
						p_resultat OUT VARCHAR)
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
	
	v_jocid PFCBD.PRODUCTES.JOCS_ID%TYPE;
	v_cerca VARCHAR(1000);
	v_id PFCBD.PRODUCTES.id%TYPE;

BEGIN
	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.CNS_PRODUCTES';
	v_paramEnt       := 'p_id = ' || p_id 
                      || ', p_joc_id = ' || p_joc_id
					  || ', p_nom = ' || p_nom
					  || ', p_dataBaixa = ' || p_dataBaixa;
	

	if p_joc_id is not NULL then
		v_cerca := 'SELECT * FROM PRODUCTES WHERE JOCS_ID = ' || p_joc_id;
	end if;
 
	if p_nom is not NULL then
		IF v_cerca IS NULL THEN
			v_cerca := 'SELECT * FROM PRODUCTES WHERE NOM LIKE ''%' || UPPER(p_nom) || '%''';
		ELSE
			v_cerca := v_cerca || ' AND NOM LIKE ''%' || UPPER(p_nom) || '%''';
		END IF;
	end if;
  
  if p_tipo is not NULL then
		IF v_cerca IS NULL THEN
			v_cerca := 'SELECT * FROM PRODUCTES WHERE TIPOLOGIA = ''' || p_tipo || '''';
		ELSE
			v_cerca := v_cerca || ' AND TIPOLOGIA = ''' || p_tipo || '''';
		END IF;
	end if;
  
	if p_dataBaixa IS NOT NULL THEN
		IF v_cerca IS NULL THEN
			v_cerca := 'SELECT * FROM PRODUCTES WHERE DATABAIXA IS NULL OR DATABAIXA < ''' || p_dataBaixa || '''';
		ELSE
			v_cerca := v_cerca || ' AND (DATABAIXA IS NULL OR DATABAIXA < ''' || p_dataBaixa || ''')';
		END IF;
	end if;
 
 	-- Consulta per extreure les dades d'un Joc
	IF p_id IS NOT NULL THEN
			OPEN p_rs FOR
			SELECT id, JOCS_ID, NOM, TIPOLOGIA, PREU, DATABAIXA
			FROM PRODUCTES
			where id = p_id;
			
	ELSIF v_cerca IS NOT NULL THEN
			OPEN p_rs FOR v_cerca;
	ELSE

	-- S'extreuen tots els productes		 
      OPEN p_rs FOR
			SELECT *
			FROM PRODUCTES;
	
	END IF;
			
	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat || ', p_rs: Recordset amb la consulta.';
	v_resultat := '00: Consulta realitzada correctament';
      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
	
	     
	EXCEPTION
            WHEN OTHERS THEN
                v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
                
  	   	 -- Es genera un recordset buit
        OPEN p_rs FOR
            SELECT * FROM DUAL WHERE DUMMY IS NULL;
				
		-- S'assignem els valors de sortida 
		p_resultat := 'KO';
		v_dataFi    := SYSDATE;
		v_paramSor	:= 'p_resultat: ' || p_resultat || ', p_rs: Recordset buit.';
			
		DBMS_OUTPUT.put_line (v_resultat);				
		-- Gravem al LOG
		GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
				
END CNS_PRODUCTES;
---------------------------------------------------------------------------------

END DM_PRODUCTES;
/
