---------------------------------------------------------------------------------
-- Alumne: Jessica Villamor 
-- Consultor: Juan Martnez Bolaos
-- Projecte: GESTIO DE VIDEOJOCS
---------------------------------------------------------------------------------

-- Procediments CU08: Gesti de les plataformes del Joc

--
-- Ens situem a l'esquema de la base de dades de Gesti de Videojocs
--
-- Creaci de la definici del paquet
ALTER SESSION SET CURRENT_SCHEMA = PFCBD;


Create or replace
PACKAGE    PFCBD.PLATAF_JOCS AS

	PROCEDURE INS_PLATAFORMES_JOC( p_joc_id IN PFCBD.PLATAFORMES_JOC.JOCS_ID%TYPE, 
								p_plat_id IN PFCBD.PLATAFORMES_JOC.PLATAFORMES_ID%TYPE, 
								p_resultat OUT VARCHAR);
								
	PROCEDURE DEL_PLATAFORMES_JOC( p_joc_id IN PFCBD.PLATAFORMES_JOC.JOCS_ID%TYPE, p_plat_id IN PFCBD.PLATAFORMES_JOC.PLATAFORMES_ID%TYPE, p_resultat OUT VARCHAR);							
								
	PROCEDURE CNS_PLATAFORMES_JOC(p_rs IN OUT sys_refcursor, p_joc_id IN PFCBD.PLATAFORMES_JOC.JOCS_ID%TYPE, p_resultat OUT VARCHAR);

END PLATAF_JOCS;
/


-- Creaci del cos del paquet
CREATE OR REPLACE PACKAGE BODY PFCBD.PLATAF_JOCS AS

---------------------------------------------------------------------------------
-- Procediments CU08: Gesti d'Assignaci Plataformes/Joc
---------------------------------------------------------------------------------


---------------------------------------------------------------------------------
-- CU08-01: Assignar Plataformes Joc
-- Parmetres d'entrada:
--		* p_joc_id: Identificador del Joc
--		* p_plat_id: Identificador de la Plataforma
-- Parmetres de sortida: 
--		* p_resultat: Resultat del procediment OK/KO
---------------------------------------------------------------------------------
PROCEDURE INS_PLATAFORMES_JOC( p_joc_id IN PFCBD.PLATAFORMES_JOC.JOCS_ID%TYPE, 
								p_plat_id IN PFCBD.PLATAFORMES_JOC.PLATAFORMES_ID%TYPE, 
								p_resultat OUT VARCHAR)
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
	
	v_jocid PFCBD.PLATAFORMES_JOC.JOCS_ID%TYPE;
	v_platid PFCBD.PLATAFORMES_JOC.PLATAFORMES_ID%TYPE;
  
BEGIN
	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.ASG_PLATAFORMES_JOC';
	v_paramEnt       := 'p_joc_id = ' || p_joc_id 
                      || ', p_plat_id = ' || p_plat_id;

	-- Es verifica que el Joc existeix al sistema
	BEGIN
	  IF p_joc_id is not NULL THEN
		SELECT ID into v_jocid
				FROM JOCS
				WHERE ID = p_joc_id;
	  END IF;
		EXCEPTION
			WHEN OTHERS THEN
				IF (SQLCODE = 100) THEN
					-- Joc no trobat
					RAISE_APPLICATION_ERROR(-20000,'');
				END IF; 
	END;   
  
	-- Es verifica que la Plataforma existeix al sistema
	BEGIN
	  IF p_plat_id is not NULL THEN
		SELECT ID into v_platid
				FROM PLATAFORMES
				WHERE ID = p_plat_id;
	  END IF;
		EXCEPTION
			WHEN OTHERS THEN
				IF (SQLCODE = 100) THEN
					-- Plataforma no trobada
					RAISE_APPLICATION_ERROR(-20001,'');
				END IF; 
	END;   
  
	-- S'assigna la nova plataforma al Joc	
	INSERT INTO PLATAFORMES_JOC(PLATAFORMES_ID, JOCS_ID) VALUES (v_platid, p_joc_id);

	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat;
	v_resultat     := '00: Assignaci realitzada correctament';
      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
	

	EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE) 
                    WHEN -1 THEN
                        -- Registre duplicat
                        v_resultat := '01: La plataforma ja havia estat assignada.';
                    WHEN -1400 THEN
                        -- Valor nulo
                        v_resultat := '02: S''ha intentat inserir un valor NULL' || SUBSTR(SQLERRM,INSTR(SQLERRM,'('));
                    WHEN -20000 THEN
                        v_resultat := '03: Joc no trobat amb ID = ' || p_joc_id;
					WHEN -20001 THEN
                        v_resultat := '03: Plataforma no trobada amb ID = ' || p_plat_id;
					ELSE
                        -- Error no controlat
                        v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
                END CASE;            

	-- S'assignem els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat;
        
	DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
				
END INS_PLATAFORMES_JOC;
---------------------------------------------------------------------------------


---------------------------------------------------------------------------------
-- CU08-02: Desassignar Plataformes Joc
-- Parmetres d'entrada:
--		* p_joc_id: Identificador del Joc
--		* p_plat_id: Identificador de la Plataforma
-- Parmetres de sortida: 
--		* p_resultat: Resultat del procediment OK/KO
---------------------------------------------------------------------------------
PROCEDURE DEL_PLATAFORMES_JOC( p_joc_id IN PFCBD.PLATAFORMES_JOC.JOCS_ID%TYPE, p_plat_id IN PFCBD.PLATAFORMES_JOC.PLATAFORMES_ID%TYPE, p_resultat OUT VARCHAR)
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
	
	v_jocid PFCBD.PLATAFORMES_JOC.JOCS_ID%TYPE;
	v_platid PFCBD.PLATAFORMES_JOC.PLATAFORMES_ID%TYPE;
  
BEGIN
	
	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.DES_PLATAFORMES_JOC';
	v_paramEnt       := 'p_joc_id = ' || p_joc_id 
                      || ', p_plat_id = ' || p_plat_id;
					  
	-- Es verifica que el Joc existeix al sistema
	BEGIN
	  IF p_joc_id is not NULL THEN
		SELECT ID into v_jocid
				FROM JOCS
				WHERE ID = p_joc_id;
	  END IF;
		EXCEPTION
			WHEN OTHERS THEN
				IF (SQLCODE = 100) THEN
					-- Joc no trobat
					RAISE_APPLICATION_ERROR(-20000,'');
				END IF; 
	END;   
  
	-- Es verifica que la Plataforma existeix al sistema  
	BEGIN
	  IF p_plat_id is not NULL THEN
		SELECT ID into v_platid
				FROM PLATAFORMES
				WHERE ID = p_plat_id;
	  END IF;
		EXCEPTION
			WHEN OTHERS THEN
				IF (SQLCODE = 100) THEN
					-- Plataforma no trobada
					RAISE_APPLICATION_ERROR(-20001,'');
				END IF; 
	END;   
  
  
  IF v_jocid IS NOT NULL AND v_platid IS NOT NULL THEN
		-- Desassigna la plataforma al Joc	
		DELETE FROM PLATAFORMES_JOC WHERE PLATAFORMES_ID = v_platid and JOCS_ID =  p_joc_id;
	   
	   -- Comprovem si s'ha trobat el registre a esborrar
		IF SQL%rowcount = 0 THEN
			 RAISE_APPLICATION_ERROR(-20003,'');
		ELSE
			v_resultat := '00: Desassignaci realitzada correctament';
		END IF;

   ELSE
	RAISE_APPLICATION_ERROR(-20002,'');
   END IF;
	
		 
	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat;
      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	

EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE) 
                    WHEN -20000 THEN
                        v_resultat := '03: Joc no trobat amb ID = ' || p_joc_id;
					WHEN -20001 THEN
                        v_resultat := '03: Plataforma no trobada amb ID = ' || p_plat_id;
					WHEN -20002 THEN
                        v_resultat := '04: Per realitzar la desassignaci ha d''informar els identificadors de la plataforma i el joc';
					WHEN -20003 THEN
                        v_resultat := '01: Assignaci a desfer no trobada, plataforma_id ' || v_platid || ' joc_id = ' || v_jocid;
					ELSE
                        -- Error no controlat
                        v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
                END CASE;            

	-- S'assignem els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat;
        
	DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
				
END DEL_PLATAFORMES_JOC;
---------------------------------------------------------------------------------


---------------------------------------------------------------------------------
-- CU08-03: Consultar Plataformes Joc
-- Parmetres d'entrada:
--		* p_joc_id: Identificador del Joc
-- Parmetres de sortida: 
--		* p_rs: Recordset amb la informaci
--		* p_resultat: Resultat del procediment OK/KO
---------------------------------------------------------------------------------
PROCEDURE CNS_PLATAFORMES_JOC(p_rs IN OUT sys_refcursor, p_joc_id IN PFCBD.PLATAFORMES_JOC.JOCS_ID%TYPE, p_resultat OUT VARCHAR)
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
		
	v_jocid PFCBD.PLATAFORMES_JOC.JOCS_ID%TYPE;
BEGIN
	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.CNS_PLATAFORMES_JOC';
	v_paramEnt       := 'p_joc_id = ' || p_joc_id;
	
-- Es verifica que el Joc existeix al sistema
	BEGIN
  	  IF p_joc_id is not NULL THEN
		SELECT ID into v_jocid
				FROM JOCS
				WHERE ID = p_joc_id;
	  END IF;
		EXCEPTION
			WHEN OTHERS THEN
				IF (SQLCODE = 100) THEN
					-- Joc no trobat
					RAISE_APPLICATION_ERROR(-20000,'');
				END IF; 
	END;   
  
	if v_jocid is null then
		  RAISE_APPLICATION_ERROR(-20001,'');
	end if;
  
  open p_rs for
	SELECT * from PLATAFORMES_JOC WHERE JOCS_ID = v_jocid;

	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat || ', p_rs: Recordset amb el resultat de la consulta.';
	v_resultat     := '00: Consulta realitzada correctament';
      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	

EXCEPTION
    WHEN OTHERS THEN
          CASE (SQLCODE) 
                WHEN -20000 THEN
                        v_resultat := '03: Joc no trobat amb ID = ' || p_joc_id;
          		WHEN -20001 THEN
                        v_resultat := '04: Ha d''introduir el codi del Joc.';
                ELSE
                        -- Error no controlat
                        v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
                END CASE;            

-- Es genera un recordset buit
        OPEN p_rs FOR
            SELECT * FROM DUAL WHERE DUMMY IS NULL;
				
				
-- S'assignem els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat || ', p_rs: Recordset amb el resultat de la consulta.';
        
	DBMS_OUTPUT.put_line (v_resultat);				
-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
				
END CNS_PLATAFORMES_JOC; 
---------------------------------------------------------------------------------

END PLATAF_JOCS;
/
