---------------------------------------------------------------------------------
-- Alumne: Jessica Villamor 
-- Consultor: Juan Martnez Bolaos
-- Projecte: GESTIO DE VIDEOJOCS
---------------------------------------------------------------------------------

-- Procediments CU09: Gesti dels navegadors del joc


--
-- Ens situem a l'esquema de la base de dades de Gesti de Videojocs
--

ALTER SESSION SET CURRENT_SCHEMA = PFCBD;

-- Creaci de la definici del paquet
Create or replace
PACKAGE    PFCBD.NAVEG_JOCS AS

	PROCEDURE INS_JOC_NAVEGADORS( p_joc_id IN PFCBD.JOC_NAVEGADORS.JOCS_ID%TYPE, p_nav_id IN PFCBD.JOC_NAVEGADORS.NAVEGADORS_ID%TYPE, p_resultat OUT VARCHAR);

	PROCEDURE DEL_JOC_NAVEGADORS( p_joc_id IN PFCBD.JOC_NAVEGADORS.JOCS_ID%TYPE, p_nav_id IN PFCBD.JOC_NAVEGADORS.NAVEGADORS_ID%TYPE, p_resultat OUT VARCHAR);

	PROCEDURE CNS_JOC_NAVEGADORS(p_rs IN OUT sys_refcursor, p_joc_id IN PFCBD.JOC_NAVEGADORS.JOCS_ID%TYPE, p_resultat OUT VARCHAR);

END NAVEG_JOCS;
/


-- Creaci del cos del paquet
CREATE OR REPLACE PACKAGE BODY PFCBD.NAVEG_JOCS AS

---------------------------------------------------------------------------------
-- Procediments CU09: Gesti d'Assignaci Navegadors/Joc
---------------------------------------------------------------------------------

---------------------------------------------------------------------------------
-- CU09-01: Assignar Navegadors Joc
-- Parmetres d'entrada:
--		* p_joc_id: Identificador del Joc
--		* p_nav_id: Identificador del Navegador
-- Parmetres de sortida: 
--		* p_resultat: Resultat del procediment OK/KO
---------------------------------------------------------------------------------
PROCEDURE INS_JOC_NAVEGADORS( p_joc_id IN PFCBD.JOC_NAVEGADORS.JOCS_ID%TYPE, p_nav_id IN PFCBD.JOC_NAVEGADORS.NAVEGADORS_ID%TYPE, p_resultat OUT VARCHAR)
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
	
	v_jocid PFCBD.JOC_NAVEGADORS.JOCS_ID%TYPE;
	v_navid PFCBD.JOC_NAVEGADORS.NAVEGADORS_ID%TYPE;
  
BEGIN
	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.ASG_JOC_NAVEGADORS';
	v_paramEnt       := 'p_joc_id = ' || p_joc_id 
                      || ', p_nav_id = ' || p_nav_id;

	-- Es verifica que el Joc existeix al sistema
	BEGIN
	  IF p_joc_id is not NULL THEN
		SELECT ID into v_jocid
				FROM JOCS
				WHERE ID = p_joc_id;
	  END IF;
		EXCEPTION
			WHEN OTHERS THEN
				IF (SQLCODE = 100) THEN
					-- Joc no trobat
					RAISE_APPLICATION_ERROR(-20000,'');
				END IF; 
	END;   
  
	-- Es verifica que el Navegador existeix al sistema
	BEGIN

	  IF p_nav_id is not NULL THEN
		SELECT ID into v_navid
				FROM NAVEGADORS
				WHERE ID = p_nav_id;
	  END IF;
		EXCEPTION
			WHEN OTHERS THEN
				IF (SQLCODE = 100) THEN
					-- Navegador no trobat
					RAISE_APPLICATION_ERROR(-20001,'');
				END IF; 
	eND;   
  
	-- S'assigna el nou Navegador al Joc
	INSERT INTO JOC_NAVEGADORS(NAVEGADORS_ID, JOCS_ID) VALUES (v_navid, p_joc_id);

	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat;
	v_resultat     := '00: Assignaci realitzada correctament';
      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	

EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE) 
                    WHEN -1 THEN
                        -- Registre duplicat
                        v_resultat := '01: El navegador ja havia estat assignat.';
                    WHEN -1400 THEN
                        -- Valor nulo
                        v_resultat := '02: S''ha intentat inserir un valor NULL' || SUBSTR(SQLERRM,INSTR(SQLERRM,'('));
                    WHEN -20000 THEN
                        v_resultat := '03: Joc no trobat amb ID = ' || p_joc_id;
					WHEN -20001 THEN
                        v_resultat := '03: Navegador no trobat amb ID = ' || p_nav_id;
					ELSE
                        -- Error no controlat
                        v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
                END CASE;            

					
	-- S'assignem els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat;
        
	DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);
				
END INS_JOC_NAVEGADORS;
---------------------------------------------------------------------------------


---------------------------------------------------------------------------------
-- CU09-02: Desassignar Navegadors Joc
-- Parmetres d'entrada:
--		* p_joc_id: Identificador del Joc
--		* p_nav_id: Identificador del Navegador
-- Parmetres de sortida: 
--		* p_resultat: Resultat del procediment OK/KO
--
-- Procediment per desassignar un navegador d'un Joc.
---------------------------------------------------------------------------------
PROCEDURE DEL_JOC_NAVEGADORS( p_joc_id IN PFCBD.JOC_NAVEGADORS.JOCS_ID%TYPE, p_nav_id IN PFCBD.JOC_NAVEGADORS.NAVEGADORS_ID%TYPE, p_resultat OUT VARCHAR)
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
	
	v_jocid PFCBD.JOC_NAVEGADORS.JOCS_ID%TYPE;
	v_navid PFCBD.JOC_NAVEGADORS.NAVEGADORS_ID%TYPE;
BEGIN
	
	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.DES_JOC_NAVEGADORS';
	v_paramEnt       := 'p_joc_id = ' || p_joc_id 
                      || ', p_nav_id = ' || p_nav_id;

					  
	-- Es verifica que el Joc existeix al sistema
	BEGIN
	  IF p_joc_id is not NULL THEN
		SELECT ID into v_jocid
				FROM JOCS
				WHERE ID = p_joc_id;
	  END IF;
		EXCEPTION
			WHEN OTHERS THEN
				IF (SQLCODE = 100) THEN
					-- Joc no trobat
					RAISE_APPLICATION_ERROR(-20000,'');
				END IF; 
	END;   
  
	-- Es verifica que el Navegador existeix al sistema
	BEGIN
	  IF p_nav_id is not NULL THEN
		SELECT ID into v_navid
				FROM PLATAFORMES
				WHERE ID = p_nav_id;
	  END IF;
		EXCEPTION
			WHEN OTHERS THEN
				IF (SQLCODE = 100) THEN
					-- Navegador no trobat
					RAISE_APPLICATION_ERROR(-20001,'');
				END IF; 
	END;   
  
  
	  IF v_jocid IS NOT NULL AND v_navid IS NOT NULL THEN
			-- Desassigna la plataforma al Joc	
			DELETE FROM JOC_NAVEGADORS WHERE NAVEGADORS_ID = v_navid and JOCS_ID =  v_jocid;
		   
			-- Comprovem si s'ha trobat el registre a esborrar
			IF SQL%rowcount = 0 THEN
				RAISE_APPLICATION_ERROR(-20003,'');
			ELSE
				v_resultat := '00: Desassignaci realitzada correctament';
			END IF;

	   ELSE
		RAISE_APPLICATION_ERROR(-20002,'');
	  END IF;
	
	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat;
	      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
	   

EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE) 
                    WHEN -20000 THEN
                        v_resultat := '03: Joc no trobat amb ID = ' || p_joc_id;
					WHEN -20001 THEN
                        v_resultat := '03: Navegador no trobat amb ID = ' || p_nav_id;
					
					WHEN -20002 THEN
                        v_resultat := '04: Per realitzar la desassignaci ha d''informar els identificadors del navegador i el joc';
					WHEN -20003 THEN
                        v_resultat := '01: Assignaci a desfer no trobada, navegadors_id ' || v_navid || ' joc_id = ' || v_jocid;
				
					ELSE
                        -- Error no controlat
                        v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
                END CASE;            

	-- S'assignem els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat;
        
	DBMS_OUTPUT.put_line (v_resultat);				
-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
				
END DEL_JOC_NAVEGADORS;
---------------------------------------------------------------------------------


---------------------------------------------------------------------------------
-- CU09-03: Consultar Navegadors Joc
-- Parmetres d'entrada:
--		* p_joc_id: Identificador del Joc
-- Parmetres de sortida: 
--		* p_rs: Recordset amb la informaci
--		* p_resultat: Resultat del procediment OK/KO
---------------------------------------------------------------------------------
PROCEDURE CNS_JOC_NAVEGADORS(p_rs IN OUT sys_refcursor, p_joc_id IN PFCBD.JOC_NAVEGADORS.JOCS_ID%TYPE, p_resultat OUT VARCHAR)
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
	
	v_jocid PFCBD.JOC_NAVEGADORS.JOCS_ID%TYPE;
BEGIN
	
	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.CNS_JOC_NAVEGADORS';
	v_paramEnt       := 'p_joc_id = ' || p_joc_id;
	
	-- Es verifica que el Joc existeix al sistema
	BEGIN

	  IF p_joc_id is not NULL THEN
		SELECT ID into v_jocid
				FROM JOCS
				WHERE ID = p_joc_id;
	  END IF;
		EXCEPTION
			WHEN OTHERS THEN
				IF (SQLCODE = 100) THEN
					-- Joc no trobat
					RAISE_APPLICATION_ERROR(-20000,'');
				END IF; 
	END;   
  
	  if v_jocid is null then
		  RAISE_APPLICATION_ERROR(-20001,'');
	  end if;
  
	open p_rs for
	SELECT * from JOC_NAVEGADORS WHERE JOCS_ID = v_jocid;

	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat || ', p_rs: Recordset amb el resultat de la consulta.';
	v_resultat     := '00: Consulta realitzada correctament';
      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
	

EXCEPTION
    WHEN OTHERS THEN
          CASE (SQLCODE) 
                WHEN -20000 THEN
                        v_resultat := '03: Joc no trobat amb ID = ' || p_joc_id;
          			WHEN -20001 THEN
                        v_resultat := '04: Ha d''introduir el codi del Joc.';
                ELSE
                        -- Error no controlat
                        v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
                END CASE;            

	-- Es genera un recordset buit
    OPEN p_rs FOR
    SELECT * FROM DUAL WHERE DUMMY IS NULL;
								
	-- S'assignem els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat || ', p_rs: Recordset buit.';
	
	DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
				
END CNS_JOC_NAVEGADORS;
---------------------------------------------------------------------------------

END NAVEG_JOCS;
/
