---------------------------------------------------------------------------------
-- Alumne: Jessica Villamor 
-- Consultor: Juan Martnez Bolaos
-- Projecte: GESTIO DE VIDEOJOCS
---------------------------------------------------------------------------------

-- Procediments CU10: Consultar dades per valorar la penetraci del Joc

--
-- Ens situem a l'esquema de la base de dades de Gesti de Videojocs
--
ALTER SESSION SET CURRENT_SCHEMA = PFCBD;

-- Creaci de la definici del paquet
Create or replace
PACKAGE    PFCBD.EST_PENETRACIOJOC AS

	PROCEDURE CNS_TJUGJOC_XZONA(p_rs IN OUT sys_refcursor, p_joc_id IN PFCBD.JOC_NAVEGADORS.JOCS_ID%TYPE, p_resultat OUT VARCHAR);

	PROCEDURE CNS_TJUGJOC_XSEXE(p_rs IN OUT sys_refcursor, p_joc_id IN PFCBD.JOC_NAVEGADORS.JOCS_ID%TYPE, p_resultat OUT VARCHAR);

	PROCEDURE CNS_TJUGJOC_XXARSOC(p_rs IN OUT sys_refcursor, p_joc_id IN PFCBD.JOC_NAVEGADORS.JOCS_ID%TYPE, p_resultat OUT VARCHAR);

	PROCEDURE CNS_TCOL_MES(p_rs IN OUT sys_refcursor, p_joc_id IN PFCBD.JOC_NAVEGADORS.JOCS_ID%TYPE, p_resultat OUT VARCHAR);

	PROCEDURE CNS_TCOM_MES(p_rs IN OUT sys_refcursor, p_joc_id IN PFCBD.JOC_NAVEGADORS.JOCS_ID%TYPE, p_resultat OUT VARCHAR);

	PROCEDURE CNS_TJUGJOC_PLAT_NAV(p_rs IN OUT sys_refcursor, p_joc_id IN PFCBD.JOC_NAVEGADORS.JOCS_ID%TYPE, p_resultat OUT VARCHAR);

END EST_PENETRACIOJOC;
/


-- Creaci del cos del paquet
CREATE OR REPLACE PACKAGE BODY PFCBD.EST_PENETRACIOJOC AS

---------------------------------------------------------------------------------
-- Procediments CU10: Consultes per valorar el grau de penetraci del joc
---------------------------------------------------------------------------------


---------------------------------------------------------------------------------
-- CU10-01: Consulta Totals de Jugadors per Zona Geogrfica
-- Parmetres d'entrada:
--		* p_joc_id: Identificador del Joc
-- Parmetres de sortida: 
--		* p_rs: Recordset amb la Informaci
--		* p_resultat: Resultat del procediment OK/KO
--
-- El procediment realitzar una consulta extraient de forma agrupada el nmero total de jugadors del joc per zona geogrfica.
-- S'extreuran 3 tipus de registres 'PAI', 'PRO' i 'POB' per a que sigui qui utilitza aquest procediment el que discrimini quin tipus d'agrupaci vol. 
-- Es podr saber la distribuci per pas, provincia i poblaci, ja que no tots els valors sn obligatoris en el nostre model de dades.
---------------------------------------------------------------------------------
PROCEDURE CNS_TJUGJOC_XZONA(p_rs IN OUT sys_refcursor, p_joc_id IN PFCBD.JOC_NAVEGADORS.JOCS_ID%TYPE, p_resultat OUT VARCHAR)
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
	
	v_jocid PFCBD.JOC_NAVEGADORS.JOCS_ID%TYPE;
BEGIN
	
	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.CNS_TJUGJOC_XZONA';
	v_paramEnt       := 'p_joc_id = ' || p_joc_id;
			
	-- Es verifiquen els parmetres d'entrada obligatoris  
	if p_joc_id is null then
		  RAISE_APPLICATION_ERROR(-20000,'');
	end if;
  
  
	 -- Es verifica que el Joc existeix al sistema 
	  BEGIN
	  IF p_joc_id is not NULL THEN
		SELECT ID into v_jocid
				FROM JOCS
				WHERE ID = p_joc_id;
	  END IF;
		EXCEPTION
			WHEN OTHERS THEN
				IF (SQLCODE = 100) THEN
					-- Joc no trobat
					RAISE_APPLICATION_ERROR(-20001,'');
				END IF; 
	  END;   
  
	  open p_rs for
			select 'PAI' as Filtre, p.CODPAIS, null as CODPROV, null as CODPOB, p.nom , count(*) as Total 
				from jugadors J, paisos p 
				where j.codpais = p.codpais 
					  and j.ID IN (select distinct JU.ID
										   from Jugadors JU, 
												jocs JO, 
												NIVELLS NI,
												NIVELLS_JUGADORS NJ
											Where JO.ID = ni.jocs_id
												  and ni.id = nj.nivells_id
												  and ju.id = nj.jugadors_id and jo.id = p_joc_id)
				group by p.CODPAIS, p.nom 
			UNION
				select 'PRO' as Filtre, pv.CODPAIS, pv.CODPROV, null as CODPOB,  pv.nom , count(*) as Total 
				from jugadors J LEFT OUTER JOIN provincies pv ON j.codpais = pv.codpais AND  j.codprov = pv.codprov 
				WHERE j.ID IN (select distinct JU.ID
										   from Jugadors JU, 
												jocs JO, 
												NIVELLS NI,
												NIVELLS_JUGADORS NJ
											Where JO.ID = ni.jocs_id
												  and ni.id = nj.nivells_id
												  and ju.id = nj.jugadors_id and jo.id = p_joc_id)
				group by pv.CODPAIS, pv.CODPROV, pv.nom
			UNION
				select 'POB' as Filtre, po.CODPAIS, po.CODPROV, po.CODPOB,  po.nom , count(*) as Total 
				from jugadors J LEFT OUTER JOIN poblacions po ON j.codpais = po.codpais AND  j.codprov = po.codprov AND j.codpob = po.codpob
				WHERE j.ID IN (select distinct JU.ID
										   from Jugadors JU, 
												jocs JO, 
												NIVELLS NI,
												NIVELLS_JUGADORS NJ
											Where JO.ID = ni.jocs_id
												  and ni.id = nj.nivells_id
												  and ju.id = nj.jugadors_id and jo.id = p_joc_id)
				group by po.CODPAIS, po.CODPROV, po.CODPOB, po.nom;

	
	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat || ', p_rs: Recordset amb el resultat de la consulta.';
	v_resultat     := '00: Consulta realitzada correctament';
      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
	   			
	

EXCEPTION
    WHEN OTHERS THEN
          CASE (SQLCODE) 
          			WHEN -20000 THEN
                        v_resultat := '04: Ha d''introduir el codi del Joc.';
					WHEN -20001 THEN
                        v_resultat := '03: Joc no trobat amb ID = ' || p_joc_id;
					ELSE
                        -- Error no controlat
                        v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
                END CASE;            

	-- Es genera un recordset buit
        OPEN p_rs FOR
        SELECT * FROM DUAL WHERE DUMMY IS NULL;
				
	-- S'assignem els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat || ', p_rs: Recordset buit.';
        
	DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
				
END CNS_TJUGJOC_XZONA;
---------------------------------------------------------------------------------


---------------------------------------------------------------------------------
-- CU10-02: Consulta Totals de Jugadors per Sexe
-- Parmetres d'entrada:
--		* p_joc_id: Identificador del Joc
-- Parmetres de sortida: 
--		* p_rs: Recordset amb la Informaci
--		* p_resultat: Resultat del procediment OK/KO
-- El procediment realitzar una consulta extraient de forma agrupada el nmero total de jugadors del joc per sexe.
---------------------------------------------------------------------------------
PROCEDURE CNS_TJUGJOC_XSEXE(p_rs IN OUT sys_refcursor, p_joc_id IN PFCBD.JOC_NAVEGADORS.JOCS_ID%TYPE, p_resultat OUT VARCHAR)
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
	
	v_jocid PFCBD.JOC_NAVEGADORS.JOCS_ID%TYPE;
BEGIN
	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.CNS_TJUGJOC_XSEXE';
	v_paramEnt       := 'p_joc_id = ' || p_joc_id;
	
	
	-- Es verifiquen els parmetres d'entrada obligatoris  
	if p_joc_id is null then
		  RAISE_APPLICATION_ERROR(-20000,'');
	end if;
  
  
	-- Es verifica que el Joc existeix al sistema 
	  BEGIN
	  IF p_joc_id is not NULL THEN
		SELECT ID into v_jocid
				FROM JOCS
				WHERE ID = p_joc_id;
	  END IF;
		EXCEPTION
			WHEN OTHERS THEN
				IF (SQLCODE = 100) THEN
					-- Joc no trobat
					RAISE_APPLICATION_ERROR(-20001,'');
				END IF; 
	  END;   
  
	  open p_rs for
			select j.sexe, count(*) as Total 
			from jugadors J
			where j.ID IN (select distinct JU.ID
						   from Jugadors JU, 
								jocs JO, 
								NIVELLS NI,
								NIVELLS_JUGADORS NJ
							Where JO.ID = ni.jocs_id
							  and ni.id = nj.nivells_id
							  and ju.id = nj.jugadors_id 
				  and jo.id = p_joc_id)
				group by j.sexe;

	
	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat || ', p_rs: Recordest amb el resultat de la consulta.';
	v_resultat     := '00: Consulta realitzada correctament';
      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
	

EXCEPTION
    WHEN OTHERS THEN
          CASE (SQLCODE) 
          			WHEN -20000 THEN
                        v_resultat := '04: Ha d''introduir el codi del Joc.';
					WHEN -20001 THEN
                        v_resultat := '03: Joc no trobat amb ID = ' || p_joc_id;
					ELSE
                        -- Error no controlat
                        v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
                END CASE;            

		   
	-- Es genera un recordset buit
    OPEN p_rs FOR
        SELECT * FROM DUAL WHERE DUMMY IS NULL;
					
	-- S'assignem els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat || ', p_rs: Recordest buit.';
        
	DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
				
END CNS_TJUGJOC_XSEXE;
---------------------------------------------------------------------------------


---------------------------------------------------------------------------------
-- CU10-03: Numero de perfils per Xarxa Social
-- Parmetres d'entrada:
--		* p_joc_id: Identificador del Joc
-- Parmetres de sortida: 
--		* p_rs: Recordset amb la Informaci
--		* p_resultat: Resultat del procediment OK/KO
-- El procediment realitzar una consulta extraient de forma agrupada el numero total de perfils per xarxa social.
-- Noms es tindran en compte els perfils dels jugadors del joc analitzat.
---------------------------------------------------------------------------------
PROCEDURE CNS_TJUGJOC_XXARSOC(p_rs IN OUT sys_refcursor, p_joc_id IN PFCBD.JOC_NAVEGADORS.JOCS_ID%TYPE, p_resultat OUT VARCHAR)
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
	
	v_jocid PFCBD.JOC_NAVEGADORS.JOCS_ID%TYPE;
BEGIN
	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.CNS_TJUGJOC_XXARSOC';
	v_paramEnt       := 'p_joc_id = ' || p_joc_id;

	-- Es verifiquen els parmetres d'entrada obligatoris  
	  if p_joc_id is null then
		  RAISE_APPLICATION_ERROR(-20000,'');
	  end if;
  
  
	-- Es verifica que el Joc existeix al sistema 
	  BEGIN
	  IF p_joc_id is not NULL THEN
		SELECT ID into v_jocid
				FROM JOCS
				WHERE ID = p_joc_id;
	  END IF;
		EXCEPTION
			WHEN OTHERS THEN
				IF (SQLCODE = 100) THEN
					-- Joc no trobat
					RAISE_APPLICATION_ERROR(-20001,'');
				END IF; 
	  END;   
  
	  open p_rs for
		select x.id, x.nom, count(*) as Total 
			from jugadors J,
			jugadors_xarxes jx,
			xarxes_socials x
			where j.ID = jx.jugadors_id
			  and x.ID = jx.xarxes_socials_id
			  and j.ID IN (select distinct JU.ID
						   from Jugadors JU, 
								jocs JO, 
								NIVELLS NI,
								NIVELLS_JUGADORS NJ
							Where JO.ID = ni.jocs_id
							  and ni.id = nj.nivells_id
							  and ju.id = nj.jugadors_id
							  and jo.id = p_joc_id)
			group by x.id, x.nom;          

	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat || ', p_rs: Recordset amb el resultat de la consulta.';
	v_resultat     := '00: Consulta realitzada correctament';
      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
	
	EXCEPTION
		WHEN OTHERS THEN
			  CASE (SQLCODE) 
						WHEN -20000 THEN
							v_resultat := '04: Ha d''introduir el codi del Joc.';
						WHEN -20001 THEN
							v_resultat := '03: Joc no trobat amb ID = ' || p_joc_id;
						ELSE
							-- Error no controlat
							v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
					END CASE;            
	   
	-- Es genera un recordset buit
     OPEN p_rs FOR
        SELECT * FROM DUAL WHERE DUMMY IS NULL;
				
	-- S'assignem els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat || ', p_rs: Recordset buit.';
        
	DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
				
END CNS_TJUGJOC_XXARSOC;
---------------------------------------------------------------------------------


---------------------------------------------------------------------------------
-- CU10-04: Numero de collaboracions del darrer mes
-- Parmetres d'entrada:
--		* p_joc_id: Identificador del Joc
-- Parmetres de sortida: 
--		* p_rs: Recordset amb la Informaci
--		* p_resultat: Resultat del procediment OK/KO
-- El procediment realitzar una consulta extraient el numero de collaboracions realitzades en els darrers 30 dies.
---------------------------------------------------------------------------------
PROCEDURE CNS_TCOL_MES(p_rs IN OUT sys_refcursor, p_joc_id IN PFCBD.JOC_NAVEGADORS.JOCS_ID%TYPE, p_resultat OUT VARCHAR)
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
	
	v_jocid PFCBD.JOC_NAVEGADORS.JOCS_ID%TYPE;
BEGIN
	
	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.CNS_TCOL_MES';
	v_paramEnt       := 'p_joc_id = ' || p_joc_id;
	
	-- Es verifiquen els parmetres d'entrada obligatoris  
	if p_joc_id is null then
		  RAISE_APPLICATION_ERROR(-20000,'');
	end if;
  
  
	-- Es verifica que el Joc existeix al sistema 
	  BEGIN
	  IF p_joc_id is not NULL THEN
		SELECT ID into v_jocid
				FROM JOCS
				WHERE ID = p_joc_id;
	  END IF;
		EXCEPTION
			WHEN OTHERS THEN
				IF (SQLCODE = 100) THEN
					-- Joc no trobat
					RAISE_APPLICATION_ERROR(-20001,'');
				END IF; 
	  END;   
	  
	  open p_rs for
		select count(*) as Total
		from collaboracions c,
			productes p,
			jocs j
		where c.productes_id = p.id
			and j.id = p.jocs_id
			and j.id = p_joc_id
			and datainici >= (sysdate - 30);
	

	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat || ', p_rs: Recordset amb el resultat de la consulta.';
	v_resultat     := '00: Consulta realitzada correctament';
      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
	
EXCEPTION
    WHEN OTHERS THEN
          CASE (SQLCODE) 
          			WHEN -20000 THEN
                        v_resultat := '04: Ha d''introduir el codi del Joc.';
					WHEN -20001 THEN
                        v_resultat := '03: Joc no trobat amb ID = ' || p_joc_id;
					ELSE
                        -- Error no controlat
                        v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
                END CASE;            

		   
	-- Es genera un recordset buit
       OPEN p_rs FOR
       SELECT * FROM DUAL WHERE DUMMY IS NULL;
			
				
	-- S'assignem els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat || ', p_rs: Recordset buit.';
        
	DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
				
END CNS_TCOL_MES;
---------------------------------------------------------------------------------


---------------------------------------------------------------------------------
-- CU10-05: Nmero de compres del darrer mes
-- Parmetres d'entrada:
--		* p_joc_id: Identificador del Joc
-- Parmetres de sortida: 
--		* p_rs: Recordset amb la Informaci
--		* p_resultat: Resultat del procediment OK/KO
-- El procediment realitzar una consulta extraient el nmero de compres realitzades en els darrers 30 dies.
---------------------------------------------------------------------------------
PROCEDURE CNS_TCOM_MES(p_rs IN OUT sys_refcursor, p_joc_id IN PFCBD.JOC_NAVEGADORS.JOCS_ID%TYPE, p_resultat OUT VARCHAR)
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
	
	v_jocid PFCBD.JOC_NAVEGADORS.JOCS_ID%TYPE;
BEGIN
	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.CNS_TCOM_MES';
	v_paramEnt       := 'p_joc_id = ' || p_joc_id;
  
	-- Es verifiquen els parmetres d'entrada obligatoris  
	  if p_joc_id is null then
		  RAISE_APPLICATION_ERROR(-20000,'');
	  end if;
  
	  
	 -- Es verifica que el Joc existeix al sistema 
	  BEGIN
	  IF p_joc_id is not NULL THEN
		SELECT ID into v_jocid
				FROM JOCS
				WHERE ID = p_joc_id;
	  END IF;
		EXCEPTION
			WHEN OTHERS THEN
				IF (SQLCODE = 100) THEN
					-- Joc no trobat
					RAISE_APPLICATION_ERROR(-20001,'');
				END IF; 
	  END;   
	  
	  open p_rs for
		select count(*) as NCompres, sum(c.preu) as Import
		from compres c,
			productes p,
			jocs j
		where c.productes_id = p.id
			and j.id = p.jocs_id
			and j.id = p_joc_id
			and datacompra >= (sysdate - 30);
    
	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat || ', p_rs: Recordset amb el resultat de la consulta.';
	v_resultat     := '00: Consulta realitzada correctament';
      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);		
	

EXCEPTION
    WHEN OTHERS THEN
          CASE (SQLCODE) 
          			WHEN -20000 THEN
                        v_resultat := '04: Ha d''introduir el codi del Joc.';
					WHEN -20001 THEN
                        v_resultat := '03: Joc no trobat amb ID = ' || p_joc_id;
					ELSE
                        -- Error no controlat
                        v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
                END CASE;            

		-- Es genera un recordset buit
        OPEN p_rs FOR
           SELECT * FROM DUAL WHERE DUMMY IS NULL;
				
	-- S'assignem els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat || ', p_rs: Recordset buit.';
        
	DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);
				
END CNS_TCOM_MES;
---------------------------------------------------------------------------------


---------------------------------------------------------------------------------
-- CU10-06: Jugadors per plataforma i navegador
-- Parmetres d'entrada:
--		* p_joc_id: Identificador del Joc
-- Parmetres de sortida: 
--		* p_rs: Recordset amb la Informaci
--		* p_resultat: Resultat del procediment OK/KO
-- El procediment realitzar una consulta extraient el numero de jugadors per plataforma i navegador utilitzat.
-- Noms es tindran en compte els jugadors del joc analitzat.
---------------------------------------------------------------------------------
PROCEDURE CNS_TJUGJOC_PLAT_NAV(p_rs IN OUT sys_refcursor, p_joc_id IN PFCBD.JOC_NAVEGADORS.JOCS_ID%TYPE, p_resultat OUT VARCHAR)
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
	
	v_jocid PFCBD.JOC_NAVEGADORS.JOCS_ID%TYPE;
BEGIN
	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.CNS_TJUGJOC_PLAT_NAV';
	v_paramEnt       := 'p_joc_id = ' || p_joc_id;
	
	
	-- Es verifiquen els parmetres d'entrada obligatoris  
	  if p_joc_id is null then
		  RAISE_APPLICATION_ERROR(-20000,'');
	  end if;
		
	 -- Es verifica que el Joc existeix al sistema 
	  BEGIN
	  IF p_joc_id is not NULL THEN
		SELECT ID into v_jocid
				FROM JOCS
				WHERE ID = p_joc_id;
	  END IF;
		EXCEPTION
			WHEN OTHERS THEN
				IF (SQLCODE = 100) THEN
					-- Joc no trobat
					RAISE_APPLICATION_ERROR(-20001,'');
				END IF; 
	  END;   
	  
	  open p_rs for
			select 'PLA' as Filtre, p.id, p.nom , count(*) as Total 
				from jugadors J, plataformes_jugadors pj, plataformes p 
				where j.id = pj.jugadors_id
				and p.id = pj.plataformes_id
						and j.ID IN (select distinct JU.ID
										   from Jugadors JU, 
												jocs JO, 
												NIVELLS NI,
												NIVELLS_JUGADORS NJ
											Where JO.ID = ni.jocs_id
												  and ni.id = nj.nivells_id
												  and ju.id = nj.jugadors_id and jo.id = p_joc_id)
						group by p.id, p.nom    
				UNION
				select 'NAV' as Filtre, n.id, n.nom , count(*) as Total 
							from jugadors J, navegadors_jugadors nj, navegadors n
							where j.id = nj.jugadors_id
							and n.id = nj.navegadors_id
									and j.ID IN (select distinct JU.ID
													   from Jugadors JU, 
															jocs JO, 
															NIVELLS NI,
															NIVELLS_JUGADORS NJ
														Where JO.ID = ni.jocs_id
															  and ni.id = nj.nivells_id
															  and ju.id = nj.jugadors_id and jo.id = p_joc_id)
							group by n.id, n.nom ;
		
	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat || ', p_rs: Recordset amb la consulta realitzada.';
	v_resultat     := '00: Consulta realitzada correctament';
      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
		

  EXCEPTION
    WHEN OTHERS THEN
          CASE (SQLCODE) 
          			WHEN -20000 THEN
                        v_resultat := '04: Ha d''introduir el codi del Joc.';
					WHEN -20001 THEN
                        v_resultat := '03: Joc no trobat amb ID = ' || p_joc_id;
					ELSE
                        -- Error no controlat
                        v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
                END CASE;            

	-- Es genera un recordset buit
     OPEN p_rs FOR
       SELECT * FROM DUAL WHERE DUMMY IS NULL;
				
	-- S'assignem els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat || ', p_rs: Recordset buit.';
        
	DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);
				
END CNS_TJUGJOC_PLAT_NAV;
---------------------------------------------------------------------------------


END EST_PENETRACIOJOC;
/
