---------------------------------------------------------------------------------
-- Alumne: Jessica Villamor 
-- Consultor: Juan Martnez Bolaos
-- Projecte: GESTIO DE VIDEOJOCS
---------------------------------------------------------------------------------

-- Procediments CU11: Consultar dades per valorar la dificultat del Joc

--
-- Ens situem a l'esquema de la base de dades de Gesti de Videojocs
--
ALTER SESSION SET CURRENT_SCHEMA = PFCBD;

-- Creaci de la definici del paquet
Create or replace
PACKAGE  PFCBD.EST_DIFICULTATJOC AS

		PROCEDURE CNS_TJUGJOC_NIVELL(p_rs IN OUT sys_refcursor, p_joc_id IN PFCBD.JOC_NAVEGADORS.JOCS_ID%TYPE, p_rSup IN VARCHAR, p_resultat OUT VARCHAR);
		
		PROCEDURE CNS_TJUGJOC_INTXNIV(p_rs IN OUT sys_refcursor, p_joc_id IN PFCBD.JOC_NAVEGADORS.JOCS_ID%TYPE, p_resultat OUT VARCHAR);
		
		PROCEDURE CNS_TJUGJOC_DIESXNIV(p_rs IN OUT sys_refcursor, p_joc_id IN PFCBD.JOC_NAVEGADORS.JOCS_ID%TYPE, p_resultat OUT VARCHAR);

END EST_DIFICULTATJOC;
/


-- Creaci del cos del paquet
CREATE OR REPLACE PACKAGE BODY PFCBD.EST_DIFICULTATJOC AS

---------------------------------------------------------------------------------
-- Procediments CU11: Consultes per valorar la dificultat del Joc
---------------------------------------------------------------------------------

---------------------------------------------------------------------------------
-- CU11-01: Consulta Totals d'usuaris per Nivell
-- Parmetres d'entrada:
--		* p_joc_id: Identificador del Joc
-- 		* p_rSup: Indicador de extracci de dades noms incloent els nivells superats
-- Parmetres de sortida: 
--		* p_rs: Recordset amb la Informaci
--		* p_resultat: Resultat del procediment OK/KO
---------------------------------------------------------------------------------
PROCEDURE CNS_TJUGJOC_NIVELL(p_rs IN OUT sys_refcursor, p_joc_id IN PFCBD.JOC_NAVEGADORS.JOCS_ID%TYPE, p_rSup IN VARCHAR, p_resultat OUT VARCHAR)
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
	
	v_jocid PFCBD.JOC_NAVEGADORS.JOCS_ID%TYPE;
BEGIN
	
	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.CNS_TJUGJOC_NIVELL';
	v_paramEnt       := 'p_joc_id = ' || p_joc_id;

	-- Es verifiquen els parmetres d'entrada obligatoris  
	  if p_joc_id is null then
		  RAISE_APPLICATION_ERROR(-20000,'');
	  end if;
  
  
	 -- Es verifica que el Joc existeix al sistema 
	  BEGIN
	  IF p_joc_id is not NULL THEN
		SELECT ID into v_jocid
				FROM JOCS
				WHERE ID = p_joc_id;
	  END IF;
		EXCEPTION
			WHEN OTHERS THEN
				IF (SQLCODE = 100) THEN
					-- Joc no trobat
					RAISE_APPLICATION_ERROR(-20001,'');
				END IF; 
	  END;   
	  
	  
	  if p_rSup is not null then
	   open p_rs for
		select n.id, n.nom, n.nivseg_id, count(nj.nivells_id)
		from nivells n left outer join nivells_jugadors nj on n.id = nj.nivells_id AND nj.repte = 'Y'
		where n.jocs_id = p_joc_id
		group by n.id, n.nom, n.nivseg_id;
	  else
		  open p_rs for
			select n.id, n.nom, n.nivseg_id, count(nj.nivells_id)
			from nivells n left outer join nivells_jugadors nj on n.id = nj.nivells_id
			where n.jocs_id = p_joc_id
			group by n.id, n.nom, n.nivseg_id;
	  END IF;		
			
	
	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat || ', p_rs: Recordset amb el resultat de la consulta.';
	v_resultat     := '00: Consulta realitzada correctament';
      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);
	

EXCEPTION
    WHEN OTHERS THEN
          CASE (SQLCODE) 
          			WHEN -20000 THEN
                        v_resultat := '04: Ha d''introduir el codi del Joc.';
					WHEN -20001 THEN
                        v_resultat := '03: Joc no trobat amb ID = ' || p_joc_id;
					ELSE
                        -- Error no controlat
                        v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
                END CASE;            

	-- Es genera un recordset buit
    OPEN p_rs FOR
        SELECT * FROM DUAL WHERE DUMMY IS NULL;
				
	-- S'assignem els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat || ', p_rs: Recordset buit.';
        
	DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
				
END CNS_TJUGJOC_NIVELL;
---------------------------------------------------------------------------------


---------------------------------------------------------------------------------
-- CU11-02: Consulta numero mig d'intents per nivell
-- Parmetres d'entrada:
--		* p_joc_id: Identificador del Joc
-- Parmetres de sortida: 
--		* p_rs: Recordset amb la Informaci
--		* p_resultat: Resultat del procediment OK/KO
---------------------------------------------------------------------------------
PROCEDURE CNS_TJUGJOC_INTXNIV(p_rs IN OUT sys_refcursor, p_joc_id IN PFCBD.JOC_NAVEGADORS.JOCS_ID%TYPE, p_resultat OUT VARCHAR)
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
	
	v_jocid PFCBD.JOC_NAVEGADORS.JOCS_ID%TYPE;
BEGIN
	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.CNS_TJUGJOC_INTXNIV';
	v_paramEnt       := 'p_joc_id = ' || p_joc_id;
	
	-- Es verifiquen els parmetres d'entrada obligatoris  
	  if p_joc_id is null then
		  RAISE_APPLICATION_ERROR(-20000,'');
	  end if;
  
	  
	-- Es verifica que el Joc existeix al sistema 
	  BEGIN
	  IF p_joc_id is not NULL THEN
		SELECT ID into v_jocid
				FROM JOCS
				WHERE ID = p_joc_id;
	  END IF;
		EXCEPTION
			WHEN OTHERS THEN
				IF (SQLCODE = 100) THEN
					-- Joc no trobat
					RAISE_APPLICATION_ERROR(-20001,'');
				END IF; 
	  END;   
	  
		open p_rs for
			select n.id, n.nom, n.nivseg_id, avg(nj.nintents) as AVGINTENTS
			from nivells n left outer join nivells_jugadors nj on n.id = nj.nivells_id and nj.repte = 'Y'
			where n.jocs_id = p_joc_id
			group by n.id, n.nom, n.nivseg_id;
		
		
	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat || ', p_rs: Recordset amb el resultat de la consulta.';
	v_resultat     := '00: Consulta realitzada correctament';
      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);		
	

EXCEPTION
    WHEN OTHERS THEN
          CASE (SQLCODE) 
          			WHEN -20000 THEN
                        v_resultat := '04: Ha d''introduir el codi del Joc.';
					WHEN -20001 THEN
                        v_resultat := '03: Joc no trobat amb ID = ' || p_joc_id;
					ELSE
                        -- Error no controlat
                        v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
                END CASE;            

	-- Es genera un recordset buit
    OPEN p_rs FOR
        SELECT * FROM DUAL WHERE DUMMY IS NULL;
				
	-- S'assignem els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat || ', p_rs: Recordset buit.';
        
	DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
				
END CNS_TJUGJOC_INTXNIV;
---------------------------------------------------------------------------------


---------------------------------------------------------------------------------
-- CU11-03: Consulta nmero mig de dies per superar un nivell
-- Parmetres d'entrada:
--		* p_joc_id: Identificador del Joc
-- Parmetres de sortida: 
--		* p_rs: Recordset amb la Informaci
--		* p_resultat: Resultat del procediment OK/KO
-- Aquest procediment extraur el nmero de dies mig per superar un nivell. 
-- Per no desvirtuar l'estadstica es tindr noms en compte els registres amb nivell superat.
-- La consulta arrodonir al ala per no tenir decimals.
-- Noms es tindran en compte els nivells del joc analitzat.
---------------------------------------------------------------------------------
PROCEDURE CNS_TJUGJOC_DIESXNIV(p_rs IN OUT sys_refcursor, p_joc_id IN PFCBD.JOC_NAVEGADORS.JOCS_ID%TYPE, p_resultat OUT VARCHAR)
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
	
	v_jocid PFCBD.JOC_NAVEGADORS.JOCS_ID%TYPE;
BEGIN
	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.CNS_TJUGJOC_DIESXNIV';
	v_paramEnt       := 'p_joc_id = ' || p_joc_id;
	
	-- Es verifiquen els parmetres d'entrada obligatoris  
	  if p_joc_id is null then
		  RAISE_APPLICATION_ERROR(-20000,'');
	  end if;
	  
	  
	-- Es verifica que el Joc existeix al sistema 
	  BEGIN
	  IF p_joc_id is not NULL THEN
		SELECT ID into v_jocid
				FROM JOCS
				WHERE ID = p_joc_id;
	  END IF;
		EXCEPTION
			WHEN OTHERS THEN
				IF (SQLCODE = 100) THEN
					-- Joc no trobat
					RAISE_APPLICATION_ERROR(-20001,'');
				END IF; 
	  END;   
  
	open p_rs for
	select n.id, n.nom, n.nivseg_id, ceil(avg(nj.datafi - nj.datainici + 1)) as AVGDIES
	from nivells n left outer join nivells_jugadors nj on n.id = nj.nivells_id and nj.repte = 'Y'
	where n.jocs_id = p_joc_id
	group by n.id, n.nom, n.nivseg_id;
	
			 
	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat || ', p_rs: Recordset amb el resultat de la consulta.';
	v_resultat     := '00: Consulta realitzada correctament';
      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
	

EXCEPTION
    WHEN OTHERS THEN
          CASE (SQLCODE) 
          			WHEN -20000 THEN
                        v_resultat := '04: Ha d''introduir el codi del Joc.';
					WHEN -20001 THEN
                        v_resultat := '03: Joc no trobat amb ID = ' || p_joc_id;
					ELSE
                        -- Error no controlat
                        v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
                END CASE;            

-- Es genera un recordset buit
    OPEN p_rs FOR
    SELECT * FROM DUAL WHERE DUMMY IS NULL;			
				
-- S'assignem els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat || ', p_rs: Recordset buit.';
        
	DBMS_OUTPUT.put_line (v_resultat);				
-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
				
END CNS_TJUGJOC_DIESXNIV;
---------------------------------------------------------------------------------

END EST_DIFICULTATJOC;
/
