---------------------------------------------------------------------------------
-- Alumne: Jessica Villamor 
-- Consultor: Juan Martnez Bolaos
-- Projecte: GESTIO DE VIDEOJOCS
---------------------------------------------------------------------------------

-- Procediments CU12: Gesti de Plataformes

--
-- Ens situem a l'esquema de la base de dades de Gesti de Videojocs
--
ALTER SESSION SET CURRENT_SCHEMA = PFCBD;

-- Creaci de la definici del paquet
Create or replace
PACKAGE    PFCBD.DM_PLATAFORMES AS
		
		PROCEDURE INS_PLATAFORMES( p_nom IN PFCBD.PLATAFORMES.NOM%TYPE, p_resultat OUT VARCHAR);
		
		PROCEDURE UPD_PLATAFORMES ( p_id IN PFCBD.PLATAFORMES.ID%TYPE, p_nom IN PFCBD.PLATAFORMES.NOM%TYPE, p_resultat OUT VARCHAR);
		
		PROCEDURE  DEL_PLATAFORMES (p_id IN PFCBD.PLATAFORMES.id%TYPE, p_resultat OUT VARCHAR);
		
		PROCEDURE CNS_PLATAFORMES (p_rs IN OUT sys_refcursor, 	
								p_id IN PFCBD.PLATAFORMES.id%TYPE, 	
								p_nom IN PFCBD.PLATAFORMES.NOM%TYPE, p_resultat OUT VARCHAR);
END DM_PLATAFORMES;
/


-- Creaci del cos del paquet
CREATE OR REPLACE PACKAGE BODY PFCBD.DM_PLATAFORMES AS

---------------------------------------------------------------------------------
-- Procediments CU12: Gesti DM Plataformes
---------------------------------------------------------------------------------

---------------------------------------------------------------------------------
-- CU12-01: Alta Plataforma
-- Parmetres d'entrada:
--		* p_nom: Nom de la Plataforma
-- Parmetres de sortida: 
--		* p_resultat: Resultat del procediment OK/KO
---------------------------------------------------------------------------------
PROCEDURE INS_PLATAFORMES( p_nom IN PFCBD.PLATAFORMES.NOM%TYPE, p_resultat OUT VARCHAR)
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;

 
BEGIN
	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.INS_PLATAFORMES';
	v_paramEnt       := 'p_nom = ' || p_nom;

   
	-- S'inserta la nova Plataforma
	INSERT INTO PLATAFORMES(NOM) VALUES (UPPER(p_nom));

	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat;
	v_resultat     := '00: Plataforma creada correctament';
	
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);
	
	EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE) 
                    WHEN -1 THEN
                        -- Registre duplicat
                        v_resultat := '01: La plataforma ja existeix.';
                    WHEN -1400 THEN
                        -- Valor nul
                        v_resultat := '02: S''ha intentat inserir un valor NULL' || SUBSTR(SQLERRM,INSTR(SQLERRM,'('));
					ELSE
                        -- Error no controlat
                        v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
                END CASE;            

	-- S'assignem els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat;
        
	DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
				
END INS_PLATAFORMES;
---------------------------------------------------------------------------------


---------------------------------------------------------------------------------
-- CU12-02: Actualitzar Plataforma
-- Parmetres d'entrada:
--		* p_id: Identificador de la Plataforma
--		* p_nom: Nom de la Plataforma
-- Parmetres de sortida: 
--		* p_resultat: Resultat del procediment OK/KO
---------------------------------------------------------------------------------
PROCEDURE UPD_PLATAFORMES ( p_id IN PFCBD.PLATAFORMES.ID%TYPE, p_nom IN PFCBD.PLATAFORMES.NOM%TYPE, p_resultat OUT VARCHAR)
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
	
	v_plat_id PFCBD.PLATAFORMES.ID%TYPE;
 
BEGIN

	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.UPD_PLATAFORMES';
	v_paramEnt       := 'p_id = ' || p_id 
                      || ', p_nom = ' || p_nom;
				  
	
	-- Es verifica que la Plataforma existeix al sistema	
	BEGIN
		IF p_id IS NOT NULL THEN
			SELECT ID into v_plat_id
					  FROM PLATAFORMES
					  WHERE ID = p_id;
		END IF;
		EXCEPTION
			WHEN OTHERS THEN
				 IF (SQLCODE = 100) THEN
					   -- Plataforma no trobada
					   RAISE_APPLICATION_ERROR(-20000,'');
				 END IF; 
	END;  
    
	if p_id is null then
		RAISE_APPLICATION_ERROR(-20001,'');
    end if;
	
	UPDATE PLATAFORMES SET NOM = upper(p_nom) WHERE PLATAFORMES.ID = p_id;
    
	IF SQL%rowcount = 0	THEN
		RAISE_APPLICATION_ERROR(-20000,'');
	ELSE
		v_resultat := '00: Plataforma actualitzada correctament';
	END IF;
  
	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat;
	      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	

	EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                   WHEN -1 THEN
                        -- Registre duplicat
                        v_resultat := '01: La plataforma ja existeix.';
                   WHEN -1407 THEN
                        -- Valor nulo
                        v_resultat := '02: S''ha intentat inserir un valor NULL.' || SUBSTR(SQLERRM,INSTR(SQLERRM,'('));
                    WHEN -20000 THEN
                        v_resultat := '03: Plataforma no trobada amb ID = ' || p_id;
                    WHEN -20001 THEN
                        v_resultat := '04: Ha d''informar l''identificador de la plataforma a actualitzar';
                    ELSE
                        -- Error no controlat
                        v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
                END CASE;            

	-- S'assignem els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat;
        
	DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
				
END UPD_PLATAFORMES;
---------------------------------------------------------------------------------


---------------------------------------------------------------------------------
-- CU12-03: Esborrar Plataforma
-- Parmetres d'entrada:
--		* p_id: Identificador de la Plataforma
-- Parmetres de sortida: 
--		* p_resultat: Resultat del procediment OK/KO
---------------------------------------------------------------------------------
PROCEDURE  DEL_PLATAFORMES (p_id IN PFCBD.PLATAFORMES.id%TYPE, p_resultat OUT VARCHAR)
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
		
BEGIN

	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.DEL_PLATAFORMES';
	v_paramEnt       := 'p_id = ' || p_id;
					 
					 
	 -- Es verifica que s'indica el codi de la plataforma a esborrar
	  IF p_id is null then
		RAISE_APPLICATION_ERROR(-20000,'');
	  end if;
  
	DELETE PLATAFORMES
	WHERE id = p_id;

	-- Comprovem si s'ha trobat el registre a esborrar
	IF SQL%rowcount = 0 THEN
		RAISE_APPLICATION_ERROR(-20001,'');
	ELSE
		v_resultat := '00: Plataforma esborrada correctament';
	END IF;
 
	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat;
	
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	

  
EXCEPTION
	WHEN OTHERS THEN
		CASE (SQLCODE)
           WHEN -20000 THEN
	                 v_resultat := '01: Ha d''informar l''identificador de la plataforma a esborrar.';
		   WHEN -20001 THEN
	                 v_resultat := '03: Plataforma no trobada amb ID = ' || p_id;
           WHEN -2292 THEN
				-- Restricci d'integritat
					 v_resultat := '04: S''ha intentat esborrar un registre amb dependncies.' || SUBSTR(SQLERRM,INSTR(SQLERRM,'('));
           ELSE
                -- Error no controlat
					  v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
        END CASE;            
				
	-- S'assignem els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat;
        
	DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
		
END DEL_PLATAFORMES;
---------------------------------------------------------------------------------


---------------------------------------------------------------------------------
-- CU12-04: Consultar Plataformes
-- Parmetres d'entrada:
--		* p_id: Identificador de la plataforma
--		* p_nom: Nom de la Plataforma
-- Parmetres de sortida:
--		* p_rs: Recordset amb el resultat
--		* p_resultat: Resultat del procediment OK/KO
-- Sempre que s'introdueixi l'identificador per realitzar la consulta, es consultar la taula de Plataformes per l'identificador.
-- Es permetr realitzar consulta de plataforma per nom.
-- Si no s'especifica cap parmetre s'extreuran totes les plataformes del sistema.
---------------------------------------------------------------------------------
PROCEDURE CNS_PLATAFORMES (p_rs IN OUT sys_refcursor, 	
						p_id IN PFCBD.PLATAFORMES.id%TYPE, 	
						p_nom IN PFCBD.PLATAFORMES.NOM%TYPE, p_resultat OUT VARCHAR)
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;

BEGIN
	
	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.CNS_PLATAFORMES';
	v_paramEnt       := 'p_id = ' || p_id 
                      || ', p_nom = ' || p_nom;
	
	if p_id is not NULL then
			OPEN p_rs FOR
			SELECT *
			FROM PLATAFORMES
			where id = p_id;
	
	ELSIF p_nom is not null then
			OPEN p_rs FOR
			SELECT *
			FROM PLATAFORMES
			where nom like '%' || UPPER(p_nom) || '%';
	ELSE
			OPEN p_rs FOR
			SELECT *
			FROM PLATAFORMES;
	end if;
			
	
	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat || ', p_rs: Recordset amb el resultat de la consulta.';
	v_resultat := '00: Consulta realitzada correctament';
      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
	
EXCEPTION
            WHEN OTHERS THEN
                v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
                
	-- Es genera un recordset buit
    OPEN p_rs FOR
    SELECT * FROM DUAL WHERE DUMMY IS NULL;
				
	-- S'assignem els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
   	v_paramSor	:= 'p_resultat: ' || p_resultat || ', p_rs: Recordset buit.';
        
	DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
				
END CNS_PLATAFORMES;
---------------------------------------------------------------------------------

END DM_PLATAFORMES;
/

