---------------------------------------------------------------------------------
-- Alumne: Jessica Villamor 
-- Consultor: Juan Martnez Bolaos
-- Projecte: GESTIO DE VIDEOJOCS
---------------------------------------------------------------------------------

-- Procediments CU13: Gesti de Navegadors Web

--
-- Ens situem a l'esquema de la base de dades de Gesti de Videojocs
--
ALTER SESSION SET CURRENT_SCHEMA = PFCBD;

-- Creaci de la definici del paquet
Create or replace
PACKAGE  PFCBD.DM_NAVEGADORS AS

		PROCEDURE INS_NAVEGADORS( p_nom IN PFCBD.NAVEGADORS.NOM%TYPE, p_resultat OUT VARCHAR);

		PROCEDURE UPD_NAVEGADORS ( p_id IN PFCBD.NAVEGADORS.ID%TYPE, p_nom IN PFCBD.NAVEGADORS.NOM%TYPE, p_resultat OUT VARCHAR);

		PROCEDURE DEL_NAVEGADORS (p_id IN PFCBD.NAVEGADORS.id%TYPE, p_resultat OUT VARCHAR);

		PROCEDURE CNS_NAVEGADORS (p_rs IN OUT sys_refcursor, 	
								p_id IN PFCBD.NAVEGADORS.id%TYPE, 	
								p_nom IN PFCBD.NAVEGADORS.NOM%TYPE, p_resultat OUT VARCHAR);

END DM_NAVEGADORS;
/


-- Creaci del cos del paquet
CREATE OR REPLACE PACKAGE BODY PFCBD.DM_NAVEGADORS AS

---------------------------------------------------------------------------------
-- Procediments CU13: Gesti DM Navegadors
---------------------------------------------------------------------------------


---------------------------------------------------------------------------------
-- CU13-01: Alta Navegador Web
-- Parmetres d'entrada:
--		* p_nom: Nom del Navegador
-- Parmetres de sortida: 
--		* p_resultat: Resultat del procediment OK/KO
---------------------------------------------------------------------------------
PROCEDURE INS_NAVEGADORS( p_nom IN PFCBD.NAVEGADORS.NOM%TYPE, p_resultat OUT VARCHAR)
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
 
BEGIN
	
	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.INS_NAVEGADORS';
	v_paramEnt       := 'p_nom = ' || p_nom;
   
	-- S'inserta el nou Navegador
	INSERT INTO NAVEGADORS(NOM) VALUES (UPPER(p_nom));


	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat;
	v_resultat     := '00: Navegador creat correctament';
      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	


EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE) 
                    WHEN -1 THEN
                        -- Registre duplicat
                        v_resultat := '01: El Navegador ja existeix.';
                    WHEN -1400 THEN
                        -- Valor nul
                        v_resultat := '02: S''ha intentat inserir un valor NULL' || SUBSTR(SQLERRM,INSTR(SQLERRM,'('));
					ELSE
                        -- Error no controlat
                        v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
                END CASE;            

	-- S'assignem els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat;
        
	DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
				
END INS_NAVEGADORS;
---------------------------------------------------------------------------------


---------------------------------------------------------------------------------
-- CU13-02: Actualitzar Navegador Web
-- Parmetres d'entrada:
--		* p_id: Identificador del Navegador
--		* p_nom: Nom del Navegador
-- Parmetres de sortida: 
--		* p_resultat: Resultat del procediment OK/KO
---------------------------------------------------------------------------------
PROCEDURE UPD_NAVEGADORS ( p_id IN PFCBD.NAVEGADORS.ID%TYPE, p_nom IN PFCBD.NAVEGADORS.NOM%TYPE, p_resultat OUT VARCHAR)
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
	
	v_plat_id PFCBD.NAVEGADORS.ID%TYPE;
 
BEGIN

	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.UPD_NAVEGADORS';
	v_paramEnt       := 'p_id = ' || p_id 
                      || ', p_nom = ' || p_nom;
					  
  BEGIN
  
	-- Es verifica que el navegador existeix al sistema
    IF p_id IS NOT NULL THEN
        SELECT ID into v_plat_id
                  FROM NAVEGADORS
                  WHERE ID = p_id;
    END IF;
    EXCEPTION
        WHEN OTHERS THEN
             IF (SQLCODE = 100) THEN
                   -- Navegdor no trobat
                   RAISE_APPLICATION_ERROR(-20000,'');
             END IF; 
		END;  
    
	if p_id is null then
			RAISE_APPLICATION_ERROR(-20001,'');
    end if;
	
	UPDATE NAVEGADORS SET NOM = upper(p_nom) WHERE NAVEGADORS.ID = p_id;
    
	IF SQL%rowcount = 0 THEN
		RAISE_APPLICATION_ERROR(-20000,'');
	ELSE
		v_resultat := '00: Navegador Web actualitzat correctament';
	END IF;
  
	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat;
      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	

	EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                   WHEN -1 THEN
                        -- Registre duplicat
                        v_resultat := '01: El navegador ja existeix.';
                   WHEN -1407 THEN
                        -- Valor nulo
                        v_resultat := '02: S''ha intentat inserir un valor NULL.' || SUBSTR(SQLERRM,INSTR(SQLERRM,'('));
                    WHEN -20000 THEN
                        v_resultat := '03: Navegador no trobat amb ID = ' || p_id;
                    WHEN -20001 THEN
                        v_resultat := '04: Ha d''informar l''identificador del navegador a actualitzar.';
                    ELSE
                        -- Error no controlat
                        v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
                END CASE;            

  	
	-- S'assignem els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat;
        
	DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
				
END UPD_NAVEGADORS;
---------------------------------------------------------------------------------


---------------------------------------------------------------------------------
-- CU13-03: Esborrar Navegador Web
-- Parmetres d'entrada:
--		* p_id: Identificador del Navegador Web
-- Parmetres de sortida: 
--		* p_resultat: Resultat del procediment OK/KO
---------------------------------------------------------------------------------
PROCEDURE  DEL_NAVEGADORS (p_id IN PFCBD.NAVEGADORS.id%TYPE, p_resultat OUT VARCHAR)
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
BEGIN

	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.DEL_NAVEGADORS';
	v_paramEnt       := 'p_id = ' || p_id;
					  
	-- Es verifica que s'indica el codi del navegador a esborrar
	IF p_id is null then
		RAISE_APPLICATION_ERROR(-20000,'');
	end if;

	DELETE NAVEGADORS
	WHERE id = p_id;

	-- Comprovem si s'ha trobat el registre a esborrar
	IF SQL%rowcount = 0 THEN
		 RAISE_APPLICATION_ERROR(-20001,'');
	ELSE
		v_resultat := '00: Navegador esborrat correctament';
	END IF;

	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat;
	      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
	
  
EXCEPTION
	WHEN OTHERS THEN
		CASE (SQLCODE)
            WHEN -20000 THEN
				   v_resultat := '01: Ha d''informar l''identificador del navegador a esborrar.';
			WHEN -20001 THEN
				   v_resultat := '03: Navegador no trobat amb ID = ' || p_id;
            WHEN -2292 THEN
				-- Restricci d'integritat
                   v_resultat := '04: S''ha intentat esborrar un registre amb dependncies.' || SUBSTR(SQLERRM,INSTR(SQLERRM,'('));
            ELSE
                   -- Error no controlat
                   v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
        END CASE;            
		
  	-- S'assignem els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat;
        
	DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
		
END DEL_NAVEGADORS;
---------------------------------------------------------------------------------


---------------------------------------------------------------------------------
-- CU13-04: Consultar Navegadors Web
-- Parmetres d'entrada:
--		* p_id: Identificador del Navegador Web
--		* p_nom: Nom del Navegador
-- Parmetres de sortida:
--		* p_rs: Recordset amb el resultat
--		* p_resultat: Resultat del procediment OK/KO
-- Sempre que s'introdueixi l'identificador per realitzar la consulta es consultar la taula de navegadors per l'identificador.
-- Es permetr realitzar la consulta de navegador web per nom.
-- Si no s'especifica cap parmetre s'extreuran tots els Navegadors Web del sistema.
---------------------------------------------------------------------------------
PROCEDURE CNS_NAVEGADORS (p_rs IN OUT sys_refcursor, 	
						p_id IN PFCBD.NAVEGADORS.id%TYPE, 	
						p_nom IN PFCBD.NAVEGADORS.NOM%TYPE, p_resultat OUT VARCHAR)
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;

BEGIN
	
	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.CNS_NAVEGADORS';
	v_paramEnt       := 'p_id = ' || p_id 
                      || ', p_nom = ' || p_nom;
					  
	
	if p_id is not NULL then
			OPEN p_rs FOR
			SELECT *
			FROM NAVEGADORS
			where id = p_id;
	
	ELSIF p_nom is not null then
			OPEN p_rs FOR
			SELECT *
			FROM NAVEGADORS
			where nom like '%' || upper(p_nom) || '%';
	ELSE
			OPEN p_rs FOR
			SELECT *
			FROM NAVEGADORS;
	end if;

	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat || ', p_rs: Recordset amb el resultat de la consulta.';
	v_resultat := '00: Consulta realitzada correctament';
      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
     
EXCEPTION
            WHEN OTHERS THEN
                v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
                
	-- Es genera un recordset buit
    OPEN p_rs FOR
    SELECT * FROM DUAL WHERE DUMMY IS NULL;
		
	-- S'assignem els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
   v_paramSor	:= 'p_resultat: ' || p_resultat || ', p_rs: Recordset buit.';
        
	DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);
				
END CNS_NAVEGADORS;
---------------------------------------------------------------------------------

END DM_NAVEGADORS;
/
