---------------------------------------------------------------------------------
-- Alumne: Jessica Villamor 
-- Consultor: Juan Martnez Bolaos
-- Projecte: GESTIO DE VIDEOJOCS
---------------------------------------------------------------------------------

-- Procediments CU14: Gesti de Pasos

--
-- Ens situem a l'esquema de la base de dades de Gesti de Videojocs
--
ALTER SESSION SET CURRENT_SCHEMA = PFCBD;

-- Creaci de la definici del paquet
Create or replace
PACKAGE    PFCBD.DM_PAISOS AS

		PROCEDURE INS_PAISOS( p_codi IN PFCBD.PAISOS.CODPAIS%TYPE, p_nom IN PFCBD.PAISOS.NOM%TYPE, p_resultat OUT VARCHAR);

		PROCEDURE UPD_PAISOS( p_codi IN PFCBD.PAISOS.CODPAIS%TYPE, p_nom IN PFCBD.PAISOS.NOM%TYPE, p_resultat OUT VARCHAR);

		PROCEDURE  DEL_PAISOS(p_codi IN PFCBD.PAISOS.codpais%TYPE, p_resultat OUT VARCHAR);

		PROCEDURE CNS_PAISOS(p_rs IN OUT sys_refcursor, 	
								p_codi IN PFCBD.PAISOS.codpais%TYPE, 	
								p_nom IN PFCBD.PAISOS.NOM%TYPE, p_resultat OUT VARCHAR);

END DM_PAISOS;
/


-- Creaci del cos del paquet
CREATE OR REPLACE PACKAGE BODY PFCBD.DM_PAISOS AS

---------------------------------------------------------------------------------
-- Procediments CU14: Gesti DM Pasos
---------------------------------------------------------------------------------


---------------------------------------------------------------------------------
-- CU14-01: Alta d'un nou pas
-- Parmetres d'entrada:
--		* p_codi: Codi de Pas
--		* p_nom: Nom del Pas
-- Parmetres de sortida: 
--		* p_resultat: Resultat del procediment OK/KO
---------------------------------------------------------------------------------
PROCEDURE INS_PAISOS( p_codi IN PFCBD.PAISOS.CODPAIS%TYPE, p_nom IN PFCBD.PAISOS.NOM%TYPE, p_resultat OUT VARCHAR)
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
 
BEGIN
	
	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.INS_PAISOS';
	v_paramEnt       := 'p_codi = ' || p_codi 
                      || ', p_nom = ' || p_nom;
	  
	-- S'inserta el nou Pas
	INSERT INTO PAISOS(CODPAIS, NOM) VALUES (UPPER(p_codi), UPPER(p_nom));

	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat;
	v_resultat     := '00: Pas creat correctament';
      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);

EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE) 
                    WHEN -1 THEN
                        -- Registre duplicat
                        v_resultat := '01: El pas ja existeix.';
                    WHEN -1400 THEN
                        -- Valor nulo
                        v_resultat := '02: S''ha intentat inserir un valor NULL' || SUBSTR(SQLERRM,INSTR(SQLERRM,'('));
					ELSE
                        -- Error no controlat
                        v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
                END CASE;            

	-- S'assignem els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat;
        
	DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
				
END INS_PAISOS;
---------------------------------------------------------------------------------


---------------------------------------------------------------------------------
-- CU14-02: Actualitzar Pas
-- Parmetres d'entrada:
--		* p_codi: Codi de Pas
--		* p_nom: Nom del Pas
-- Parmetres de sortida: 
--		* p_resultat: Resultat del procediment OK/KO
---------------------------------------------------------------------------------
PROCEDURE UPD_PAISOS( p_codi IN PFCBD.PAISOS.CODPAIS%TYPE, p_nom IN PFCBD.PAISOS.NOM%TYPE, p_resultat OUT VARCHAR)
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
	
	v_codi PFCBD.PAISOS.CODPAIS%TYPE;
 
BEGIN

	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.UPD_PAISOS';
	v_paramEnt       := 'p_codi = ' || p_codi 
                      || ', p_nom = ' || p_nom;
					  
  
  	-- Es verifica que el Pas esta donat d'alta en el sistema
	BEGIN
  		IF p_codi IS NOT NULL THEN
			SELECT CODPAIS into v_codi
					  FROM PAISOS
					  WHERE CODPAIS = upper(p_codi);
		END IF;
		EXCEPTION
			WHEN OTHERS THEN
				 IF (SQLCODE = 100) THEN
					   -- Codi Pas no trobat
					   RAISE_APPLICATION_ERROR(-20000,'');
				 END IF; 
	END;  
    
	if p_codi is null then
			RAISE_APPLICATION_ERROR(-20001,'');
    end if;
	
	UPDATE PAISOS SET NOM = upper(p_nom) WHERE PAISOS.CODPAIS = upper(p_codi);
    
	IF SQL%rowcount = 0 THEN
		RAISE_APPLICATION_ERROR(-20000,'');
	ELSE
		v_resultat := '00: Dades de pas actualitzades correctament';
	END IF;
  
	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat;
	      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
	

EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                   WHEN -1 THEN
                        -- Registre duplicat
                        v_resultat := '01: El Pas ja estava donat d''alta en el sistema.';
                   WHEN -1407 THEN
                        -- Valor nul
                        v_resultat := '02: S''ha intentat inserir un valor NULL.' || SUBSTR(SQLERRM,INSTR(SQLERRM,'('));
                    WHEN -20000 THEN
                        v_resultat := '03: Pas no trobat amb CODI = ' || p_codi;
                    WHEN -20001 THEN
                        v_resultat := '04: Ha d''informar el codi del pas a actualitzar.';
                    ELSE
                        -- Error no controlat
                        v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
                END CASE;            

	-- S'assignen els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat;
        
	DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
				
END UPD_PAISOS;
---------------------------------------------------------------------------------


---------------------------------------------------------------------------------
-- CU14-03: Esborrar Pas
-- Parmetres d'entrada:
--		* p_codi: Codi de Pas
-- Parmetres de sortida: 
--		* p_resultat: Resultat del procediment OK/KO
---------------------------------------------------------------------------------
PROCEDURE DEL_PAISOS(p_codi IN PFCBD.PAISOS.codpais%TYPE, p_resultat OUT VARCHAR)
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
BEGIN

	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.DEL_PAISOS';
	v_paramEnt       := 'p_codi = ' || p_codi;
					  
					  
	-- Es verifica que s'indica el codi de pas a esborrar
	IF p_codi is null then
		RAISE_APPLICATION_ERROR(-20000,'');
	end if;

	DELETE PAISOS
	WHERE codpais = upper(p_codi);

	-- Comprovem si s'ha trobat el registre a esborrar
	IF SQL%rowcount = 0 THEN
		RAISE_APPLICATION_ERROR(-20001,'');
	ELSE
		v_resultat := '00: Pas esborrat correctament';
	END IF;

	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat;
	      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
  
	EXCEPTION
	WHEN OTHERS THEN
		CASE (SQLCODE)
            WHEN -20000 THEN
				   v_resultat := '01: Ha d''informar el codi del pas a esborrar.';
			WHEN -20001 THEN
				   v_resultat := '03: Pas no trobat amb codi = ' || p_codi;
            WHEN -2292 THEN
				-- Restricci d'integritat
                   v_resultat := '04: S''ha intentat esborrar un registre amb dependncies.' || SUBSTR(SQLERRM,INSTR(SQLERRM,'('));
            ELSE
                -- Error no controlat
                   v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
        END CASE;            
		
	-- S'assignen els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat;
        
	DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
		
END DEL_PAISOS;
---------------------------------------------------------------------------------


---------------------------------------------------------------------------------
-- CU14-04: Consultar Pas/Pasos
-- Parmetres d'entrada:
--		* p_codi: Codi de Pas
--		* p_nom: Nom del Pas
-- Parmetres de sortida:
--		* p_rs: Recordset amb el resultat
--		* p_resultat: Resultat del procediment OK/KO
-- Sempre que s'introdueixi l'identificador per realitzar la consulta es consultar la taula de pasos per l'identificador.
-- Es permetr realitzar la consulta de pais per nom.
-- Si no s'especifica cap parmetre s'extreuran tots els pasos del sistema
---------------------------------------------------------------------------------
PROCEDURE CNS_PAISOS(p_rs IN OUT sys_refcursor, 	
						p_codi IN PFCBD.PAISOS.codpais%TYPE, 	
						p_nom IN PFCBD.PAISOS.NOM%TYPE, p_resultat OUT VARCHAR)
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;

BEGIN
	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.CNS_PAISOS';
	v_paramEnt       := 'p_codi = ' || p_codi 
                      || ', p_nom = ' || p_nom;
	
	if p_codi is not NULL then
			OPEN p_rs FOR
			SELECT *
			FROM PAISOS
			where codpais = p_codi;
	
	ELSIF p_nom is not null then
			OPEN p_rs FOR
			SELECT *
			FROM PAISOS
			where nom like '%' || upper(p_nom) || '%';
	ELSE
			OPEN p_rs FOR
			SELECT *
			FROM PAISOS;
	end if;
			
-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat || ', p_rs: Recordset amb el resultat de la consulta.';
	v_resultat := '00: Consulta realitzada correctament';
      
-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	

     
EXCEPTION
            WHEN OTHERS THEN
                v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
                
  
	-- Es genera un recordset buit
    OPEN p_rs FOR
    SELECT * FROM DUAL WHERE DUMMY IS NULL;
	
	-- S'assignen els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat || ', p_rs: Recordset buit.';
        
	DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
				
END CNS_PAISOS;
---------------------------------------------------------------------------------

END DM_PAISOS;
/
