---------------------------------------------------------------------------------
-- Alumne: Jessica Villamor 
-- Consultor: Juan Martnez Bolaos
-- Projecte: GESTIO DE VIDEOJOCS
---------------------------------------------------------------------------------

-- Procediments CU15: Gesti de Provncies

--
-- Ens situem a l'esquema de la base de dades de Gesti de Videojocs
--
ALTER SESSION SET CURRENT_SCHEMA = PFCBD;

-- Creaci de la definici del paquet
Create or replace
PACKAGE    PFCBD.DM_PROVINCIES AS

		PROCEDURE INS_PROVINCIES( p_codPais IN PFCBD.PROVINCIES.CODPAIS%TYPE, p_codProv IN PFCBD.PROVINCIES.CODPROV%TYPE, p_nom IN PFCBD.PROVINCIES.NOM%TYPE, p_resultat OUT VARCHAR);

		PROCEDURE UPD_PROVINCIES ( p_codPais IN PFCBD.PROVINCIES.CODPAIS%TYPE, p_codProv IN PFCBD.PROVINCIES.CODPROV%TYPE, p_nom IN PFCBD.PROVINCIES.NOM%TYPE, p_resultat OUT VARCHAR);

		PROCEDURE DEL_PROVINCIES ( p_codPais IN PFCBD.PROVINCIES.CODPAIS%TYPE, p_codProv IN PFCBD.PROVINCIES.CODPROV%TYPE, p_resultat OUT VARCHAR);

		PROCEDURE CNS_PROVINCIES (p_rs IN OUT sys_refcursor, 	
								p_codPais IN PFCBD.PROVINCIES.CODPAIS%TYPE, p_codProv IN PFCBD.PROVINCIES.CODPROV%TYPE, p_nom  IN PFCBD.PROVINCIES.NOM%TYPE, p_resultat OUT VARCHAR);

END DM_PROVINCIES;
/


-- Creaci del cos del paquet
CREATE OR REPLACE PACKAGE BODY PFCBD.DM_PROVINCIES AS

---------------------------------------------------------------------------------
-- Procediments CU15: Gesti DM Provncies
---------------------------------------------------------------------------------


---------------------------------------------------------------------------------
-- CU15-01: Alta d'una provncia
-- Parmetres d'entrada:
--		* p_codPais: Codi de Pas
--		* p_codProv: Codi de Provncia
--		* p_nom: Nom de la provncia
-- Parmetres de sortida: 
--		* p_resultat: Resultat del procediment OK/KO
---------------------------------------------------------------------------------
PROCEDURE INS_PROVINCIES( p_codPais IN PFCBD.PROVINCIES.CODPAIS%TYPE, p_codProv IN PFCBD.PROVINCIES.CODPROV%TYPE, p_nom IN PFCBD.PROVINCIES.NOM%TYPE, p_resultat OUT VARCHAR)
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
  
BEGIN  

	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.INS_PROVINCIES';
	v_paramEnt       := 'p_codPais = ' || p_codPais 
                      || ', p_codProv = ' || p_codProv
					  || ', p_nom = ' || p_nom;
 
	-- S'inserta la nova provncia
	INSERT INTO PROVINCIES(CODPAIS, CODPROV, NOM) VALUES (UPPER(p_codPais),UPPER(p_codProv), UPPER(p_nom));

	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat;
	v_resultat     := '00: Provncia creada correctament';
      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	

	
	EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE) 
                   WHEN -02291 THEN
                        -- Error d'integritat referencial 
                        v_resultat := '05: Error d''integritat referencial.';
                    WHEN -1 THEN
                        -- Registre duplicat
                        v_resultat := '01: La provncia ja existeix.';
                    WHEN -1400 THEN
                        -- Valor nul
                        v_resultat := '02: S''ha intentat inserir un valor NULL' || SUBSTR(SQLERRM,INSTR(SQLERRM,'('));
					ELSE
                        -- Error no controlat
                        v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
                END CASE;            

	-- S'assignen els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat;
        
	DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
				
END INS_PROVINCIES;
---------------------------------------------------------------------------------

---------------------------------------------------------------------------------
-- CU15-02: Actualitzar Provncia
-- Parmetres d'entrada:
--		* p_codPais: Codi de Pas
--		* p_codProv: Codi de Provncia
--		* p_nom: Nom de la Provncia
-- Parmetres de sortida: 
--		* p_resultat: Resultat del procediment OK/KO
---------------------------------------------------------------------------------
PROCEDURE UPD_PROVINCIES ( p_codPais IN PFCBD.PROVINCIES.CODPAIS%TYPE, p_codProv IN PFCBD.PROVINCIES.CODPROV%TYPE, p_nom IN PFCBD.PROVINCIES.NOM%TYPE, p_resultat OUT VARCHAR)
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
	
	v_codPais PFCBD.PROVINCIES.CODPAIS%TYPE;
	v_codProv PFCBD.PROVINCIES.CODPROV%TYPE;
BEGIN

	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.UPD_PROVINCIES';
	v_paramEnt       := 'p_codPais = ' || p_codPais 
                      || ', p_codProv = ' || p_codProv
					  || ', p_nom = ' || p_nom;
					  
	-- Es verifica que la provncia esta donada d'alta al sistema
	BEGIN
		IF p_codPais IS NOT NULL and p_codProv is not null THEN
			SELECT CODPAIS, CODPROV into v_codPais, v_codProv
					  FROM PROVINCIES
					  WHERE CODPAIS = upper(p_codPais) AND CODPROV = upper(p_codProv);
		END IF;
		EXCEPTION
			WHEN OTHERS THEN
				 IF (SQLCODE = 100) THEN
					   -- Codi provncia no trobada
					   RAISE_APPLICATION_ERROR(-20000,'');
				 END IF; 
	END;  
    
	if p_codPais is null or p_codProv is null then
			RAISE_APPLICATION_ERROR(-20001,'');
    end if;
	
	UPDATE PROVINCIES SET NOM = upper(p_nom) WHERE PROVINCIES.CODPAIS = upper(p_codPais) and PROVINCIES.CODPROV = upper(p_codProv) ;
    
	IF SQL%rowcount = 0 THEN
		RAISE_APPLICATION_ERROR(-20000,'');
	ELSE
		v_resultat := '00: Dades de provncia actualitzades correctament';
	END IF;
  
	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat;
	      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	

	EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                    WHEN -1407 THEN
                        -- Valor nul
                        v_resultat := '02: S''ha intentat inserir un valor NULL.' || SUBSTR(SQLERRM,INSTR(SQLERRM,'('));
                    WHEN -20000 THEN
                        v_resultat := '03: Provncia no trobada Prov: ' || p_codProv || ' Pas: ' || p_codPais;
                    WHEN -20001 THEN
                        v_resultat := '04: Ha d''informar el codi del pas i provncia a actualitzar.';
                    ELSE
                        -- Error no controlat
                        v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
                END CASE;            

	-- S'assignen els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat;
        
	DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
				
END UPD_PROVINCIES;
---------------------------------------------------------------------------------


---------------------------------------------------------------------------------
-- CU15-03: Esborrar Provncia
-- Parmetres d'entrada:
--		* p_codPais: Codi de Pas
--		* p_codProv: Codi de Provncia
-- Parmetres de sortida: 
--		* p_resultat: Resultat del procediment OK/KO
---------------------------------------------------------------------------------
PROCEDURE DEL_PROVINCIES ( p_codPais IN PFCBD.PROVINCIES.CODPAIS%TYPE, p_codProv IN PFCBD.PROVINCIES.CODPROV%TYPE, p_resultat OUT VARCHAR)
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
BEGIN

	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.DEL_PROVINCIES';
	v_paramEnt       := 'p_codPais = ' || p_codPais 
                      || ', p_codProv = ' || p_codProv;
					  
					  
	if p_codPais is null or p_codProv is null then
			RAISE_APPLICATION_ERROR(-20001,'');
    end if;
	
	DELETE PROVINCIES
	WHERE codpais = upper(p_codPais) and codprov = upper(p_codProv);

	-- Comprovem si s'ha trobat el registre a esborrar
	IF SQL%rowcount = 0
	THEN
		RAISE_APPLICATION_ERROR(-20000,'');
	ELSE
		v_resultat := '00: Provncia esborrada correctament';
	END IF;

  
	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat;
      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	

  
	EXCEPTION
		WHEN OTHERS THEN
			CASE (SQLCODE)
			   WHEN -20000 THEN
						v_resultat := '03: Provncia no trobada amb codi: ' || p_codProv ||'('|| p_codPais ||')';
			   WHEN -20001 THEN
						v_resultat := '04: Ha d''informar el codi del pas i provncia a esborrar.';
			   WHEN -2292 THEN
					-- Restricci d'integritat
					   v_resultat := '04: S''ha intentat esborrar un registre amb dependncies.' || SUBSTR(SQLERRM,INSTR(SQLERRM,'('));
				ELSE
					-- Error no controlat
					   v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
			END CASE;            
		
	-- S'assignem els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat;
        
	DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
		
END DEL_PROVINCIES;
---------------------------------------------------------------------------------


---------------------------------------------------------------------------------
-- CU15-04: Consultar Provncies
-- Parmetres d'entrada:
--		* p_codPais: Codi de Pas
--		* p_codProv: Codi de Provncia
--		* p_nom: Nom de la Provncia
-- Parmetres de sortida:
--		* p_rs: Recordset amb el resultat
--		* p_resultat: Resultat del procediment OK/KO
-- Es permetr realitzar consultes per: 
-- * Identificador de la Provncia
-- * A partir del codi de pas
-- * A partir del codi de pas i nom de la provncia
---------------------------------------------------------------------------------
PROCEDURE CNS_PROVINCIES (p_rs IN OUT sys_refcursor, 	
						p_codPais IN PFCBD.PROVINCIES.CODPAIS%TYPE, p_codProv IN PFCBD.PROVINCIES.CODPROV%TYPE, p_nom  IN PFCBD.PROVINCIES.NOM%TYPE, p_resultat OUT VARCHAR)
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;

BEGIN
	
	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.CNS_PROVINCIES';
	v_paramEnt       := 'p_codPais = ' || p_codPais 
                      || ', p_codProv = ' || p_codProv
					  || ', p_nom = ' || p_nom;
	
	IF p_codPais IS NOT NULL AND p_codProv IS NOT NULL THEN
			OPEN p_rs FOR
			SELECT *
			FROM PROVINCIES
			where codpais = UPPER(p_codPais) AND CODPROV = p_codProv;
	ELSIF p_codPais is not null AND p_nom IS NULL then
			OPEN p_rs FOR
			SELECT *
			FROM PROVINCIES
			where codpais = UPPER(p_codPais);
	ELSIF p_codPais is not null AND p_nom IS NOT NULL THEN
			OPEN p_rs FOR
			SELECT *
			FROM PROVINCIES
			where codpais = UPPER(p_codPais) and (nom like '%' || upper(p_nom) || '%');
	ELSE
		RAISE_APPLICATION_ERROR(-20000,'');
	end if;

	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat || ', p_rs: Recordset amb el resultat de la consulta.';
	v_resultat := '00: Consulta realitzada correctament';
      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
    
EXCEPTION
        WHEN OTHERS THEN
				CASE (SQLCODE)
				        WHEN -20000 THEN
                  v_resultat := '05: S''han de completar els criteris de selecci(CODPAIS i CODPROV / CODPAIS i NOM / CODPAIS)';
                ELSE
                  v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
				END CASE;
                
	-- Es genera un recordset buit
    OPEN p_rs FOR
    SELECT * FROM DUAL WHERE DUMMY IS NULL;
		
	-- S'assignen els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat || ', p_rs: Recordset buit.';
        
	DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
				
END CNS_PROVINCIES;
---------------------------------------------------------------------------------

END DM_PROVINCIES;
/
