---------------------------------------------------------------------------------
-- Alumne: Jessica Villamor 
-- Consultor: Juan Martnez Bolaos
-- Projecte: GESTIO DE VIDEOJOCS
---------------------------------------------------------------------------------

-- Procediments CU16: Gesti de Poblacions

--
-- Ens situem a l'esquema de la base de dades de Gesti de Videojocs
--
ALTER SESSION SET CURRENT_SCHEMA = PFCBD;

-- Creaci de la definici del paquet
Create or replace
PACKAGE    PFCBD.DM_POBLACIONS AS

		PROCEDURE INS_POBLACIONS( p_codPais IN PFCBD.POBLACIONS.CODPAIS%TYPE, p_codProv IN PFCBD.POBLACIONS.CODPROV%TYPE, p_codPob IN PFCBD.POBLACIONS.CODPOB%TYPE, p_nom IN PFCBD.POBLACIONS.NOM%TYPE, p_resultat OUT VARCHAR);

		PROCEDURE UPD_POBLACIONS ( p_codPais IN PFCBD.POBLACIONS.CODPAIS%TYPE, p_codProv IN PFCBD.POBLACIONS.CODPROV%TYPE, p_codPob IN PFCBD.POBLACIONS.CODPOB%TYPE, p_nom IN PFCBD.POBLACIONS.NOM%TYPE, p_resultat OUT VARCHAR);

		PROCEDURE DEL_POBLACIONS(p_codPais IN PFCBD.POBLACIONS.CODPAIS%TYPE, p_codProv IN PFCBD.POBLACIONS.CODPROV%TYPE, p_codPob IN PFCBD.POBLACIONS.CODPOB%TYPE, p_resultat OUT VARCHAR);

		PROCEDURE CNS_POBLACIONS(p_rs IN OUT sys_refcursor, 	
								p_codPais IN PFCBD.POBLACIONS.CODPAIS%TYPE, p_codProv IN PFCBD.POBLACIONS.CODPROV%TYPE, p_codPob  IN PFCBD.POBLACIONS.CODPOB%TYPE, p_nom  IN PFCBD.POBLACIONS.NOM%TYPE, p_resultat OUT VARCHAR);

END DM_POBLACIONS;
/


-- Creaci del cos del paquet
CREATE OR REPLACE PACKAGE BODY PFCBD.DM_POBLACIONS AS


---------------------------------------------------------------------------------
-- Procediments CU16: Gesti DM Poblacions
---------------------------------------------------------------------------------


---------------------------------------------------------------------------------
-- CU16-01: Alta d'una poblaci
-- Parmetres d'entrada:
--		* p_codPais: Codi de Pas
--		* p_codProv: Codi de Provncia
--		* p_codPob: Codi de Poblaci
--		* p_nom: Nom de la Poblaci
-- Parmetres de sortida: 
--		* p_resultat: Resultat del procediment OK/KO
---------------------------------------------------------------------------------
PROCEDURE INS_POBLACIONS( p_codPais IN PFCBD.POBLACIONS.CODPAIS%TYPE, p_codProv IN PFCBD.POBLACIONS.CODPROV%TYPE, p_codPob IN PFCBD.POBLACIONS.CODPOB%TYPE, p_nom IN PFCBD.POBLACIONS.NOM%TYPE, p_resultat OUT VARCHAR)
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
 
BEGIN
	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.INS_POBLACIONS';
	v_paramEnt       := 'p_codPais = ' || p_codPais 
                      || ', p_codProv = ' || p_codProv
					  || ', p_codPob = ' || p_codPob
					  || ', p_nom = ' || p_nom;
	
   
	-- S'inserta la nova poblaci
	INSERT INTO POBLACIONS(CODPAIS, CODPROV, CODPOB, NOM) VALUES (UPPER(p_codPais),UPPER(p_codProv), UPPER(p_codPob), UPPER(p_nom));

	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat;
	v_resultat     := '00: Poblaci creada correctament';
		      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
		
	EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE) 
                    WHEN -1 THEN
                        -- Registre duplicat
                        v_resultat := '01: La poblaci ja existeix.';
                    WHEN -1400 THEN
                        -- Valor nulo
                        v_resultat := '02: S''ha intentat inserir un valor NULL' || SUBSTR(SQLERRM,INSTR(SQLERRM,'('));
                    WHEN -02291 THEN
                        -- Error d'integritat referencial 
                        v_resultat := '05: Error d''integritat referencial.';
                    ELSE
                        -- Error no controlat
                        v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
                END CASE;            

	-- S'assignem els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat;
        
	DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
				
END INS_POBLACIONS;
---------------------------------------------------------------------------------


---------------------------------------------------------------------------------
-- CU16-02: Actualitzar Poblacions
-- Parmetres d'entrada:
--		* p_codPais: Codi de Pas
--		* p_codProv: Codi de Provncia
--		* p_codPob: Codi de Poblaci
--		* p_nom: Nom de la Poblaci
-- Parmetres de sortida: 
--		* p_resultat: Resultat del procediment OK/KO
---------------------------------------------------------------------------------
PROCEDURE UPD_POBLACIONS ( p_codPais IN PFCBD.POBLACIONS.CODPAIS%TYPE, p_codProv IN PFCBD.POBLACIONS.CODPROV%TYPE, p_codPob IN PFCBD.POBLACIONS.CODPOB%TYPE, p_nom IN PFCBD.POBLACIONS.NOM%TYPE, p_resultat OUT VARCHAR)
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
	
	v_codPais PFCBD.POBLACIONS.CODPAIS%TYPE;
	v_codProv PFCBD.POBLACIONS.CODPROV%TYPE;
	v_codPob PFCBD.POBLACIONS.CODPOB%TYPE;
 
BEGIN

	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.UPD_POBLACIONS';
	v_paramEnt       := 'p_codPais = ' || p_codPais 
                      || ', p_codProv = ' || p_codProv
					  || ', p_codPob = ' || p_codPob
					  || ', p_nom = ' || p_nom;
					  
	
	-- Es verifica que la poblaci esta donada d'alta al sistema
	BEGIN
		IF p_codPais IS NOT NULL and p_codProv is not null and  p_codPob is not null THEN
			SELECT CODPAIS, CODPROV, CODPOB into v_codPais, v_codProv, v_codPob
			FROM POBLACIONS
			WHERE CODPAIS = upper(p_codPais) AND (CODPROV = p_codProv) AND (CODPOB = p_codPob);
	   
		END IF;
		EXCEPTION
			WHEN OTHERS THEN
				 IF (SQLCODE = 100) THEN
					   -- Codi poblaci no trobada
					   RAISE_APPLICATION_ERROR(-20000,'');
				 END IF; 
	END;  
    
	if (p_codPais is null or p_codProv is null or p_codPob is null ) then
			RAISE_APPLICATION_ERROR(-20001,'');
    end if;
	
	UPDATE POBLACIONS SET NOM = upper(p_nom) WHERE (POBLACIONS.CODPAIS = v_codPais and POBLACIONS.CODPROV = v_codProv and POBLACIONS.CODPOB = v_codPob);
    
	IF SQL%rowcount = 0 THEN
		RAISE_APPLICATION_ERROR(-20000,'');
	ELSE
		v_resultat := '00: Dades de poblaci actualitzades correctament';
	END IF;
    			  
	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat;
      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	


	EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                    WHEN -1407 THEN
                        -- Valor nul
                        v_resultat := '02: S''ha intentat inserir un valor NULL.' || SUBSTR(SQLERRM,INSTR(SQLERRM,'('));
                    WHEN -20000 THEN
                        v_resultat := '03: Poblaci no trobada Pob: ' || p_codPob || ' Provncia: ' || p_codProv || '('|| p_codPais ||')';
                    WHEN -20001 THEN
                        v_resultat := '04: Ha d''informar el codi del pas, provncia i poblaci a actualitzar.';
                    ELSE
                        -- Error no controlat
                        v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
                END CASE;            

	-- S'assignen els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat;
        
	DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
				
END UPD_POBLACIONS;
---------------------------------------------------------------------------------


---------------------------------------------------------------------------------
-- CU16-03: Esborrar Poblaci
-- Parmetres d'entrada:
--		* p_codPais: Codi de Pas
--		* p_codProv: Codi de Provncia
--		* p_codPob: Codi de Poblaci
-- Parmetres de sortida: 
--		* p_resultat: Resultat del procediment OK/KO
---------------------------------------------------------------------------------
PROCEDURE DEL_POBLACIONS(p_codPais IN PFCBD.POBLACIONS.CODPAIS%TYPE, p_codProv IN PFCBD.POBLACIONS.CODPROV%TYPE, p_codPob IN PFCBD.POBLACIONS.CODPOB%TYPE, p_resultat OUT VARCHAR)
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
BEGIN

	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.DEL_POBLACIONS';
	v_paramEnt       := 'p_codPais = ' || p_codPais 
                      || ', p_codProv = ' || p_codProv
					  || ', p_codPob = ' || p_codPob;
					  
	if p_codPais is null or p_codProv is null or p_codPob is null then
			RAISE_APPLICATION_ERROR(-20001,'');
    end if;
	
	DELETE POBLACIONS
	WHERE codpais = upper(p_codPais) and codprov = upper(p_codProv) and codpOB = upper(p_codPob);

	-- Comprovem si s'ha trobat el registre a esborrar
	IF SQL%rowcount = 0 THEN
		RAISE_APPLICATION_ERROR(-20000,'');
	ELSE
		v_resultat := '00: Poblaci esborrada correctament';
	END IF;

	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat;
      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	

  
EXCEPTION
	WHEN OTHERS THEN
		CASE (SQLCODE)
            WHEN -20000 THEN
                v_resultat := '03: Poblaci no trobada amb codi: ' || p_codPob || ' Provncia: ' || p_codProv || '('|| p_codPais ||')';
			WHEN -20001 THEN
                v_resultat := '04: Ha d''informar el codi del pas, provncia i poblaci a esborrar.';
            WHEN -2292 THEN
				-- Restricci d'integritat
                v_resultat := '04: S''ha intentat esborrar un registre amb dependncies.' || SUBSTR(SQLERRM,INSTR(SQLERRM,'('));
            ELSE
                -- Error no controlat
                v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
        END CASE;            
		
	-- S'assignem els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat;
        
	DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
		
END DEL_POBLACIONS;
---------------------------------------------------------------------------------


---------------------------------------------------------------------------------
-- CU16-04: Consultar Poblacions
-- Parmetres d'entrada:
--		* p_codPais: Codi de Pas
--		* p_codProv: Codi de Provncia
--		* p_codPob: Codi de Poblaci
--		* p_nom: Nom de la Poblaci
-- Parmetres de sortida:
--		* p_rs: Recordset amb el resultat
--		* p_resultat: Resultat del procediment OK/KO
-- Es permetr realitzar consultes per: 
-- * Identificador de la Poblaci
-- * A partir del codi de pas/provncia
-- * A partir del codi de pas/provncia i nom de la poblaci
---------------------------------------------------------------------------------
PROCEDURE CNS_POBLACIONS(p_rs IN OUT sys_refcursor, 	
						p_codPais IN PFCBD.POBLACIONS.CODPAIS%TYPE, p_codProv IN PFCBD.POBLACIONS.CODPROV%TYPE, p_codPob  IN PFCBD.POBLACIONS.CODPOB%TYPE, p_nom  IN PFCBD.POBLACIONS.NOM%TYPE, p_resultat OUT VARCHAR)
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;

BEGIN
	
	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.CNS_POBLACIONS';
	v_paramEnt       := 'p_codPais = ' || p_codPais 
                      || ', p_codProv = ' || p_codProv
					  || ', p_codPob = ' || p_codPob
					  || ', p_nom = ' || p_nom;
					  
	IF p_codPais IS NOT NULL AND p_codProv IS NOT NULL AND p_codPob IS NOT NULL THEN
			OPEN p_rs FOR
			SELECT *
			FROM POBLACIONS
			where codpais = UPPER(p_codPais) AND CODPROV = p_codProv AND CODPOB = p_codPob;
	ELSIF p_codPais is not null AND p_codProv IS NOT NULL AND p_nom IS NULL then
			OPEN p_rs FOR
			SELECT *
			FROM POBLACIONS
			where codpais = UPPER(p_codPais) AND CODPROV = p_codProv;
	ELSIF p_codPais is not null AND p_codProv IS NOT NULL AND p_nom IS NOT NULL THEN
			OPEN p_rs FOR
			SELECT *
			FROM POBLACIONS
			where codpais = UPPER(p_codPais) AND CODPROV = p_codProv and (nom like '%' || upper(p_nom) || '%');
	ELSE
		RAISE_APPLICATION_ERROR(-20000,'');
	end if;
			
	
	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat || ', p_rs: Recordset amb el resultat de la consulta.';
	v_resultat := '00: Consulta realitzada correctament';
      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
     
	EXCEPTION
        WHEN OTHERS THEN
				CASE (SQLCODE)
				        WHEN -20000 THEN
                  v_resultat := '05: S''han de completar els criteris de selecci(CODPAIS CODPROV CODPOB/ CODPAIS CODPROV NOM / CODPAIS CODPROV)"';
                ELSE
                  v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
				END CASE;
                
-- Es genera un recordset buit
    OPEN p_rs FOR
    SELECT * FROM DUAL WHERE DUMMY IS NULL;
		
-- S'assignem els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat || ', p_rs: Recordset amb el resultat de la consulta.';
        
	DBMS_OUTPUT.put_line (v_resultat);				
-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
				
END CNS_POBLACIONS;
---------------------------------------------------------------------------------

END DM_POBLACIONS;
/
