---------------------------------------------------------------------------------
-- Alumne: Jessica Villamor 
-- Consultor: Juan Martnez Bolaos
-- Projecte: GESTIO DE VIDEOJOCS
---------------------------------------------------------------------------------

-- Procediments CU17: Gesti d'Estats de collaboraci

--
-- Ens situem a l'esquema de la base de dades de Gesti de Videojocs
--
ALTER SESSION SET CURRENT_SCHEMA = PFCBD;

-- Creaci de la definici del paquet
Create or replace
PACKAGE    PFCBD.DM_ESTATS AS
		
		PROCEDURE INS_ESTATS( p_nom IN PFCBD.ESTATS.NOM%TYPE, p_desc IN PFCBD.ESTATS.DESCRIPTIU%TYPE, p_nivant IN PFCBD.ESTATS.ESTANT_ID%TYPE, p_resultat OUT VARCHAR);
		
		PROCEDURE UPD_ESTATS ( p_id IN PFCBD.ESTATS.ID%TYPE, p_nom IN PFCBD.ESTATS.NOM%TYPE, p_desc IN PFCBD.ESTATS.DESCRIPTIU%TYPE, p_nivant IN PFCBD.ESTATS.ESTANT_ID%TYPE, p_resultat OUT VARCHAR);
		
		PROCEDURE DEL_ESTATS(p_id IN PFCBD.ESTATS.ID%TYPE, p_resultat OUT VARCHAR);
		
		PROCEDURE CNS_ESTATS(p_rs IN OUT sys_refcursor, p_id IN PFCBD.ESTATS.ID%TYPE,  p_nom IN PFCBD.ESTATS.NOM%TYPE, p_resultat OUT VARCHAR);

END DM_ESTATS;
/


-- Creaci del cos del paquet
CREATE OR REPLACE PACKAGE BODY PFCBD.DM_ESTATS AS

---------------------------------------------------------------------------------
-- Procediments CU17: Gesti DM Estats
---------------------------------------------------------------------------------


---------------------------------------------------------------------------------
-- CU17-01: Alta d'un estat
-- Parmetres d'entrada:
--		* p_nom: Nom de l'estat
--		* p_desc: Descripci de l'estat
--		* p_nivant: Nivell anterior
-- Parmetres de sortida: 
--		* p_resultat: Resultat del procediment OK/KO
---------------------------------------------------------------------------------
PROCEDURE INS_ESTATS( p_nom IN PFCBD.ESTATS.NOM%TYPE, p_desc IN PFCBD.ESTATS.DESCRIPTIU%TYPE, p_nivant IN PFCBD.ESTATS.ESTANT_ID%TYPE, p_resultat OUT VARCHAR)
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
	
	v_antniv PFCBD.ESTATS.ESTANT_ID%TYPE;
 
BEGIN
	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.INS_ESTATS';
	v_paramEnt       := 'p_nom = ' || p_nom 
                      || ', p_desc = ' || p_desc
					  || ', p_nivant = ' || p_nivant;
					  
	-- Es verifica que el segent nivell ja existeix al sistema
	BEGIN
		IF p_nivant IS NOT NULL THEN
				SELECT ID INTO v_antniv
				FROM ESTATS
				WHERE ID = p_nivant;
		END IF;    
		
			EXCEPTION
				WHEN OTHERS THEN
					IF (SQLCODE = 100) THEN
						-- Estat no trobat
						RAISE_APPLICATION_ERROR(-20000,'');
					END IF; 
	END; 
   
	INSERT INTO ESTATS(NOM, DESCRIPTIU, ESTANT_ID) VALUES (UPPER(p_nom), p_desc, v_antniv);

	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat;
	v_resultat     := '00: Estat creat correctament';
      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	

	EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE) 
                    WHEN -1 THEN
                        -- Registre duplicat
                        v_resultat := '01: L''estat ja existeix.';
                    WHEN -1400 THEN
                        -- Valor nulo
                        v_resultat := '02: S''ha intentat inserir un valor NULL' || SUBSTR(SQLERRM,INSTR(SQLERRM,'('));
                    WHEN -02291 THEN
                        -- Error d'integritat referencial 
                        v_resultat := '05: Error d''integritat referencial.';
                    WHEN -20000 THEN
                         v_resultat := '03: Estat no trobat amb ID = ' || p_nivant;
					ELSE
                        -- Error no controlat
                        v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
                END CASE;            

	-- S'assignen els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat;
        
	DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
				
END INS_ESTATS;
---------------------------------------------------------------------------------


---------------------------------------------------------------------------------
-- CU17-02: Actualitzar un estat
-- Parmetres d'entrada:
--		* p_id: Identificador de l'estat
--		* p_nom: Nom de l'estat
--		* p_desc: Descripci de l'estat
--		* p_nivant: Nivell anterior
-- Parmetres de sortida: 
--		* p_resultat: Resultat del procediment OK/KO
---------------------------------------------------------------------------------
PROCEDURE UPD_ESTATS ( p_id IN PFCBD.ESTATS.ID%TYPE, p_nom IN PFCBD.ESTATS.NOM%TYPE, p_desc IN PFCBD.ESTATS.DESCRIPTIU%TYPE, p_nivant IN PFCBD.ESTATS.ESTANT_ID%TYPE, p_resultat OUT VARCHAR)
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
	
	v_id PFCBD.ESTATS.ID%TYPE;
	v_antid PFCBD.ESTATS.ESTANT_ID%TYPE;
BEGIN

	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.UPD_ESTATS';
	v_paramEnt       := 'p_id = ' || p_id 
                      || ', p_nom = ' || p_nom
					  || ', p_desc = ' || p_desc
					  || ', p_nivant = ' || p_nivant;
					  
	IF p_id IS NULL THEN
			RAISE_APPLICATION_ERROR(-20001,'');
	end if;
    
	-- Es verifica que l'estat esta donat d'alta al sistema     	
	BEGIN
		select ID into v_id
		from ESTATS
		where id = p_id;
			
		EXCEPTION
			WHEN OTHERS THEN
				 IF (SQLCODE = 100) THEN
					   -- Estat a actualitzar no trobat
					   RAISE_APPLICATION_ERROR(-20000,'');
				 END IF; 
		end;
	
	-- Es verifica que si s'ha informat l'estat anterior, aquest estigui donat d'alta al sistmea
	BEGIN
		IF p_nivant IS NOT NULL THEN
			select ID into v_antid
			from ESTATS
			where id = p_nivant;
	   
		 END IF;
		  EXCEPTION
			WHEN OTHERS THEN
				 IF (SQLCODE = 100) THEN
					   -- Estat Anterior no trobat
					   RAISE_APPLICATION_ERROR(-20002,'');
				 END IF; 
	END;  

	UPDATE ESTATS SET NOM = upper(p_nom) , descriptiu = p_desc, ESTANT_ID = v_antid WHERE (ESTATS.ID = p_id);
    
	IF SQL%rowcount = 0 THEN
		RAISE_APPLICATION_ERROR(-20000,'');
	ELSE
		v_resultat := '00: Estat actualitzat correctament';
	END IF;
  
  
	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat;
      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	

	EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                    WHEN -1 THEN
                        -- Registre duplicat
                        v_resultat := '01: L''estat ja existeix.';
                    WHEN -1407 THEN
                        -- Valor nul
                        v_resultat := '02: S''ha intentat inserir un valor NULL.' || SUBSTR(SQLERRM,INSTR(SQLERRM,'('));
                    WHEN -20000 THEN
                        v_resultat := '03: Estat no trobat Id: ' || p_id;
                    WHEN -20001 THEN
                        v_resultat := '04: Ha d''informar l''identificador de l''estat a actualitzar.';
                    WHEN -20002 THEN
                        v_resultat := '03: Estat Anterior no trobat Id: ' || p_nivant;
                    ELSE
                        -- Error no controlat
                        v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
                END CASE;            

	-- S'assignem els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat;
        
	DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
				
END UPD_ESTATS;
---------------------------------------------------------------------------------


---------------------------------------------------------------------------------
-- CU17-03: Esborrar Estat
-- Parmetres d'entrada:
--		* p_id: Identificador de l'estat
-- Parmetres de sortida: 
--		* p_resultat: Resultat del procediment OK/KO
---------------------------------------------------------------------------------
PROCEDURE DEL_ESTATS(p_id IN PFCBD.ESTATS.ID%TYPE, p_resultat OUT VARCHAR)
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
BEGIN

	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.DEL_ESTATS';
	v_paramEnt       := 'p_id = ' || p_id;
					  
	if p_id is null then
			RAISE_APPLICATION_ERROR(-20001,'');
    end if;
	
	DELETE ESTATS
	WHERE ID = p_id;

	-- Comprovem si s'ha trobat el registre a esborrar
	IF SQL%rowcount = 0 THEN
		RAISE_APPLICATION_ERROR(-20000,'');
	ELSE
		v_resultat := '00: Estat esborrat correctament';
	END IF;
  
	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat;
	      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
  
EXCEPTION
	WHEN OTHERS THEN
		CASE (SQLCODE)
            WHEN -20000 THEN
                   v_resultat := '03: Estat no trobat amb codi: ' || p_id;
            WHEN -20001 THEN
                   v_resultat := '04: Ha d''informar l''identificador de l''estat a esborrar.';
            WHEN -2292 THEN
				-- Restricci d'integritat
                   v_resultat := '04: S''ha intentat esborrar un registre amb dependncies.' || SUBSTR(SQLERRM,INSTR(SQLERRM,'('));
            ELSE
                -- Error no controlat
                   v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
        END CASE;            
		
	-- S'assignen els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat;
        
	DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
		
END DEL_ESTATS;
---------------------------------------------------------------------------------


---------------------------------------------------------------------------------
-- CU17-04: Consultar Estats
-- Parmetres d'entrada:
--		* p_id: Codi d'estat
--		* p_nom: Nom de l'estat
-- Parmetres de sortida:
--		* p_rs: Recordset amb el resultat
--		* p_resultat: Resultat del procediment OK/KO
-- La consulta d'estats permetr obtenir els estats en jerarquia a partir d'un estat concret identificat pels parmetres p_id/p_nom, o tots els estats a partir de l'inicial.
---------------------------------------------------------------------------------
PROCEDURE CNS_ESTATS(p_rs IN OUT sys_refcursor, p_id IN PFCBD.ESTATS.ID%TYPE,  p_nom IN PFCBD.ESTATS.NOM%TYPE, p_resultat OUT VARCHAR)
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;

BEGIN
	
	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.CNS_ESTATS';
	v_paramEnt       := 'p_id = ' || p_id;
	
	if p_id IS NULL AND p_nom IS NULL then
		OPEN p_rs FOR
    SELECT LEVEL, ESTANT_ID, id, nom, descriptiu
		FROM ESTATS
		START WITH ESTANT_ID is null
		CONNECT BY PRIOR id = ESTANT_ID; 	
	ELSE
	
		IF p_id IS NOT NULL THEN
			OPEN p_rs FOR
			SELECT LEVEL, ESTANT_ID, id, nom, descriptiu
				FROM ESTATS
				START WITH ID = P_ID
				CONNECT BY PRIOR id = ESTANT_ID; 	
				
		ELSE
			OPEN p_rs FOR
			SELECT LEVEL, ESTANT_ID, id, nom, descriptiu
				FROM ESTATS
				START WITH NOM = p_nom
				CONNECT BY PRIOR id = ESTANT_ID; 	
		END IF;
	END IF;	
			
			
	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat || ', p_rs: Recordset amb el resultat de la consulta.';
	v_resultat := '00: Consulta realitzada correctament';
      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
	
     
	EXCEPTION
        WHEN OTHERS THEN
            v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;

  
	-- Es genera un recordset buit
    OPEN p_rs FOR
    SELECT * FROM DUAL WHERE DUMMY IS NULL;
		
	-- S'assignen els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat || ', p_rs: Recordset buit.';
        
	DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
				
END CNS_ESTATS;
---------------------------------------------------------------------------------

END DM_ESTATS;
/
