---------------------------------------------------------------------------------
-- Alumne: Jessica Villamor 
-- Consultor: Juan Martnez Bolaos
-- Projecte: GESTIO DE VIDEOJOCS
---------------------------------------------------------------------------------

-- Procediments CU18:  Gesti de Xarxes Socials

--
-- Ens situem a l'esquema de la base de dades de Gesti de Videojocs
--
ALTER SESSION SET CURRENT_SCHEMA = PFCBD;

-- Creaci de la definici del paquet
Create or replace
PACKAGE    PFCBD.DM_XARXESSOCIALS AS

		PROCEDURE INS_XARXES_SOCIALS(p_nom IN PFCBD.XARXES_SOCIALS.NOM%TYPE, p_resultat OUT VARCHAR);

		PROCEDURE UPD_XARXES_SOCIALS(p_id IN PFCBD.XARXES_SOCIALS.ID%TYPE, p_nom IN PFCBD.XARXES_SOCIALS.NOM%TYPE, p_resultat OUT VARCHAR);

		PROCEDURE DEL_XARXES_SOCIALS(p_id IN PFCBD.XARXES_SOCIALS.ID%TYPE, p_resultat OUT VARCHAR);

		PROCEDURE CNS_XARXES_SOCIALS(p_rs IN OUT sys_refcursor, p_id IN PFCBD.XARXES_SOCIALS.ID%TYPE, p_nom IN PFCBD.XARXES_SOCIALS.NOM%TYPE, p_resultat OUT VARCHAR);

END DM_XARXESSOCIALS;
/


-- Creaci del cos del paquet
CREATE OR REPLACE PACKAGE BODY PFCBD.DM_XARXESSOCIALS AS

---------------------------------------------------------------------------------
-- Procediments CU18: Gesti DM Xarxes Socials
---------------------------------------------------------------------------------


---------------------------------------------------------------------------------
-- CU18-01: Alta d'una xarxa Social
-- Parmetres d'entrada:
--		* p_nom: Nom de la xarxa social
-- Parmetres de sortida: 
--		* p_resultat: Resultat del procediment OK/KO
---------------------------------------------------------------------------------
PROCEDURE INS_XARXES_SOCIALS(p_nom IN PFCBD.XARXES_SOCIALS.NOM%TYPE, p_resultat OUT VARCHAR)
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
 
BEGIN
	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.INS_XARXES_SOCIALS';
	v_paramEnt       := 'p_nom = ' || p_nom;
	
	  
	-- S'inserta la nova Xarxa Social
	INSERT INTO XARXES_SOCIALS(NOM) VALUES (UPPER(p_nom));

	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat;
	v_resultat     := '00: Xarxa Social creada correctament';
      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	

	EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE) 
                    WHEN -1 THEN
                        -- Registre duplicat
                        v_resultat := '01: La Xarxa Social ja existeix.';
                    WHEN -1400 THEN
                        -- Valor nul
                        v_resultat := '02: S''ha intentat inserir un valor NULL' || SUBSTR(SQLERRM,INSTR(SQLERRM,'('));
                    ELSE
                        -- Error no controlat
                        v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
                END CASE;            
			
	-- S'assignen els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat;
        
	DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);
				
END INS_XARXES_SOCIALS;
---------------------------------------------------------------------------------


---------------------------------------------------------------------------------
-- CU18-02: Actualitzar Xarxes Socials
-- Parmetres d'entrada:
--		* p_id: Id de la xarxa social
--		* p_nom: Nom de la xarxa social
-- Parmetres de sortida: 
--		* p_resultat: Resultat del procediment OK/KO
---------------------------------------------------------------------------------
PROCEDURE UPD_XARXES_SOCIALS(p_id IN PFCBD.XARXES_SOCIALS.ID%TYPE, p_nom IN PFCBD.XARXES_SOCIALS.NOM%TYPE, p_resultat OUT VARCHAR)
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
	
BEGIN
  
  	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.UPD_XARXES_SOCIALS';
	v_paramEnt       := 'p_id = ' || p_id 
                      || ', p_nom = ' || p_nom;
					  
	-- Es verifica que l'identificador est informat
	if (p_id is null) then
			RAISE_APPLICATION_ERROR(-20000,'');
    end if;
	
	UPDATE XARXES_SOCIALS SET NOM = upper(p_nom) WHERE (XARXES_SOCIALS.ID = p_id) ;
    
	IF SQL%rowcount = 0 THEN
		RAISE_APPLICATION_ERROR(-20001,'');
	ELSE
		v_resultat := '00: Xarxa Social actualitzada correctament.';
	END IF;
  
	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat;
	      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	

	EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE)
                    WHEN -1 THEN
                        -- Registre duplicat
                        v_resultat := '01: La Xarxa Social ja existeix.';
                    WHEN -1407 THEN
                        -- Valor nulo
                        v_resultat := '02: S''ha intentat inserir un valor NULL.' || SUBSTR(SQLERRM,INSTR(SQLERRM,'('));
                    WHEN -20000 THEN
                        v_resultat := '04: Ha d''informar l''identificador de la Xarxa Social a actualitzar.';
					WHEN -20001 THEN
                        v_resultat := '03: Xarxa Social no trobada Id: ' || p_id;
                    ELSE
                        -- Error no controlat
                        v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
                END CASE;            

  	-- S'assignen els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat;
        
	DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
				
END UPD_XARXES_SOCIALS;
---------------------------------------------------------------------------------


---------------------------------------------------------------------------------
-- CU18-03: Esborrar Xarxa Social
-- Parmetres d'entrada:
--		* p_id: Id de la xarxa social
-- Parmetres de sortida: 
--		* p_resultat: Resultat del procediment OK/KO
---------------------------------------------------------------------------------
PROCEDURE DEL_XARXES_SOCIALS(p_id IN PFCBD.XARXES_SOCIALS.ID%TYPE, p_resultat OUT VARCHAR)
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
BEGIN

	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.DEL_XARXES_SOCIALS';
	v_paramEnt       := 'p_id = ' || p_id;
					  
	if p_id is null then
			RAISE_APPLICATION_ERROR(-20000,'');
    end if;
	
	DELETE XARXES_SOCIALS
	WHERE ID = p_id;

	-- Comprovem si s'ha trobat el registre a esborrar
	IF SQL%rowcount = 0 THEN
		RAISE_APPLICATION_ERROR(-20001,'');
	ELSE
		v_resultat := '00: Xarxa Social esborrada correctament';
	END IF;

 
	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat;
	      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);
  
EXCEPTION
	WHEN OTHERS THEN
		CASE (SQLCODE)
            WHEN -20000 THEN
                        v_resultat := '04: Ha d''informar l''identificador de la Xarxa Social a esborrar.';
			WHEN -20001 THEN
                        v_resultat := '03: Xarxa Social no trobada amb codi Id: ' || p_id;
			WHEN -2292 THEN
				-- Restricci d'integritat
						v_resultat := '04: S''ha intentat esborrar un registre amb dependncies.' || SUBSTR(SQLERRM,INSTR(SQLERRM,'('));
            ELSE
                -- Error no controlat
						v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
        END CASE;            
		
	-- S'assignen els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat;
        
	DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
		
END DEL_XARXES_SOCIALS;
---------------------------------------------------------------------------------


---------------------------------------------------------------------------------
-- CU18-04: Consultar Xarxes Socials
-- Parmetres d'entrada:
--		* p_id: Identificador de la Xarxa Social
-- Parmetres de sortida:
--		* p_rs: Recordset amb el resultat
--		* p_resultat: Resultat del procediment OK/KO
-- Es permetr realitzar la consulta per l'identificador de la xarxa social. 
-- Si no s'informa cap criteri de selecci s'extrauran totes les xarxes socials donades d'alta al sistema.
---------------------------------------------------------------------------------
PROCEDURE CNS_XARXES_SOCIALS(p_rs IN OUT sys_refcursor, p_id IN PFCBD.XARXES_SOCIALS.ID%TYPE, p_nom IN PFCBD.XARXES_SOCIALS.NOM%TYPE, p_resultat OUT VARCHAR)
IS
	-- Valors per registrar al LOG	
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;

BEGIN
	-- S'assignen els valors d'entrada
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.CNS_XARXES_SOCIALS';
	v_paramEnt       := 'p_id = ' || p_id;
					  
	IF p_id IS NOT NULL THEN
			OPEN p_rs FOR
			SELECT *
			FROM XARXES_SOCIALS
			where id = p_id;
	ELSE 
		IF p_nom IS NOT NULL THEN
			OPEN p_rs FOR
			SELECT *
			FROM XARXES_SOCIALS
			where nom = p_nom;
		ELSE
			OPEN p_rs FOR
			SELECT *
			FROM XARXES_SOCIALS;
		END IF;
	end if;
			
	-- S'omplen els valors de sortida
    p_resultat := 'OK';
    v_dataFi    := SYSDATE;
	v_paramSor	:= 'p_resultat: ' || p_resultat || ', p_rs: Recordset amb el resultat de la consulta.';
	v_resultat := '00: Consulta realitzada correctament';
      
	-- Es grava al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
			
    
EXCEPTION
        WHEN OTHERS THEN
				    v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
				               
	-- Es genera un recordset buit
    OPEN p_rs FOR
    SELECT * FROM DUAL WHERE DUMMY IS NULL;
		
	-- S'assignen els valors de sortida 
	p_resultat := 'KO';
    v_dataFi    := SYSDATE;
    v_paramSor	:= 'p_resultat: ' || p_resultat || ', p_rs: Recordset buit.';
        
	DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
    GESTIO_LOG.INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);	
				
END CNS_XARXES_SOCIALS;
---------------------------------------------------------------------------------

END DM_XARXESSOCIALS;
/
