---------------------------------------------------------------------------------
-- Alumne: Jessica Villamor 
-- Consultor: Juan Martnez Bolaos
-- Projecte: GESTIO DE VIDEOJOCS
---------------------------------------------------------------------------------
--
-- Ens situem a l'esquema de la base de dades de Gesti de Videojocs
--
ALTER SESSION SET CURRENT_SCHEMA = PFCBD;

-- Creaci de la definici del paquet
Create or replace
PACKAGE    PFCBD.GESTIO_LOG AS
		PROCEDURE INS_LOG(p_dataInici IN PFCBD.LOG.DATAINICI%TYPE,
						  p_dataFi IN PFCBD.LOG.DATAFI%TYPE,
						  p_procediment IN PFCBD.LOG.PROCEDIMENT%TYPE,
						  p_paramEnt IN PFCBD.LOG.PARAMENTRADA%TYPE,
						  p_paramSor IN PFCBD.LOG.PARAMSORTIDA%TYPE,
						  p_resultat IN PFCBD.LOG.RESULTAT%TYPE);
						  
		PROCEDURE CNS_LOG( p_rs OUT sys_refcursor,
				  p_dataInici IN PFCBD.LOG.DATAINICI%TYPE,
				  p_dataFi IN PFCBD.LOG.DATAINICI%TYPE,
				  p_resultat OUT VARCHAR);
				  
END GESTIO_LOG;
/

-- Creaci del cos del paquet
CREATE OR REPLACE PACKAGE BODY PFCBD.GESTIO_LOG AS

---------------------------------------------------------------------------------
-- Procediments CU19: Registre de Log
---------------------------------------------------------------------------------


---------------------------------------------------------------------------------
-- CU19-01: Registre al log
-- Parmetres d'entrada:
--		* p_dataInici: Timestamp inici d'execuci del procediment
--		* p_dataFi: Timestamp fi d'execuci del procediment
--		* p_procediment: Nom del procediment
--		* p_paramEnt: Parmetres d'entrada
--		* p_paramSor: Parmetres de sortida
--		* p_resultat: Resultat del procediment
---------------------------------------------------------------------------------
PROCEDURE INS_LOG(p_dataInici IN PFCBD.LOG.DATAINICI%TYPE,
				  p_dataFi IN PFCBD.LOG.DATAFI%TYPE,
				  p_procediment IN PFCBD.LOG.PROCEDIMENT%TYPE,
				  p_paramEnt IN PFCBD.LOG.PARAMENTRADA%TYPE,
				  p_paramSor IN PFCBD.LOG.PARAMSORTIDA%TYPE,
				  p_resultat IN PFCBD.LOG.RESULTAT%TYPE)
IS
	 v_resultat PFCBD.LOG.RESULTAT%TYPE;
-- Aquesta transacci ha de ser autnoma per no interferir en la lgica de l'aplicaci
-- El commit noms li afectar a les seves sentncies.
   PRAGMA AUTONOMOUS_TRANSACTION;
BEGIN

	-- S'inserta el nou registre de log
	INSERT INTO LOG(DATAINICI, DATAFI, PROCEDIMENT, PARAMENTRADA, PARAMSORTIDA, RESULTAT) VALUES (p_dataInici, p_dataFi, p_procediment, p_paramEnt, p_paramSor, p_resultat);

	COMMIT;

EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE) 
                    WHEN -1400 THEN
                        -- Valor nul
                        v_resultat := '02: S''ha intentat inserir un valor NULL' || SUBSTR(SQLERRM,INSTR(SQLERRM,'('));
                    ELSE
                        -- Error no controlat
                        v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
                END CASE;            
			DBMS_OUTPUT.put_line (v_resultat);
END INS_LOG;
---------------------------------------------------------------------------------


---------------------------------------------------------------------------------
-- CU19-02: Consulta Log
-- Parmetres d'entrada:
--		* p_dataInici: Data inici del procediment
--		* p_dataFi: Data fi del procediment
-- Parmetres Sortida:
--		* p_rs: Recordset amb el resultat
--		* p_resultat: Resultat del procediment OK/KO
---------------------------------------------------------------------------------
PROCEDURE CNS_LOG( p_rs OUT sys_refcursor,
				  p_dataInici IN PFCBD.LOG.DATAINICI%TYPE,
				  p_dataFi IN PFCBD.LOG.DATAINICI%TYPE,
				  p_resultat OUT VARCHAR)
IS
	v_dataInici PFCBD.LOG.DATAINICI%TYPE;
	v_dataFi PFCBD.LOG.DATAFI%TYPE;
	v_procediment PFCBD.LOG.PROCEDIMENT%TYPE;
	v_paramEnt PFCBD.LOG.PARAMENTRADA%TYPE;
	v_paramSor PFCBD.LOG.PARAMSORTIDA%TYPE;
	v_resultat PFCBD.LOG.RESULTAT%TYPE;
	
BEGIN
	
	-- S'assignen els valors per gravar al LOG
	v_dataInici        := SYSDATE;   
	v_procediment := $$PLSQL_UNIT || '.CNS_LOG';
	v_paramEnt       := 'p_dataInici = ' || p_dataInici 
                      || ' ,p_dataFi = ' || p_dataFi;
    
	-- Es realitza la consulta demanada
        OPEN p_rs FOR
        SELECT * 
        FROM LOG
        WHERE datainici >= p_dataInici
          AND datafi <= p_dataFi;
        
	-- S'omplen els valors de sortida, no es registren abans per si hi ha un error en la consulta
        p_resultat := 'OK';
		v_paramSor	:= 'p_resultat: ' || p_resultat || ', p_rs: Es retonen els registrs de LOG en el cursor';
        v_dataFi    := SYSDATE;
		v_resultat     := '00: Consulta al LOG realitzada correctament';
        
	-- Es grava al LOG
       INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);		
	          
	EXCEPTION
            WHEN OTHERS THEN
                CASE (SQLCODE) 
                    WHEN -100 THEN
                        -- Valor nul
                        v_resultat := '02: Registre no trobat' || SUBSTR(SQLERRM,INSTR(SQLERRM,'('));
					ELSE
                        -- Error no controlat
                        v_resultat := '99: ' || SQLCODE || ' - ' || SQLERRM;
                END CASE;    

	-- Es genera un recordset buit 
        OPEN p_rs FOR
        SELECT * FROM DUAL WHERE DUMMY IS NULL;
				
	-- S'assignen els valors de sortida 
		p_resultat := 'KO';
		v_paramSor      := 'p_rs: Es retorna un cursor buit.';
		v_dataFi    := SYSDATE;
        DBMS_OUTPUT.put_line (v_resultat);				
	-- Gravem al LOG
        INS_LOG(v_dataInici, v_dataFi, v_procediment, v_paramEnt, v_paramSor, v_resultat);		

END CNS_LOG;
---------------------------------------------------------------------------------

END GESTIO_LOG;

/