---------------------------------------------------------------------------------
-- Alumne: Jessica Villamor 
-- Consultor: Juan Martnez Bolaos
-- Projecte: GESTIO DE VIDEOJOCS
---------------------------------------------------------------------------------

--
-- Ens situem a l'esquema del magatzem de dades
--
ALTER SESSION SET CURRENT_SCHEMA = PFCDW;


-- Creaci de la definici del paquet
CREATE 	OR REPLACE PACKAGE PFCDW.ETL AS

	-- Procediments per carregar Dimensions
	PROCEDURE CARREGA_D_JUGADORS;
	PROCEDURE CARREGA_D_NIVELLS;
	PROCEDURE CARREGA_D_TEMPS;
    PROCEDURE CARREGA_D_XARXESSOCIALS;
    PROCEDURE CARREGA_D_PRODUCTES;
	PROCEDURE CARREGA_D_ESTATS;
   

	-- Procediments per carregar Fets
    PROCEDURE CARREGA_F_COLLABORACIONS;
    PROCEDURE CARREGA_F_COMPRES (p_dataInici IN date, p_dataFi IN date);
    PROCEDURE CARREGA_F_PARTIDES (p_dataInici IN date, p_dataFi IN date);
    
    -- Procediment de crrega
	PROCEDURE CARREGA (p_dataInici IN date, p_dataFi IN date);
	
END ETL;
/


-- Creaci del cos del paquet
CREATE OR REPLACE PACKAGE BODY PFCDW.ETL AS

---------------------------------------------------------------------------------
-- Procediments ETL: Extracci, transformaci i Crrega del magatzem de dades
---------------------------------------------------------------------------------



---------------------------------------------------------------------------------
-- Crrega de la dimensi D_JUGADORS
---------------------------------------------------------------------------------
	PROCEDURE CARREGA_D_JUGADORS
    AS
    BEGIN
        -- Actualitzem les dades dels jugadors existents
        UPDATE (
				SELECT DJUG.Nom AS ANOM, concat(concat(JUG.Nom,nvl(concat(' ',JUG.primercog),'')),nvl(concat(' ',JUG.segoncog),'')) AS NNOM, DJUG.SEXE AS ASEXE, JUG.SEXE AS NSEXE
				FROM PFCBD.Jugadors JUG, PFCDW.D_JUGADORS DJUG
				WHERE DJUG.IdJugador = JUG.Id AND (DJUG.Nom <> concat(concat(JUG.Nom,nvl(concat(' ',JUG.primercog),'')),nvl(concat(' ',JUG.segoncog),''))  OR DJUG.SEXE <> JUG.SEXE )
				)
				SET ANOM = NNOM, ASEXE = NSEXE;
 		
		-- Afegim els Jugadors que no existeixen
        INSERT INTO PFCDW.D_JUGADORS
        SELECT ID, concat(concat(Nom,nvl(concat(' ',primercog),'')),nvl(concat(' ',segoncog),'')) as Nom, sexe
        FROM PFCBD.JUGADORS JUG
        WHERE JUG.Id NOT IN (SELECT IdJugador FROM PFCDW.D_JUGADORS);
    END CARREGA_D_JUGADORS;
---------------------------------------------------------------------------------


---------------------------------------------------------------------------------
-- Crrega de la dimensi D_NIVELLS
---------------------------------------------------------------------------------
	PROCEDURE CARREGA_D_NIVELLS
    AS
    BEGIN
        -- Actualitzem dels nivells que han canviat
        UPDATE (
				SELECT DNIV.Nom As ANomNiv, NIV.Nom As NNomNiv, DNIV.Joc As ANomJoc, JOC.Nom As NNomJoc
				FROM PFCBD.Nivells NIV, PFCDW.D_NIVELLS DNIV, PFCBD.Jocs JOC
				WHERE NIV.Id = DNIV.IdNivell 
					AND NIV.Jocs_id = JOC.Id
					and (DNIV.Nom <> NIV.Nom or dniv.joc <> JOC.NOM)
				)
				SET ANomNiv = NNomNiv, ANomJoc = NNomJoc;
 		
		-- Afegim els Nivells que no existeixen
        INSERT INTO D_NIVELLS
		SELECT n.id, j.nom, n.nom
		from pfcbd.nivells n, pfcbd.jocs j
		where n.jocs_id = j.id and n.id not in (select IdNivell from D_NIVELLS);
    END CARREGA_D_NIVELLS;
---------------------------------------------------------------------------------
	

---------------------------------------------------------------------------------
-- Crrega de la dimensi D_TEMPS
---------------------------------------------------------------------------------	
	PROCEDURE CARREGA_D_TEMPS
    AS
    BEGIN
    
       -- Inserim les dates al sistema
       INSERT INTO PFCDW.D_TEMPS 
        SELECT IdTemps, Dia, Mes, Anyo
        FROM (
              SELECT TO_CHAR(C.DataInici,'YYYYMMDD') AS IdTemps, TO_CHAR(C.DataInici,'DD') AS Dia, TO_CHAR(C.DataInici, 'MM') AS Mes, TO_CHAR(C.DataInici, 'YYYY') AS Anyo FROM PFCBD.COLLABORACIONS C
              UNION
              SELECT TO_CHAR(C.DataFi, 'YYYYMMDD') AS IdTemps, TO_CHAR(C.DataFi,'DD') AS Dia, TO_CHAR(C.DataFi, 'MM') AS Mes, TO_CHAR(C.DataFi, 'YYYY') AS Anyo FROM PFCBD.COLLABORACIONS C WHERE C.DataFi is not null
              UNION
              SELECT TO_CHAR(Com.DataCompra, 'YYYYMMDD') AS IdTemps, TO_CHAR(Com.DataCompra,'DD') AS Dia, TO_CHAR(Com.DataCompra, 'MM') AS Mes, TO_CHAR(Com.DataCompra, 'YYYY') AS Anyo FROM PFCBD.COMPRES Com
              UNION
              SELECT TO_CHAR(P.DataPartida, 'YYYYMMDD') AS IdTemps, TO_CHAR(P.DataPartida,'DD') AS Dia, TO_CHAR(P.DataPartida, 'MM') AS Mes, TO_CHAR(P.DataPartida, 'YYYY') AS Anyo FROM PFCBD.PARTIDES P
               ) 
        WHERE IdTemps NOT IN (SELECT Idtemps FROM PFCDW.D_TEMPS);
        
    END CARREGA_D_TEMPS;
---------------------------------------------------------------------------------

	
---------------------------------------------------------------------------------
-- Crrega de la dimensi D_XARXESSOCIALS
---------------------------------------------------------------------------------
	PROCEDURE CARREGA_D_XARXESSOCIALS
    AS
    BEGIN
        -- Actualitzem dels nivells que han canviat
        UPDATE (
				SELECT DXAR.Nom As ANomXar, XAR.Nom As NNomXar
				FROM PFCBD.XARXES_SOCIALS XAR, PFCDW.D_XARXESSOCIALS DXAR
				WHERE XAR.Id = DXAR.IdXarxa 
					and (DXAR.Nom <> XAR.Nom)
				)
				SET ANomXar = NNomXar;
 		
		-- Afegim les Xarxes Socials que no existeixen 
        INSERT INTO PFCDW.D_XARXESSOCIALS 
				(SELECT ID, Nom
				FROM PFCBD.XARXES_SOCIALS WHERE ID NOT IN (SELECT IdXarxa FROM PFCDW.D_XARXESSOCIALS));
    END CARREGA_D_XARXESSOCIALS;
---------------------------------------------------------------------------------	


---------------------------------------------------------------------------------
-- Crrega de la dimensi D_PRODUCTES
---------------------------------------------------------------------------------
	PROCEDURE CARREGA_D_PRODUCTES
    AS
    BEGIN
        -- Actualitzem dels Productes que han canviat
        UPDATE (
				SELECT DPRO.Nom As ANomPro, PRO.Nom As NNomPro, DPRO.Tipologia As ATipo, PRO.Tipologia As NTipo, DPRO.Preu As APreu, PRO.Preu As NPreu
				FROM PFCBD.PRODUCTES PRO, PFCDW.D_PRODUCTES DPRO
				WHERE PRO.Id = DPRO.IdProducte 
					and ((DPRO.Nom <> PRO.Nom) OR (DPRO.Tipologia <> PRO.Tipologia) OR (DPRO.Preu <> PRO.Preu))
				)
				SET ANomPro = NNomPro, ATipo = NTipo, APreu = NPreu;
 		
		-- Afegim els Productes que no existeixen
        INSERT INTO PFCDW.D_PRODUCTES 
				(SELECT ID, Nom, Tipologia, Preu
				FROM PFCBD.PRODUCTES WHERE ID NOT IN (SELECT IdProducte FROM PFCDW.D_PRODUCTES));
    END CARREGA_D_PRODUCTES;
---------------------------------------------------------------------------------	

	
---------------------------------------------------------------------------------
-- Crrega de la dimensi D_ESTATS
---------------------------------------------------------------------------------	
	PROCEDURE CARREGA_D_ESTATS
    AS
    BEGIN
        -- Actualitzem dels Estats que han canviat
        UPDATE (
				SELECT DEST.Nom As ANomEst, EST.descriptiu As NNomEst 
				FROM PFCBD.ESTATS EST, PFCDW.D_ESTATS DEST
				WHERE EST.Id = DEST.IdEstat 
					and (DEST.Nom <> EST.Descriptiu)
				)
				SET ANomEst = NNomEst;
 		
		-- Afegim els Estats que no existeixen
        INSERT INTO PFCDW.D_ESTATS 
				(SELECT ID, descriptiu
				FROM PFCBD.ESTATS E WHERE E.ID NOT IN (SELECT IdEstat FROM PFCDW.D_ESTATS));
    END CARREGA_D_ESTATS;
---------------------------------------------------------------------------------	
	
	
	
---------------------------------------------------------------------------------
-- Crrega de la taula de Fet F_COLLABORACIONS
-- Obs: Donat que les collaboracions poden o no canviar d'estat i no tenen data de caducitat, 
-- s'haur d'esborrar la taula de fet i tornar-la a carregar cada cop.
---------------------------------------------------------------------------------	
	PROCEDURE CARREGA_F_COLLABORACIONS
    AS
    BEGIN
      
		DELETE F_COLLABORACIONS;
        		
		-- Carreguem la taula de fet
		INSERT INTO PFCDW.F_COLLABORACIONS 
				(SElECT JXS.XARXES_SOCIALS_ID, JXS.JUGADORS_ID, JXA.JUGADORS_ID, COL.PRODUCTES_ID, TO_CHAR(COL.DATAINICI, 'YYYYMMDD'), COL.ESTATS_ID, count(*) As NumRegistrades, CEIL(AVG(datafi - datainici)) As PromigDies
				 FROM PFCBD.COLLABORACIONS COL,
					  PFCBD.JUGADORS_XARXES JXS,
					  PFCBD.JUGADORS_XARXES JXA
				 WHERE COL.JUGXAR_SOL_ID = JXS.ID
						AND COL.JUGXAR_AMIC_ID = JXA.ID
			group by JXS.XARXES_SOCIALS_ID, JXS.JUGADORS_ID, JXA.JUGADORS_ID, COL.PRODUCTES_ID, TO_CHAR(COL.DATAINICI, 'YYYYMMDD'), COL.ESTATS_ID);
      
      
    END CARREGA_F_COLLABORACIONS;
---------------------------------------------------------------------------------	
	

---------------------------------------------------------------------------------
-- Crrega de la taula de Fet F_COMPRES
-- Parmetres d'entrada:
--		* p_dataInici: Data d'inici de la finestra de selecci de dades
--		* p_dataFi: Data fi de la finestra de selecci de dades
-- Obs: Si no s'introdueixen parmetres de selecci es carregaran totes les compres des de l'inici de la nostra aplicaci (suposem 01/01/2013) fins al dia d'execuci menys 1.
-- Inicialment es realitzar una crrega total de dades, pero peridicament podran incorporar-se noms les dades noves realitzant crregues delta donat que es pot assegurar
-- que no es realitzaran compres a pasat.
---------------------------------------------------------------------------------		
	PROCEDURE CARREGA_F_COMPRES (p_dataInici IN date, p_dataFi IN date)
    AS
		vDataInici date;
		vDataFi date;
    BEGIN
        -- Inicialitzem les varibles amb la informaci per defecte si no es reben parmetres d'entrada
		if p_dataInici is NULL then
			vDataInici :=  to_date('01/01/2013', 'MM/DD/YYYY');
		else
			vDataInici := p_dataInici;
		end if;
		
		if p_dataFi is NULL then
			vDataFi :=  sysdate - 1;
		else
			vDataFi := p_dataFi;
		end if;
		
        		
		-- Afegim les compres que no existeixen, no es tractaran les compres del dia que ja existeixen a la taula de fet
		INSERT INTO PFCDW.F_COMPRES 
			SELECT IdProducte, IdTemps, IdNivell, IdJugador, NumcompRegistrades, Import
			FROM(   
				SELECT COM.PRODUCTES_ID as IdProducte, TO_CHAR(COM.DATACOMPRA, 'YYYYMMDD') as IdTemps, COM.NIVELLS_ID as IdNivell, COM.JUGADORS_ID as IdJugador, count(*) As NumcompRegistrades, sum(PREU) as Import
				FROM PFCBD.COMPRES COM
				WHERE trunc(COM.datacompra) >= trunc(vDataInici)
				AND trunc(COM.datacompra) <=  trunc(vDataFi)
				GROUP BY COM.PRODUCTES_ID, TO_CHAR(COM.DATACOMPRA, 'YYYYMMDD'), COM.NIVELLS_ID, COM.JUGADORS_ID
				)
			WHERE (IdProducte , IdTemps ,  IdNivell , IdJugador) NOT IN ( SELECT DC.IdProducte , DC.IdTemps,  DC.IdNivell ,  DC.IdJugador from PFCDW.F_COMPRES DC);
    END CARREGA_F_COMPRES;
---------------------------------------------------------------------------------	
	
	
	
---------------------------------------------------------------------------------
-- Crrega de la taula de Fet F_PARTIDES
-- Parmetres d'entrada:
--		* p_dataInici: Data d'inici de la finestra de selecci de dades
--		* p_dataFi: Data fi de la finestra de selecci de dades
-- Obs: Si no s'introdueixen parmetres de selecci es carregaran totes les Partides des de l'inici de la nostra aplicaci (suposem 01/01/2013) fins al dia d'execuci menys 1.
-- Si s'especifiquen dates es realitzar la crrega de dades de partides entre aquestes dates. 
-- Amb aquesta configuraci es pretn poder realitzar una primera crrega inicial i desprs configurar noms la crrega incremental peridicament
---------------------------------------------------------------------------------
   PROCEDURE CARREGA_F_PARTIDES (p_dataInici IN date, p_dataFi IN date)
    AS
		vDataInici date;
		vDataFi date;
    BEGIN
        -- Inicialitzem les varibles amb la informaci per defecte si no es reben parmetres d'entrada
		IF p_dataInici IS NULL then
			vDataInici :=  to_date('01/01/2013', 'MM/DD/YYYY');
		ELSE
			vDataInici := p_dataInici;
		end if;
		
		if p_dataFi IS NULL then
			vDataFi :=  sysdate - 1;
		else
			vDataFi := p_dataFi;
		end if;
		
        		
		-- Afegim les partides que no existeixen, no es tractaran les partides del dia ja existents en el fet
		INSERT INTO PFCDW.F_PARTIDES 
		SELECT TO_CHAR(PAR.DATAPARTIDA, 'YYYYMMDD') as IdTemps, PAR.NIVJUG_NIVELLS_ID as IdNivell, PAR.NIVJUG_JUGADORS_ID as IdJugador, count(*) AS NumPartidesReg, (SELECT count(*)
                                                                                                                                                                          FROM PFCBD.PARTIDES PPAR,
                                                                                                                                                                               PFCBD.NIVELLS_JUGADORS NNJ
                                                                                                                                                                          WHERE NNJ.NIVELLS_ID = PPAR.NIVJUG_NIVELLS_ID
                                                                                                                                                                                AND NNJ.JUGADORS_ID = PPAR.NIVJUG_JUGADORS_ID
                                                                                                                                                                                AND trunc(PPAR.datapartida) >= TRUNC(vDataInici)
                                                                                                                                                                                AND trunc(PPAR.datapartida) <=  TRUNC(vDataFi)
                                                                                                                                                                                and NNJ.datafi < PPAR.datapartida
                                                                                                                                                                                and PPAR.NIVJUG_NIVELLS_ID = PAR.NIVJUG_NIVELLS_ID
                                                                                                                                                                                AND PPAR.NIVJUG_JUGADORS_ID = PAR.NIVJUG_JUGADORS_ID) AS NumAmbNivellSuperat 
                    , AVG(par.VIDESUTIL) as NumVidesUtilitzades, AVG(PAR.VIDESEXT) as NumVidesExtra, AVG(PAR.VIDESCED) As NumVidesCedides, (SELECT count(distinct ANIV.ID)
                                                                                                                                            FROM PFCBD.PARTIDES APAR,
                                                                                                                                                 PFCBD.AJUDES_NIVELL ANIV
                                                                                                                                                  WHERE ANIV.NIVJUG_nivells_ID = APAR.NIVJUG_NIVELLS_ID
                                                                                                                                                  AND ANIV.NIVJUG_JUGADORS_ID = APAR.NIVJUG_JUGADORS_ID
                                                                                                                                                  AND TRUNC(APAR.datapartida) >= TRUNC(vDataInici)
                                                                                                                                                  AND trunc(APAR.datapartida) <=  TRUNC(vDataFi)
                                                                                                                                                  and ANIV.DATAINICI >= APAR.datapartida
                                                                                                                                                  and ANIV.activa = 'Y'
                                                                                                                                                  and APAR.NIVJUG_NIVELLS_ID = PAR.NIVJUG_NIVELLS_ID
                                                                                                                                                  AND APAR.NIVJUG_JUGADORS_ID = PAR.NIVJUG_JUGADORS_ID ) AS NumAjudes
           FROM PFCBD.PARTIDES PAR,
                 PFCBD.NIVELLS_JUGADORS NJ
            WHERE NJ.NIVELLS_ID = PAR.NIVJUG_NIVELLS_ID
                  AND NJ.JUGADORS_ID = PAR.NIVJUG_JUGADORS_ID
                  AND PAR.datapartida >= TRUNC(vDataInici)
                  AND PAR.datapartida <= TRUNC(vDataFi)
                  AND (TO_CHAR(PAR.DATAPARTIDA, 'YYYYMMDD')) NOT IN (SELECT DWC.IDTEMPS FROM PFCDW.F_PARTIDES DWC)
            GROUP BY TO_CHAR(PAR.DATAPARTIDA, 'YYYYMMDD'), PAR.NIVJUG_NIVELLS_ID, PAR.NIVJUG_JUGADORS_ID;
			
			
	END CARREGA_F_PARTIDES;
---------------------------------------------------------------------------------	
			
	
---------------------------------------------------------------------------------
-- Procediment de Crrega
-- Parmetres d'entrada:
--		* p_dataInici: Data d'inici de la finestra de selecci de dades
--		* p_dataFi: Data fi de la finestra de selecci de dades
-- Obs: Realitzar la crrega del magatzem de dades, en funci dels parmetres d'entrada
-- es llanar la crrega total d'informaci, o la crrega incremental limitant les dades
-- a la finestra de temps indicada.
---------------------------------------------------------------------------------	
    PROCEDURE CARREGA (p_dataInici IN date, p_dataFi IN date)
    AS
		vDataInici date;
		vDataFi date;
    BEGIN
    
	 -- Inicialitzem les varibles amb la informaci per defecte si no es reben parmetres
		IF p_dataInici IS NULL then
			vDataInici :=  to_date('01/01/2013', 'MM/DD/YYYY');
		ELSE
			vDataInici := p_dataInici;
		end if;
		
		if p_dataFi IS NULL then
			vDataFi :=  sysdate - 1;
		else
			vDataFi := p_dataFi;
		end if;
		
		
	-- Crida als procediments de crrega de dimensions
			CARREGA_D_JUGADORS;
			CARREGA_D_NIVELLS;
			CARREGA_D_TEMPS;
			CARREGA_D_XARXESSOCIALS;
			CARREGA_D_PRODUCTES;
			CARREGA_D_ESTATS;
   
	-- Crida als procediments de crrega de Fets
			CARREGA_F_COLLABORACIONS;
			CARREGA_F_COMPRES(p_dataInici, p_dataFi);
			CARREGA_F_PARTIDES (p_dataInici, p_dataFi);
			        
    END CARREGA;
---------------------------------------------------------------------------------	

    
END ETL;
/


